'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const core = require('./core-be07326f.js');
const graphql = require('./graphql-9ac85ea8.js');
const rubricUtils = require('./rubric-utils-f266bf0e.js');
const utils = require('./utils-f61bbf7e.js');
const version = require('./version-32e55417.js');

const ErrorMessage = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
    }
    render() {
        return (core.h("div", { class: "errorContainer" }, core.h("div", { class: "errorIcon" }, core.h("svg", { xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 25 25", fill: "none", stroke: "currentColor", "stroke-width": "2", "stroke-linecap": "round", "stroke-linejoin": "round", class: "errorSvg" }, core.h("path", { d: "M10.29 3.86L1.82 18a2 2 0 0 0 1.71 3h16.94a2 2 0 0 0 1.71-3L13.71 3.86a2 2 0 0 0-3.42 0z" }), core.h("line", { x1: "12", y1: "9", x2: "12", y2: "13" }), core.h("line", { x1: "12", y1: "17", x2: "12", y2: "17" }))), core.h("div", { class: "errorMessage" }, this.message)));
    }
    static get style() { return ".errorSvg{height:250px;width:250px;stroke:#d3d3d3}.errorMessage{font-family:sans-serif}.errorContainer{display:-ms-flexbox;display:flex;-ms-flex-direction:column;flex-direction:column;-ms-flex-align:center;align-items:center}"; }
};

const isMethodAndPath = (m) => m && m.path && m.method;
const toMethodAndPath = (m) => {
    if (isMethodAndPath(m)) {
        return m;
    }
    if (typeof m === "string") {
        return { method: "POST", path: m };
    }
    throw new Error(`Unknown type ${m}`);
};
class PlayerApiClient extends graphql.NewRelicEnabledClient {
    constructor(token, endpoints, optimize, retryConfig) {
        super();
        this.token = token;
        this.endpoints = endpoints;
        this.optimize = optimize;
        // Create a FetchUtils instance with tracking info
        this.fetchUtils = new graphql.FetchUtils({
            host: this.endpoints.host,
            optimize: this.optimize,
            hasToken: !!this.token
        }, retryConfig);
    }
    /**
     * Implementation of the abstract method from NewRelicEnabledClient
     */
    getTrackingBaseAttributes() {
        return {
            host: this.endpoints.host,
            optimize: this.optimize,
            hasToken: !!this.token,
            networkInfo: this.getNetworkInfo()
        };
    }
    /**
     * Make an API call with retry mechanism
     *
     * @param endpoint - The API endpoint to call (method and path)
     * @param data - The data to send in the request body (will be JSON stringified)
     * @returns A promise that resolves to the parsed response data
     * @throws Error if all retry attempts fail
     */
    async call(endpoint, data) {
        return this.fetchUtils.callApi(`${this.endpoints.host}${endpoint.path}`, endpoint.method, this.token, data);
    }
    load(itemId, sessionId, env, assignmentId, overrides) {
        const params = {
            itemId,
            sessionId,
            assignmentId,
            env
        };
        if (overrides) {
            params.overrides = overrides;
        }
        return this.call(toMethodAndPath(this.endpoints.load), params);
    }
    saveSession(session, env) {
        return this.call(toMethodAndPath(this.endpoints.saveSession), {
            data: session.data,
            sessionId: session.id,
            env
        });
    }
    model(session, env, overrides) {
        const params = {
            sessionId: session.id,
            data: session.data,
            env
        };
        if (overrides) {
            params.overrides = overrides;
        }
        return this.call(toMethodAndPath(this.endpoints.model), params);
    }
    score(session, env, overrides) {
        const params = {
            sessionId: session.id,
            data: session.data,
            env,
            overrides
        };
        if (overrides) {
            params.overrides = overrides;
        }
        return this.call(toMethodAndPath(this.endpoints.score), params);
    }
}

/**
 * Gets the timestamp of the number of milliseconds that have elapsed since
 * the Unix epoch (1 January 1970 00:00:00 UTC).
 *
 * @static
 * @memberOf _
 * @since 2.4.0
 * @category Date
 * @returns {number} Returns the timestamp.
 * @example
 *
 * _.defer(function(stamp) {
 *   console.log(_.now() - stamp);
 * }, _.now());
 * // => Logs the number of milliseconds it took for the deferred invocation.
 */
var now = function() {
  return rubricUtils.root.Date.now();
};

var now_1 = now;

/** Error message constants. */
var FUNC_ERROR_TEXT = 'Expected a function';

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeMax = Math.max,
    nativeMin = Math.min;

/**
 * Creates a debounced function that delays invoking `func` until after `wait`
 * milliseconds have elapsed since the last time the debounced function was
 * invoked. The debounced function comes with a `cancel` method to cancel
 * delayed `func` invocations and a `flush` method to immediately invoke them.
 * Provide `options` to indicate whether `func` should be invoked on the
 * leading and/or trailing edge of the `wait` timeout. The `func` is invoked
 * with the last arguments provided to the debounced function. Subsequent
 * calls to the debounced function return the result of the last `func`
 * invocation.
 *
 * **Note:** If `leading` and `trailing` options are `true`, `func` is
 * invoked on the trailing edge of the timeout only if the debounced function
 * is invoked more than once during the `wait` timeout.
 *
 * If `wait` is `0` and `leading` is `false`, `func` invocation is deferred
 * until to the next tick, similar to `setTimeout` with a timeout of `0`.
 *
 * See [David Corbacho's article](https://css-tricks.com/debouncing-throttling-explained-examples/)
 * for details over the differences between `_.debounce` and `_.throttle`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Function
 * @param {Function} func The function to debounce.
 * @param {number} [wait=0] The number of milliseconds to delay.
 * @param {Object} [options={}] The options object.
 * @param {boolean} [options.leading=false]
 *  Specify invoking on the leading edge of the timeout.
 * @param {number} [options.maxWait]
 *  The maximum time `func` is allowed to be delayed before it's invoked.
 * @param {boolean} [options.trailing=true]
 *  Specify invoking on the trailing edge of the timeout.
 * @returns {Function} Returns the new debounced function.
 * @example
 *
 * // Avoid costly calculations while the window size is in flux.
 * jQuery(window).on('resize', _.debounce(calculateLayout, 150));
 *
 * // Invoke `sendMail` when clicked, debouncing subsequent calls.
 * jQuery(element).on('click', _.debounce(sendMail, 300, {
 *   'leading': true,
 *   'trailing': false
 * }));
 *
 * // Ensure `batchLog` is invoked once after 1 second of debounced calls.
 * var debounced = _.debounce(batchLog, 250, { 'maxWait': 1000 });
 * var source = new EventSource('/stream');
 * jQuery(source).on('message', debounced);
 *
 * // Cancel the trailing debounced invocation.
 * jQuery(window).on('popstate', debounced.cancel);
 */
function debounce(func, wait, options) {
  var lastArgs,
      lastThis,
      maxWait,
      result,
      timerId,
      lastCallTime,
      lastInvokeTime = 0,
      leading = false,
      maxing = false,
      trailing = true;

  if (typeof func != 'function') {
    throw new TypeError(FUNC_ERROR_TEXT);
  }
  wait = version.toNumber(wait) || 0;
  if (rubricUtils.isObject(options)) {
    leading = !!options.leading;
    maxing = 'maxWait' in options;
    maxWait = maxing ? nativeMax(version.toNumber(options.maxWait) || 0, wait) : maxWait;
    trailing = 'trailing' in options ? !!options.trailing : trailing;
  }

  function invokeFunc(time) {
    var args = lastArgs,
        thisArg = lastThis;

    lastArgs = lastThis = undefined;
    lastInvokeTime = time;
    result = func.apply(thisArg, args);
    return result;
  }

  function leadingEdge(time) {
    // Reset any `maxWait` timer.
    lastInvokeTime = time;
    // Start the timer for the trailing edge.
    timerId = setTimeout(timerExpired, wait);
    // Invoke the leading edge.
    return leading ? invokeFunc(time) : result;
  }

  function remainingWait(time) {
    var timeSinceLastCall = time - lastCallTime,
        timeSinceLastInvoke = time - lastInvokeTime,
        timeWaiting = wait - timeSinceLastCall;

    return maxing
      ? nativeMin(timeWaiting, maxWait - timeSinceLastInvoke)
      : timeWaiting;
  }

  function shouldInvoke(time) {
    var timeSinceLastCall = time - lastCallTime,
        timeSinceLastInvoke = time - lastInvokeTime;

    // Either this is the first call, activity has stopped and we're at the
    // trailing edge, the system time has gone backwards and we're treating
    // it as the trailing edge, or we've hit the `maxWait` limit.
    return (lastCallTime === undefined || (timeSinceLastCall >= wait) ||
      (timeSinceLastCall < 0) || (maxing && timeSinceLastInvoke >= maxWait));
  }

  function timerExpired() {
    var time = now_1();
    if (shouldInvoke(time)) {
      return trailingEdge(time);
    }
    // Restart the timer.
    timerId = setTimeout(timerExpired, remainingWait(time));
  }

  function trailingEdge(time) {
    timerId = undefined;

    // Only invoke if we have `lastArgs` which means `func` has been
    // debounced at least once.
    if (trailing && lastArgs) {
      return invokeFunc(time);
    }
    lastArgs = lastThis = undefined;
    return result;
  }

  function cancel() {
    if (timerId !== undefined) {
      clearTimeout(timerId);
    }
    lastInvokeTime = 0;
    lastArgs = lastCallTime = lastThis = timerId = undefined;
  }

  function flush() {
    return timerId === undefined ? result : trailingEdge(now_1());
  }

  function debounced() {
    var time = now_1(),
        isInvoking = shouldInvoke(time);

    lastArgs = arguments;
    lastThis = this;
    lastCallTime = time;

    if (isInvoking) {
      if (timerId === undefined) {
        return leadingEdge(lastCallTime);
      }
      if (maxing) {
        // Handle invocations in a tight loop.
        clearTimeout(timerId);
        timerId = setTimeout(timerExpired, wait);
        return invokeFunc(lastCallTime);
      }
    }
    if (timerId === undefined) {
      timerId = setTimeout(timerExpired, wait);
    }
    return result;
  }
  debounced.cancel = cancel;
  debounced.flush = flush;
  return debounced;
}

var debounce_1 = debounce;

const PieApiPlayer = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
        this.initialized = false;
        /**
         * Use the beta controllers for pie-elements
         */
        this.optimize = false;
        /**
         * Enable screen recording with New Relic.
         * Note: not to be used in production unless for temporary use to track down a specific user problem.
         * Defaults to false
         */
        this.recordScreen = false;
        /**
         * Enable exception tracking with New Relic.
         */
        this.exceptions = false;
        /**
         * Enable tracking page actions/events with New Relic.
         * Note: Setting this to true increases data sent to New Relic, which may affect costs.
         */
        this.trackPageActions = false;
        /**
         * In evaluate mode, add a bottom border to visually separate each item in the case of a multi-item
         */
        this.showBottomBorder = false;
        /**
         * The mode to be used when player is initialized
         */
        this.initialMode = graphql.Mode.gather;
        /**
         * The role to be used when player is initialized
         */
        this.initialRole = graphql.Role.student;
        /**
         * If the item contains a passage, the player will render it by default.
         * Set this property to false to not render passage.
         */
        this.renderStimulus = true;
        /**
         * It will allow to resize passage and item width for items with stimulus
         */
        this.allowedResize = false;
        /**
         * Choose 'dev' , 'stage' or 'prod' environments for back end.
         */
        this.host = 'prod';
        /**
         * used to automatically re-fetch the bundle (in case we get a 503); defaults to false
         */
        this.reFetchBundle = false;
        /**
         * Customize markup.
         */
        this.customMarkup = '';
        /**
         * If this property is set the item will be rendered in a `preview` state by the player.
         * This means no server-side calls will be made and no reponse-session will be created/stored in pie-api.
         */
        this.preview = false;
        this.recordingStarted = core.createEvent(this, "recordingStarted", 7);
        this.saveSessionError = core.createEvent(this, "saveSessionError", 7);
        this.sessionSaved = core.createEvent(this, "sessionSaved", 7);
        this.modelReload = core.createEvent(this, "model-reload", 7);
        this.playerError = core.createEvent(this, "api-player-error", 7);
        this.sessionCreated = core.createEvent(this, "session-created", 7);
        this.doc = core.getContext(this, "document");
    }
    watchHostEnvironment(newVal) {
        if (newVal) {
            this._apiConfig = this._apiConfig || {};
            this._apiConfig.host = graphql.HOST_DEFAULTS[newVal].apiUrl;
            this._apiConfig.bundleBaseUrl = graphql.HOST_DEFAULTS[newVal].bundleBaseUrl;
        }
    }
    onCustomConfigChange(newValue) {
        console.log('customConfig now', newValue);
    }
    onCustomModelsChange(newValue) {
        console.log('customModels now', newValue);
    }
    onCustomPassageModelsChange(newValue) {
        console.log('customPassageModels now', newValue);
    }
    onCustomMarkupChange(newValue) {
        console.log('customMarkup now', newValue);
    }
    get safeConfig() {
        return Object.assign(Object.assign(Object.assign({}, graphql.PLAYER_HOST_CONFIG), this._apiConfig), this.customConfig);
    }
    /**
     * Get the retry configuration from component properties
     */
    getRetryConfig() {
        const config = {};
        if (this.maxRetryAttempts !== undefined)
            config.maxRetryAttempts = this.maxRetryAttempts;
        if (this.requestTimeoutMs !== undefined)
            config.requestTimeoutMs = this.requestTimeoutMs;
        if (this.initialRetryDelayMs !== undefined)
            config.initialRetryDelayMs = this.initialRetryDelayMs;
        if (this.maxRetryDelayMs !== undefined)
            config.maxRetryDelayMs = this.maxRetryDelayMs;
        if (this.trackPageActions !== undefined)
            config.trackPageActions = this.trackPageActions;
        return Object.keys(config).length > 0 ? config : undefined;
    }
    initPlayerClient() {
        if (this.token) {
            this.playerClient = new PlayerApiClient(this.token, this.safeConfig, this.optimize, this.getRetryConfig());
        }
    }
    async watchToken() {
        try {
            this.initPlayerClient();
            await this.load();
            /**
             * Have to call this cos playerInfo is not part of the component state?
             */
            this.el.forceUpdate();
        }
        catch (err) {
            this.trackError('Error during token update', err, {
                errorContext: 'watchToken',
                hasPlayerClient: !!this.playerClient
            });
            this.playerError.emit({
                message: 'Error during token update',
                details: err.message || err,
            });
        }
    }
    async watchItemId(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.updateItemOrSession();
        }
    }
    async watchRetryConfig() {
        // Reinitialize the player client with the new retry configuration
        if (this.token) {
            this.initPlayerClient();
        }
    }
    async watchEnv(newEnv) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
        const updatedEnv = Object.assign(Object.assign({}, (this.player && this.player.env
            ? this.player.env
            : {})), newEnv);
        if (this.playerInfo && this.player) {
            try {
                if (!this.preview) {
                    const allModels = await this.playerClient.model(this.player.session, updatedEnv, this.getOverrides());
                    const player = this.player;
                    const models = ((_b = (_a = player) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.models) || ((_e = (_d = (_c = player) === null || _c === void 0 ? void 0 : _c.config) === null || _d === void 0 ? void 0 : _d.pie) === null || _e === void 0 ? void 0 : _e.models) || [];
                    const passageModels = ((_h = (_g = (_f = player) === null || _f === void 0 ? void 0 : _f.config) === null || _g === void 0 ? void 0 : _g.passage) === null || _h === void 0 ? void 0 : _h.models) || [];
                    // search model based on id and element
                    const findModel = (models, model) => { var _a; return (_a = models) === null || _a === void 0 ? void 0 : _a.find((m) => m.id === model.id && m.element === model.element); };
                    // filter models from allModels
                    const updatedModels = allModels.filter((model) => findModel(models, model));
                    // filter passage models from allModels
                    const updatedPassageModels = allModels.filter((model) => findModel(passageModels, model));
                    const config = this.getConfigParsed(player.config, updatedModels, updatedPassageModels);
                    player.config = config;
                }
                this.player.env = updatedEnv;
                return { error: null };
            }
            catch (e) {
                this.trackError('Environment update error', e, {
                    errorContext: 'watchEnv',
                    originalEnv: JSON.stringify(this.env || {}),
                    newEnv: JSON.stringify(newEnv || {}),
                    updatedEnv: JSON.stringify(updatedEnv || {}),
                    hasPlayerInfo: !!this.playerInfo,
                    hasPlayerConfig: !!((_j = this.player) === null || _j === void 0 ? void 0 : _j.config),
                    modelCount: ((_m = (_l = (_k = this.player) === null || _k === void 0 ? void 0 : _k.config) === null || _l === void 0 ? void 0 : _l.models) === null || _m === void 0 ? void 0 : _m.length) || ((_r = (_q = (_p = (_o = this.player) === null || _o === void 0 ? void 0 : _o.config) === null || _p === void 0 ? void 0 : _p.pie) === null || _q === void 0 ? void 0 : _q.models) === null || _r === void 0 ? void 0 : _r.length) || 0
                });
                this.playerError.emit({
                    message: 'Error updating environment',
                    details: e.message || e,
                });
                return { error: e.message };
            }
        }
    }
    watchRecordScreen(newVal, oldVal) {
        if (newVal && newVal !== oldVal) {
            if (this.newRelic && this.recordScreenIdentity) {
                // Set user identity for New Relic session
                this.newRelic.setUserId(this.recordScreenIdentity);
                // Record session start
                if (this.trackPageActions) {
                    this.newRelic.addPageAction('session_recording_started', {
                        appVersion: graphql.VERSION,
                        itemId: this.itemId,
                        sessionId: this.sessionId
                    });
                }
                this.recordingStarted.emit('Session recording started with New Relic');
            }
        }
    }
    /**
     * Return the evaluated score for the current response.
     * The api token must provide permission to authorize the use of this method.
     *
     * Note: Score does not (yet) work when in preview mode - will resolve to null if `preview` is `true`
     *
     * @param disablePartialScoring - if true all interactions will be scored as either correct or incorrect
     */
    async score(disablePartialScoring) {
        if (disablePartialScoring === true || disablePartialScoring === false) {
            console.warn('disablePartialScoring is no longer used - set env instead: {partialScoring: true|false, ...}');
        }
        if (this.preview) {
            return Promise.resolve(null);
        }
        else {
            return this.playerClient.score(this.playerInfo.session, this.env, this.getOverrides());
        }
    }
    async saveSession() {
        var _a, _b, _c, _d;
        if (this.player && this.player.session && !this.preview) {
            try {
                await this.playerClient.saveSession(this.player.session, this.env || {});
                // Track successful session save in New Relic
                if (this.newRelic && this.trackPageActions) {
                    this.newRelic.addPageAction('session_saved', {
                        sessionId: this.sessionId
                    });
                }
                this.sessionSaved.emit(this.sessionId);
            }
            catch (error) {
                console.warn('could not save session');
                this.saveSessionError.emit(error);
                this.trackError('Error saving session', error, {
                    errorContext: 'session-changed event',
                    envMode: ((_a = this.env) === null || _a === void 0 ? void 0 : _a.mode) || this.initialMode,
                    envRole: ((_b = this.env) === null || _b === void 0 ? void 0 : _b.role) || this.initialRole,
                    sessionData: JSON.stringify(((_c = this.player.session) === null || _c === void 0 ? void 0 : _c.data) || []),
                    hasSessionId: !!this.sessionId,
                    sessionPath: ((_d = this.player.session) === null || _d === void 0 ? void 0 : _d.id) ? 'session.id exists' : 'no session.id'
                });
                this.playerError.emit({
                    message: 'Error saving session',
                    details: error.message || error,
                });
            }
        }
    }
    /**
     * Listens for session changed events and saves updated data for session
     */
    handleSessionChange() {
        this.debouncedSaveSession();
    }
    handlePlayerError(ev) {
        this.playerInfo = null;
        this.errorMessage = ev.detail;
        // Collect as much context as possible for player errors
        const errorContext = {
            errorContext: 'player-error event',
            errorPayload: typeof ev.detail === 'object' ? JSON.stringify(ev.detail) : String(ev.detail),
            playerState: this.player ? 'initialized' : 'not initialized',
            playerConfig: this.player && this.player.config ? 'has config' : 'no config',
            currentRoute: typeof window !== 'undefined' ? window.location.pathname : 'unknown'
        };
        this.trackError('Player error', ev.detail || new Error('Unknown player error'), errorContext);
    }
    /**
     * Helper method to track errors with New Relic
     * This is a utility method that centralizes our error reporting logic and ensures
     * we capture comprehensive error information including stack traces.
     */
    trackError(message, error, customAttributes = {}) {
        var _a, _b, _c;
        // First check if newRelic exists to avoid null reference
        if (!this.newRelic) {
            console.error(`[pie-api-player] ${message}:`, error);
            return;
        }
        const shouldReportToNewRelic = this.newRelic && (_a = this.exceptions, (_a !== null && _a !== void 0 ? _a : false));
        if (shouldReportToNewRelic) {
            // Ensure we have an actual Error object with stack trace
            const errorObj = error instanceof Error ? error : new Error(typeof error === 'string' ? error : JSON.stringify(error));
            // Extract stack trace if available
            const stackTrace = errorObj.stack || '';
            // Add default attributes with comprehensive context
            const attributes = Object.assign(Object.assign({}, customAttributes), { itemId: this.itemId, sessionId: this.sessionId, message, appVersion: graphql.VERSION, errorMessage: errorObj.message || String(error), stackTrace, componentState: {
                    mode: ((_b = this.env) === null || _b === void 0 ? void 0 : _b.mode) || this.initialMode,
                    role: ((_c = this.env) === null || _c === void 0 ? void 0 : _c.role) || this.initialRole,
                    preview: this.preview
                }, time: new Date().toISOString() });
            // Use New Relic's noticeError API to report the error
            this.newRelic.noticeError(errorObj, attributes);
        }
        // Always log to console for debugging
        console.error(`[pie-api-player] ${message}:`, error, customAttributes);
    }
    async updateItemOrSession() {
        try {
            this.playerInfo = null;
            if (!this.preview) {
                await this.load();
            }
            this.el.forceUpdate();
        }
        catch (err) {
            this.trackError('Error updating item or session', err, {
                errorContext: 'updateItemOrSession',
                itemId: this.itemId,
                sessionId: this.sessionId,
                preview: this.preview,
                updateTriggeredBy: 'property change'
            });
            this.playerError.emit({
                message: 'Error updating item or session',
                details: err.message || err,
            });
        }
    }
    readyToLoad() {
        return (this.itemId || this.sessionId) && this.playerClient && this.token;
    }
    getOverrides() {
        let overrides = undefined;
        try {
            if (typeof this.overrides === 'string') {
                overrides = JSON.parse(this.overrides);
            }
            else {
                if (this.overrides) {
                    overrides = this.overrides;
                }
            }
        }
        catch (e) {
        }
        return overrides;
    }
    async load() {
        var _a, _b;
        try {
            if (this.preview) {
                this.env
                    ? (this.env.role = graphql.Role.instructor)
                    : (this.env = { role: graphql.Role.instructor, mode: graphql.Mode.view });
                const buildPlayerConfig = (response) => {
                    var _a;
                    // TODO pie-api-aws 10. test this once passages are tested
                    const passage = (_a = response) === null || _a === void 0 ? void 0 : _a.passage;
                    const out = {
                        session: { id: '', data: [] },
                    };
                    if (passage) {
                        return Object.assign(Object.assign({}, out), { item: { pie: response.config, passage: passage.config } });
                    }
                    else {
                        return Object.assign(Object.assign({}, out), { item: response.config });
                    }
                };
                if (this.itemId && this.token) {
                    const contentData = await graphql.loadFromPieApi(this.token, this.itemId, this.safeConfig.host, this.getOverrides());
                    const info = buildPlayerConfig(contentData);
                    console.log(info);
                    console.log(this);
                    this.modelReload.emit(info);
                    this.playerInfo = info;
                }
                else {
                    this.errorMessage = 'no item id passed to player';
                }
            }
            else {
                if (!this.playerInfo && this.readyToLoad()) {
                    // Track load start in New Relic
                    if (this.newRelic && this.trackPageActions) {
                        this.newRelic.addPageAction('item_load_started', {
                            itemId: this.itemId,
                            sessionId: this.sessionId
                        });
                    }
                    this.playerInfo = await this.playerClient.load(this.itemId, this.sessionId, Object.assign(Object.assign({}, this.env), { mode: this.env && this.env.mode ? this.env.mode : this.initialMode, role: this.env && this.env.role ? this.env.role : this.initialRole }), this.assignmentId, this.getOverrides());
                    // Track successful load in New Relic
                    if (this.newRelic && this.trackPageActions) {
                        this.newRelic.addPageAction('item_loaded', {
                            itemId: this.itemId,
                            sessionId: (_a = this.playerInfo.session) === null || _a === void 0 ? void 0 : _a.id
                        });
                    }
                    let apiInfo = this.playerInfo.item;
                    // is passage model
                    if (apiInfo && apiInfo.pie) {
                        if (this.playerInfo.js && this.playerInfo.js.view) {
                            // deprecated, pie-api should return `bundle.url` prop
                            // we need to create 2 different bundles because for stimulus items we render two pie-player instances PD-4664
                            const [bundle1, bundle2] = graphql.splitStimulusItemsBundle(this.playerInfo.js.view);
                            apiInfo.pie.bundle = { url: bundle1 };
                            apiInfo.passage.bundle = { url: bundle2 };
                        }
                    }
                    else {
                        if (this.playerInfo.js && this.playerInfo.js.view) {
                            apiInfo.bundle = { url: this.playerInfo.js.view };
                        }
                    }
                    this.playerInfo.item = apiInfo;
                    if (!this.sessionId &&
                        this.playerInfo.session &&
                        this.playerInfo.session.id) {
                        this.sessionCreated.emit(this.playerInfo.session);
                    }
                }
            }
        }
        catch (err) {
            this.errorMessage = 'error loading item';
            console.error(err);
            this.trackError('Error loading item', err, {
                errorContext: 'load method',
                itemId: this.itemId,
                sessionId: this.sessionId,
                isPreview: this.preview,
                hasToken: !!this.token,
                hasApiConfig: !!this._apiConfig,
                apiHost: (_b = this.safeConfig) === null || _b === void 0 ? void 0 : _b.host,
                loadState: this.playerInfo ? 'has playerInfo' : 'no playerInfo',
                readyToLoad: this.readyToLoad(),
                initialMode: this.initialMode,
                initialRole: this.initialRole,
                hasAssignmentId: !!this.assignmentId,
                hasOverrides: !!this.getOverrides()
            });
            this.playerError.emit({
                message: 'Error loading item',
                details: err.message || err,
            });
        }
    }
    async componentWillLoad() {
        this.debouncedSaveSession = debounce_1(this.saveSession.bind(this), 100);
        this.newRelic = new graphql.NewRelicInstrumentationService(this.doc);
        this.newRelic.setApplicationVersion(`pie-api-components@${graphql.VERSION}`);
        this.newRelic.setCustomAttribute('componentType', 'pie-api-player');
        if (this.newRelic && this.trackPageActions) {
            this.newRelic.addPageAction('component_loaded', {
                itemId: this.itemId,
                sessionId: this.sessionId
            });
        }
        if (this.recordScreen) {
            this.watchRecordScreen(this.recordScreen, false);
        }
        this.watchHostEnvironment(this.host);
        this.initPlayerClient();
        if (this.readyToLoad()) {
            await this.load();
        }
    }
    disconnectedCallback() {
        // Record component unload if tracking is enabled
        if (this.newRelic && this.trackPageActions) {
            this.newRelic.addPageAction('component_unloaded', {
                itemId: this.itemId,
                sessionId: this.sessionId
            });
        }
    }
    overrideModels(models, customModels) {
        var _a;
        if (!((_a = customModels) === null || _a === void 0 ? void 0 : _a.length)) {
            return models || [];
        }
        return (models || []).map((model) => (Object.assign(Object.assign({}, model), (customModels.find((cm) => cm.id === model.id) || {}))));
    }
    getConfigParsed(item, models, passageModels) {
        var _a, _b;
        // Check if 'item' has the 'pie' structure
        if (item && item.pie) {
            // Structure with 'pie'
            return Object.assign(Object.assign(Object.assign({}, item), { pie: Object.assign(Object.assign({}, item.pie), { models: this.overrideModels(models || item.pie.models, this.customModels), markup: this.customMarkup || item.pie.markup }) }), (item.passage
                ? {
                    passage: Object.assign(Object.assign({}, item.passage), { models: this.overrideModels(passageModels || item.passage.models, this.customPassageModels) }),
                }
                : {}));
        }
        // Standard structure
        return Object.assign(Object.assign({}, item), { models: this.overrideModels(models || ((_a = item) === null || _a === void 0 ? void 0 : _a.models), this.customModels), markup: this.customMarkup || ((_b = item) === null || _b === void 0 ? void 0 : _b.markup) });
    }
    render() {
        const { item } = this.playerInfo || {};
        const configParsed = this.getConfigParsed(item);
        if (this.playerInfo && this.playerInfo.item) {
            return (core.h("pie-player", { class: `pie-api-player ${this.customClassname || ''}`.trim(), hosted: !this.preview, bundleHost: this.host, addCorrectResponse: this.preview, config: configParsed, session: this.playerInfo.session, renderStimulus: this.renderStimulus, allowedResize: this.allowedResize, showBottomBorder: this.showBottomBorder, env: {
                    mode: this.env && this.env.mode ? this.env.mode : this.initialMode,
                    role: this.env && this.env.role ? this.env.role : this.initialRole,
                }, ref: (el) => (this.player = el), bundleEndpoints: this.bundleEndpoints, reFetchBundle: this.reFetchBundle, externalStyleUrls: this.externalStyleUrls, customClassname: this.customClassname }, ' '));
        }
        else {
            if (this.errorMessage) {
                return core.h("error-message", { message: this.errorMessage });
            }
            else {
                return core.h("div", null, "not yet loaded");
            }
        }
    }
    get el() { return core.getElement(this); }
    static get watchers() { return {
        "host": ["watchHostEnvironment"],
        "customConfig": ["onCustomConfigChange"],
        "customModels": ["onCustomModelsChange"],
        "customPassageModels": ["onCustomPassageModelsChange"],
        "customMarkup": ["onCustomMarkupChange"],
        "token": ["watchToken"],
        "sessionId": ["watchItemId"],
        "itemId": ["watchItemId"],
        "maxRetryAttempts": ["watchRetryConfig"],
        "requestTimeoutMs": ["watchRetryConfig"],
        "initialRetryDelayMs": ["watchRetryConfig"],
        "maxRetryDelayMs": ["watchRetryConfig"],
        "trackPageActions": ["watchRetryConfig"],
        "env": ["watchEnv"],
        "recordScreen": ["watchRecordScreen"]
    }; }
    static get style() { return ".pie-api-player img{max-width:inherit;max-height:inherit}.book-title-k5{text-decoration:underline}.book-title{font-style:italic}.block-quote,.text-block{font-family:serif;margin-left:20%;margin-right:20%;margin-top:5px;margin-bottom:5px;width:60%;border:1px solid #000;padding:2px;display:block}.short-quote{font-weight:700}span.short-quote:before{content:open-quote}span.short-quote:after{content:close-quote}.word-callout{text-decoration:underline}.relative-emphasis{text-transform:uppercase}.content-emphasis{font-weight:700;color:red}span.passage-title:before{content:open-quote}span.passage-title:after{content:close-quote}.proper-name,.variable{font-style:italic}.equation-block{text-align:center;margin-left:20%;margin-right:20%;margin-top:5px;margin-bottom:5px;width:60%;padding:2px;display:block}.embedded-error{font-style:italic}div.passage-subtitle,div.passage-title{font-weight:700;text-align:center}div.passage-author{text-align:center}.p-number{float:left;font-size:.85em}.numbered-paragraph{margin-left:36px}\@media print{.kds-noprint,.noprint{display:none}}.abs,.kds-absolute{border-left:1px solid #000;border-right:1px solid #000;padding:0 .3em 0 .3em}.kds-border-1{border:1px solid #000}.kds-border-2{border:2px solid #000}.center,.kds-center{margin:0;text-align:center}.kds-exponent{font-size:85%}.kds-exponent,sup.frac{margin-left:-.4em;position:relative;top:-1.4em}sup.frac{font-size:70%}.kds-fillin{display:inline-block}.fillin,.kds-fillin{border:1px solid #000;margin:0 .2em;padding-left:1em}.fillin{display:inline-block;*display:inline}.kds-font-larger{font-size:larger}.kds-font-smaller{font-size:smaller}table.kds-fraction{border:0;display:inline-block;font-size:85%;margin:0;padding:0;vertical-align:middle}table.kds-fraction>tbody>tr>td.kds-numerator{border-bottom:1px solid #000;border-left:0;border-right:0;border-top:0;text-align:center}table.kds-fraction>tbody>tr>td.kds-denominator{border:0;text-align:center}table.frac{display:inline-block;*display:inline;vertical-align:middle;font-size:85%;border:0;margin:0;padding:0}.frac .nu{border-left:0;border-top:0;border-right:0;border-bottom:1px solid #000;text-align:center}.frac .de{border:0;text-align:center}.kds-hanging-indent{margin:0;padding-left:3em;text-indent:-3em}.kds-image-caption{font-family:Verdana,Arial,sans-serif;text-align:center}.kds-image-left{float:left;margin-right:5px}.kds-image-right{float:right;margin-left:5px}.indent,.kds-indent{text-indent:3em}.kds-nowrap{display:inline-block;white-space:nowrap}.kds-overline{text-decoration:overline}.kds-parentheses{font-family:Arial Narrow,Arial,sans-serif}.kds-parentheses,b.frac{font-size:300%;font-weight:400;position:relative;top:8px}b.frac{font-family:arial narrow}.kds-pi{font-family:Times New Roman,serif;font-size:1.2em}.newradical{border-collapse:collapse;display:inline-block;*display:inline;vertical-align:middle}.newradical td{border:0;padding:0;vertical-align:top}.newradical td.vinculum{border-top:1px solid #000;padding:0;vertical-align:top}.newradical div.exp{font-size:.7em;position:relative;left:7px;bottom:5px}.kds-underline,.under{text-decoration:underline}.kds-verdana2t,.Verdana2t{border:1px solid #fff;text-align:left;text-decoration:none;vertical-align:top}.kds-whole-number{display:inline-block}.kds-whole-number,.whole{font-size:120%;margin:0 0 0 .2em}.whole{display:inline-block;*display:inline}table.KdsTable01{border:1px solid #000;border-collapse:collapse;border-spacing:0}table.KdsTable01>tbody>tr>th{font-weight:400}table.KdsTable01>tbody>tr>th,table.KdsTable01>tbody>tr>th.bold{background-color:#d3d3d3;border:1px solid #000;padding:2px;text-align:center}table.KdsTable01>tbody>tr>th.bold{font-weight:700}table.KdsTable01>tbody>tr>td,table.KdsTable01>tbody>tr>td.bold{border:1px solid #000;padding:2px;text-align:center}table.KdsTable01>tbody>tr>td.bold{font-weight:700}table.KdsTable02{border:1px solid #000;border-collapse:collapse;border-spacing:0}table.KdsTable02>tbody>tr>th{background-color:#d3d3d3;font-weight:400}table.KdsTable02>tbody>tr>td,table.KdsTable02>tbody>tr>th{border:1px solid #000;padding:2px}.lrn_feature h3{margin-top:0!important}#item,#stimulus{width:50%;float:left}.table{width:100%;margin-bottom:1em;background-color:transparent}th{text-align:inherit}.table td,.table th{padding:.75em;vertical-align:top;border-top:1px solid #dee2e6}.table thead th{vertical-align:bottom;border-bottom:2px solid #dee2e6}.table-bordered,.table-bordered td,.table-bordered th{border:1px solid #dee2e6}.table-bordered thead td,.table-bordered thead th{border-bottom-width:2px}"; }
};

var lib = utils.createCommonjsModule(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });
class ModelSetEvent extends CustomEvent {
    constructor(component, complete, hasModel) {
        super(ModelSetEvent.TYPE, { bubbles: true, composed: true, detail: { complete, component, hasModel } });
        this.component = component;
        this.complete = complete;
    }
}
ModelSetEvent.TYPE = 'model-set';
exports.ModelSetEvent = ModelSetEvent;
class SessionChangedEvent extends CustomEvent {
    constructor(component, complete) {
        super(SessionChangedEvent.TYPE, { bubbles: true, composed: true, detail: { complete, component } });
        this.component = component;
        this.complete = complete;
    }
}
SessionChangedEvent.TYPE = 'session-changed';
exports.SessionChangedEvent = SessionChangedEvent;

});

utils.unwrapExports(lib);
var lib_2 = lib.SessionChangedEvent;

const controllerErrorMessage = "Error processing question configuration, verify the question model?";
const Player = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
        /**
         * Allows disabling of the default behaviour which is to look up and load the JS bundle that define the Custom Elements
         * used by the item config.
         * This if for advanced use cases when using the pie-player in a container that is managing loading of Custom Elements and Controllers.
         */
        this.disableBundler = false;
        this.elementsLoaded = false;
        /**
         * Simulates a correct response for the item.
         * This property will only have this effect if the `hosted` property is
         * false and player is running client-side-only.
         */
        this.addCorrectResponse = false;
        /**
         * In evaluate mode, add a bottom border to visually separate each item in the case of a multi-item
         */
        this.showBottomBorder = false;
        /**
         * used to automatically re-fetch the bundle (in case we get a 503)
         */
        this.reFetchBundle = false;
        /**
         * The Pie Session
         */
        this.session = { id: "", data: [] };
        /**
         * Describes runtime environment for the player.
         *
         */
        this.env = { mode: "gather", role: "student" };
        /**
         * Indicates if player running in the context of a PIE hosting system.
         * Do not modify the default value for this property if you are not implementing a PIE host.
         * If true, the host is responsible for all model updates.
         */
        this.hosted = false;
        /**
         * If the item contains a stimulus, the player will render it by default.
         * Set this property to false to not render stimulus.
         */
        this.renderStimulus = true;
        this.version = version.VERSION;
        /**
         * Allow to resize pie-stimulus layout
         * Set this property to false to not render the resizer.
         */
        this.allowedResize = false;
        this.pieLoader = new version.PieLoader(null, this.loaderConfig);
        this.sessionChanged = core.createEvent(this, "session-changed", 7);
        this.playerError = core.createEvent(this, "player-error", 7);
        this.responseCompleted = core.createEvent(this, "responseCompleted", 7);
        this.loadComplete = core.createEvent(this, "load-complete", 7);
        this.doc = core.getContext(this, "document");
    }
    watchAddCorrectResponse(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.updateModels();
        }
    }
    player() {
        return this.stimulusPlayer ? this.stimulusPlayer : this;
    }
    async watchConfig(newConfig) {
        this.elementsLoaded = false;
        try {
            if (!newConfig) {
                return;
            }
            try {
                if (typeof newConfig == "string") {
                    newConfig = JSON.parse(newConfig);
                }
                if (newConfig.pie) {
                    this.stimulusItemModel = newConfig;
                    return; // if stimulus item
                }
                else if (newConfig.elements) {
                    this.pieContentModel = rubricUtils.addRubric(newConfig);
                    this.pieContentModel = utils.normalizeContentElements(this.pieContentModel);
                }
                else {
                    this.playerError.emit(`invalid pie data model`);
                    return;
                }
            }
            catch (err) {
                this.playerError.emit(`exception processing content model - ${err.message}`);
                return;
            }
            if (!this.elementsLoaded && !this.disableBundler) {
                let endpoints = version.DEFAULT_ENDPOINTS.prod;
                if (this.bundleHost &&
                    ["dev", "stage", "prod"].includes(this.bundleHost)) {
                    endpoints = version.DEFAULT_ENDPOINTS[this.bundleHost];
                }
                // if bundleEndpoints are provided, use them instead of the default
                // used for proxies, etc.
                let forceBundleUrl = false;
                if (this.bundleEndpoints) {
                    endpoints = this.bundleEndpoints;
                    forceBundleUrl = true;
                }
                await this.pieLoader.loadCloudPies({
                    content: this.pieContentModel,
                    doc: this.doc,
                    endpoints: endpoints,
                    bundle: this.hosted ? version.BundleType.player : version.BundleType.clientPlayer,
                    useCdn: false,
                    forceBundleUrl,
                    reFetchBundle: this.reFetchBundle
                });
            }
        }
        catch (err) {
            this.playerError.emit(`problem loading item (${err})`);
        }
    }
    /**
     * For previewing changes to an item. Updates the model for one question in the item model.
     * @param update the updated model
     */
    async updateElementModel(update) {
        if (this.pieContentModel && this.pieContentModel.models) {
            const index = this.pieContentModel.models.findIndex(m => m.id === update.id);
            if (index !== -1) {
                this.pieContentModel.models.splice(index, 1, update);
            }
            await this.updateModels();
        }
    }
    async provideScore() {
        // TODO I think we have to check if it's hosted or not to expose this function
        if (!this.pieContentModel || !this.pieContentModel.models) {
            console.error("No pie content model");
            return false;
        }
        return Promise.all((this.pieContentModel.models || []).map(async (model) => {
            let pieEl = this.el.querySelector(`[id='${model.id}']`);
            !pieEl && (pieEl = this.el.querySelector(`[pie-id='${model.id}']`));
            const session = this.findOrAddSession(this.session.data, model.id);
            if (pieEl) {
                const controller = this.pieLoader.getController(pieEl.localName);
                if (controller && controller.outcome) {
                    return Object.assign(Object.assign({}, session), (await controller.outcome(model, session, {
                        mode: "evaluate"
                    })));
                }
            }
        }));
    }
    updateModels(newEnv = this.env) {
        // wrapping a player in stimulus layout
        if (this.stimulusPlayer) {
            this.stimulusPlayer.env = newEnv;
            return;
        }
        performance.mark("pie-model-set-start");
        if (this.pieContentModel &&
            this.pieContentModel.models &&
            typeof this.pieContentModel.models.forEach === "function" &&
            this.pieContentModel.markup &&
            this.elementsLoaded) {
            /**
             * Block session changed events while we set model/session on the elements.
             * TODO: The elements should *not* be firing 'session-changed' when the session is set.
             * They should only fire this if a user has made a change. Can we guarantee that?
             */
            this.el.addEventListener(lib_2.TYPE, this.stopEventFromPropagating);
            this.pieContentModel.models.forEach(async (model, index) => {
                if (model && model.error) {
                    this.playerError.emit(`error loading question data`);
                    throw new Error(model.error);
                }
                const pieEl = this.el.querySelector(`[id='${model.id}']`);
                let session = this.findOrAddSession(this.session.data, model.id);
                if (pieEl) {
                    if (!this.hosted) {
                        try {
                            // use local controllers
                            const controller = this.pieLoader.getController(pieEl.localName);
                            if (controller) {
                                if (this.addCorrectResponse &&
                                    controller.createCorrectResponseSession &&
                                    typeof controller.createCorrectResponseSession === "function") {
                                    session = await controller.createCorrectResponseSession(model, Object.assign(Object.assign({}, newEnv), { role: "instructor" }));
                                }
                                pieEl.model = await controller.model(model, session, newEnv);
                            }
                            else {
                                // no controller provided
                                pieEl.model = model;
                            }
                        }
                        catch (err) {
                            this.playerError.emit(`${controllerErrorMessage}  -  (${err})`);
                        }
                    }
                    else {
                        if (model.error) {
                            this.playerError.emit(`${controllerErrorMessage}  -  '${model.error}'`);
                        }
                        pieEl.model = model;
                        if (this.addCorrectResponse) {
                            this.playerError.emit(`add-correct-response cannot be used in hosted environement`);
                        }
                    }
                    try {
                        pieEl.session = session;
                    }
                    catch (err) {
                        this.playerError.emit(`error setting item session value - ${err.message}`);
                    }
                }
            });
            performance.mark("pie-model-set-end");
            performance.measure("PIE Model Setup Time", "pie-model-set-start", "pie-model-set-end");
            const modelEntry = performance.getEntriesByName("PIE Model Setup Time")[0];
            const modelDuration = modelEntry ? modelEntry.duration : 0;
            console.log("[PIE Model Setup Time]", modelDuration.toFixed(2), "ms");
            setTimeout(() => {
                /** remove the event blocker - see above */
                this.el.removeEventListener(lib_2.TYPE, this.stopEventFromPropagating);
                //TODO: is this needed anymore? maybe it should be using 'model-set' instead of 'session-changed' which is what it was using.
                this.loadComplete.emit();
            }, 150);
        }
    }
    stopEventFromPropagating(e) {
        e.stopPropagation();
    }
    scopeCSS(cssText, customClassname) {
        return cssText.replace(/(^|\})\s*([^{]+)\s*\{/g, (_, close, selector) => {
            const scoped = selector
                .split(",")
                .map((s) => `pie-player.${customClassname} ${s.trim()}`)
                .join(", ");
            return `${close} ${scoped} {`;
        });
    }
    async loadScopedExternalStyle(url, customClassname) {
        const startTime = performance.now();
        console.log(`[style-loader] Fetching CSS: ${url}`);
        const res = await fetch(url);
        const css = await res.text();
        const fetchEnd = performance.now();
        console.log(`[style-loader] Fetch complete in ${(fetchEnd - startTime).toFixed(2)}ms`);
        const scopeStart = performance.now();
        const scopedCSS = this.scopeCSS(css, customClassname);
        const scopeEnd = performance.now();
        console.log(`[style-loader] CSS scoped in ${(scopeEnd - scopeStart).toFixed(2)}ms`);
        const styleTag = document.createElement("style");
        styleTag.setAttribute("data-pie-style", url);
        styleTag.textContent = scopedCSS;
        document.head.appendChild(styleTag);
    }
    async componentWillLoad() {
        if (this.config) {
            this.watchConfig(this.config);
        }
    }
    findOrAddSession(data, id) {
        const s = data.find(d => d.id === id);
        if (s) {
            return s;
        }
        const ss = { id };
        data.push(ss);
        return ss;
    }
    renderMath() {
        version.mathRenderingAccessible$1.renderMath(this.el);
    }
    addBottomBorder(tags) {
        if (!Array.isArray(tags)) {
            return;
        }
        tags.forEach(tag => {
            const elems = document.querySelectorAll(`${tag}`);
            for (const elem of elems) {
                if (elem && elem instanceof HTMLElement) {
                    elem.classList.add("evaluate-bottom-border");
                }
            }
        });
    }
    async afterRender() {
        if (this.pieContentModel && this.pieContentModel.markup) {
            if (this.elementsLoaded) {
                this.updateModels();
                this.renderMath();
                if (this.showBottomBorder && this.env.mode === "evaluate") {
                    const pieTags = this.pieContentModel.elements &&
                        Object.keys(this.pieContentModel.elements);
                    this.addBottomBorder(pieTags);
                }
            }
            else {
                const elements = Object.keys(this.pieContentModel.elements).map(el => ({
                    name: el,
                    tag: el
                }));
                // Note: hard to verify but it appears that we need to resolve
                // the value first rather than setting the promise directly on
                // this state property - otherwise lifecycle re-render is triggered too early
                const loadedInfo = await this.pieLoader.elementsHaveLoaded(elements);
                if (loadedInfo.val &&
                    !!loadedInfo.elements.find(el => this.pieContentModel.elements[el.name])) {
                    this.elementsLoaded = true;
                    if (performance.getEntriesByName("pie-load-end").length > 0) {
                        performance.mark("pie-elements-load-end");
                        performance.measure("PIE Elements Load Time", "pie-load-end", "pie-elements-load-end");
                        const renderEntry = performance.getEntriesByName("PIE Elements Load Time")[0];
                        const renderDuration = renderEntry ? renderEntry.duration : 0;
                        console.log("[PIE Elements Load + Render Time]", renderDuration.toFixed(2), "ms");
                    }
                }
            }
        }
        if (this.externalStyleUrls &&
            typeof this.externalStyleUrls === "string" &&
            this.customClassname &&
            typeof this.customClassname === "string") {
            const urls = this.externalStyleUrls.split(",").map(url => url.trim());
            urls.forEach(url => {
                if (url &&
                    typeof url === "string" &&
                    !document.querySelector(`style[data-pie-style="${url}"]`)) {
                    this.loadScopedExternalStyle(url, this.customClassname);
                }
            });
        }
    }
    async componentDidLoad() {
        await this.afterRender();
    }
    async componentDidUpdate() {
        await this.afterRender();
    }
    render() {
        if (this.stimulusItemModel) {
            return this.renderStimulus ? (core.h("pie-stimulus-layout", { allowedResize: this.allowedResize }, core.h("div", { slot: "stimulus", class: "player-stimulus-container" }, core.h("pie-player", { id: "stimulusPlayer", config: this.stimulusItemModel.passage, env: this.env, hosted: this.hosted, session: this.session, ref: el => (this.stimulusPlayer = el), bundleHost: this.bundleHost, bundleEndpoints: this.bundleEndpoints, class: this.customClassname })), core.h("div", { slot: "item", class: "player-item-container" }, core.h("pie-player", { id: "itemPlayer", addCorrectResponse: this.addCorrectResponse, config: this.stimulusItemModel.pie, env: this.env, hosted: this.hosted, session: this.session, bundleHost: this.bundleHost, bundleEndpoints: this.bundleEndpoints, class: this.customClassname })))) : (core.h("pie-player", { id: "itemPlayer", addCorrectResponse: this.addCorrectResponse, config: this.stimulusItemModel.pie, env: this.env, hosted: this.hosted, session: this.session, bundleEndpoints: this.bundleEndpoints, class: this.customClassname }));
        }
        else {
            if (this.elementsLoaded) {
                return (core.h("div", { class: "player-container", innerHTML: this.pieContentModel && this.pieContentModel.markup
                        ? this.pieContentModel.markup
                        : "" }));
            }
            return core.h("pie-spinner", null);
        }
    }
    get el() { return core.getElement(this); }
    static get watchers() { return {
        "addCorrectResponse": ["watchAddCorrectResponse"],
        "config": ["watchConfig"],
        "env": ["updateModels"]
    }; }
    static get style() { return ".pie-loading{position:absolute;height:100%;width:100%;background-color:#fff;bottom:0;left:0;right:0;top:0;z-index:9999;opacity:.4}.pie-loading.fixed{position:fixed}.pie-loading:before{-webkit-box-shadow:0 0 35px rgba(63,81,181,.9);box-shadow:0 0 35px rgba(63,81,181,.9);width:50px;height:50px;-moz-animation:spinPulse 1s infinite ease-in-out;-webkit-animation:spinPulse 1s linear infinite;margin:-25px 0 0 -25px}.pie-loading:after,.pie-loading:before{content:\"\";background-color:transparent;border:5px solid rgba(63,81,181,.9);opacity:.9;border-right:5px solid transparent;border-left:5px solid transparent;border-radius:50px;position:absolute;top:50%;left:50%}.pie-loading:after{-webkit-box-shadow:0 0 15px rgba(63,81,181,.9);box-shadow:0 0 15px rgba(63,81,181,.9);width:30px;height:30px;-moz-animation:spinoffPulse 1s infinite linear;-webkit-animation:spinoffPulse 1s linear infinite;margin:-15px 0 0 -15px}\@-webkit-keyframes spinPulse{0%{-webkit-transform:rotate(160deg);transform:rotate(160deg);opacity:0;-webkit-box-shadow:0 0 1px rgba(63,81,181,.9);box-shadow:0 0 1px rgba(63,81,181,.9)}50%{-webkit-transform:rotate(145deg);transform:rotate(145deg);opacity:1}to{-webkit-transform:rotate(-320deg);transform:rotate(-320deg);opacity:0}}\@keyframes spinPulse{0%{-webkit-transform:rotate(160deg);transform:rotate(160deg);opacity:0;-webkit-box-shadow:0 0 1px rgba(63,81,181,.9);box-shadow:0 0 1px rgba(63,81,181,.9)}50%{-webkit-transform:rotate(145deg);transform:rotate(145deg);opacity:1}to{-webkit-transform:rotate(-320deg);transform:rotate(-320deg);opacity:0}}\@-webkit-keyframes spinoffPulse{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}to{-webkit-transform:rotate(1turn);transform:rotate(1turn)}}\@keyframes spinoffPulse{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}to{-webkit-transform:rotate(1turn);transform:rotate(1turn)}}.pie-api-player img{max-width:inherit!important;max-height:inherit!important}.book-title-k5{text-decoration:underline}.book-title{font-style:italic}.block-quote,.text-block{font-family:serif;margin-left:20%;margin-right:20%;margin-top:5px;margin-bottom:5px;width:60%;border:1px solid #000;padding:2px;display:block}.short-quote{font-weight:700}span.short-quote:before{content:open-quote}span.short-quote:after{content:close-quote}.word-callout{text-decoration:underline}.relative-emphasis{text-transform:uppercase}.content-emphasis{font-weight:700;color:red}span.passage-title:before{content:open-quote}span.passage-title:after{content:close-quote}.proper-name,.variable{font-style:italic}.equation-block{text-align:center;margin-left:20%;margin-right:20%;margin-top:5px;margin-bottom:5px;width:60%;padding:2px;display:block}.embedded-error{font-style:italic}div.passage-subtitle,div.passage-title{font-weight:700;text-align:center}div.passage-author{text-align:center}.p-number{float:left;font-size:.85em}.numbered-paragraph{margin-left:36px}\@media print{.kds-noprint,.noprint{display:none}}.abs,.kds-absolute{border-left:1px solid #000;border-right:1px solid #000;padding:0 .3em 0 .3em}.kds-border-1{border:1px solid #000}.kds-border-2{border:2px solid #000}.center,.kds-center{margin:0;text-align:center}.kds-exponent{font-size:85%}.kds-exponent,sup.frac{margin-left:-.4em;position:relative;top:-1.4em}sup.frac{font-size:70%}.kds-fillin{display:inline-block}.fillin,.kds-fillin{border:1px solid #000;margin:0 .2em;padding-left:1em}.fillin{display:inline-block;*display:inline}.kds-font-larger{font-size:larger}.kds-font-smaller{font-size:smaller}table.kds-fraction{border:0;display:inline-block;font-size:85%;margin:0;padding:0;vertical-align:middle}table.kds-fraction>tbody>tr>td.kds-numerator{border-bottom:1px solid #000;border-left:0;border-right:0;border-top:0;text-align:center}table.kds-fraction>tbody>tr>td.kds-denominator{border:0;text-align:center}table.frac{display:inline-block;*display:inline;vertical-align:middle;font-size:85%;border:0;margin:0;padding:0}.frac .nu{border-left:0;border-top:0;border-right:0;border-bottom:1px solid #000;text-align:center}.frac .de{border:0;text-align:center}.kds-hanging-indent{margin:0;padding-left:3em;text-indent:-3em}.kds-image-caption{font-family:Verdana,Arial,sans-serif;text-align:center}.kds-image-left{float:left;margin-right:5px}.kds-image-right{float:right;margin-left:5px}.indent,.kds-indent{text-indent:3em}.kds-nowrap{display:inline-block;white-space:nowrap}.kds-overline{text-decoration:overline}.kds-parentheses{font-family:Arial Narrow,Arial,sans-serif}.kds-parentheses,b.frac{font-size:300%;font-weight:400;position:relative;top:8px}b.frac{font-family:arial narrow}.kds-pi{font-family:Times New Roman,serif;font-size:1.2em}.newradical{border-collapse:collapse;display:inline-block;*display:inline;vertical-align:middle}.newradical td{border:0;padding:0;vertical-align:top}.newradical td.vinculum{border-top:1px solid #000;padding:0;vertical-align:top}.newradical div.exp{font-size:.7em;position:relative;left:7px;bottom:5px}.kds-underline,.under{text-decoration:underline}.kds-verdana2t,.Verdana2t{border:1px solid #fff;text-align:left;text-decoration:none;vertical-align:top}.kds-whole-number{display:inline-block}.kds-whole-number,.whole{font-size:120%;margin:0 0 0 .2em}.whole{display:inline-block;*display:inline}table.KdsTable01{border:1px solid #000;border-collapse:collapse;border-spacing:0}table.KdsTable01>tbody>tr>th{font-weight:400}table.KdsTable01>tbody>tr>th,table.KdsTable01>tbody>tr>th.bold{background-color:#d3d3d3;border:1px solid #000;padding:2px;text-align:center}table.KdsTable01>tbody>tr>th.bold{font-weight:700}table.KdsTable01>tbody>tr>td,table.KdsTable01>tbody>tr>td.bold{border:1px solid #000;padding:2px;text-align:center}table.KdsTable01>tbody>tr>td.bold{font-weight:700}table.KdsTable02{border:1px solid #000;border-collapse:collapse;border-spacing:0}table.KdsTable02>tbody>tr>th{background-color:#d3d3d3;font-weight:400}table.KdsTable02>tbody>tr>td,table.KdsTable02>tbody>tr>th{border:1px solid #000;padding:2px}.table{width:100%;margin-bottom:1em;background-color:transparent;border-collapse:collapse}.table td,.table th{padding:.75em;vertical-align:top;border-top:1px solid #dee2e6}.table tbody+tbody{border-top:2px solid #dee2e6}.table thead th{vertical-align:bottom;border-bottom:2px solid #dee2e6}th{text-align:inherit}.table-bordered,.table-bordered td,.table-bordered th{border:1px solid #dee2e6}.table-bordered thead td,.table-bordered thead th{border-bottom-width:2px}.table-striped tbody tr:nth-of-type(odd){background-color:rgba(0,0,0,.05)}.text-center{text-align:center!important}.h5,h5{font-size:1.25em}h1,h2,h3,h4,h5,h6{margin-top:0}.h1,.h2,.h3,.h4,.h5,.h6,h1,h2,h3,h4,h5,h6{margin-bottom:.5em;font-family:inherit;font-weight:500;line-height:1.2;color:inherit}.lrn_feature h3{margin-top:0!important}#item,#stimulus{width:50%;float:left}.lrn_width_auto.table{width:auto}.TEX-S3{font-family:MJXZERO,MJXTEX-S3!important}.TEX-S2{font-family:MJXZERO,MJXTEX-S2!important}.TEX-S1{font-family:MJXZERO,MJXTEX-S1!important}.TEX-I{font-family:MJXZERO,MJXTEX-I!important}mjx-c.mjx-c2032{font-family:CerebriSans,serif!important}mjx-c.mjx-c22,mjx-c.mjx-c27{font-family:MJXZERO,serif!important}table{border-collapse:collapse}.evaluate-bottom-border>:first-child:after{content:\"\";display:block;border-bottom:1px solid #d3d3d3}"; }
};

const PieSpinner = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
        /**
         * Shows the spinner
         */
        this.active = true;
    }
    render() {
        return (core.h("div", null, !this.active && core.h("slot", null), this.active ?
            core.h("div", { id: "pie-spinner-container" }, core.h("div", { id: "pie-spinner" }, core.h("svg", { class: "lds-spinner", width: "200px", height: "200px", xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 100 100", preserveAspectRatio: "xMidYMid" }, core.h("g", { transform: "rotate(0 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-1.1916666666666667s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(30 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-1.0833333333333333s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(60 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-0.9750000000000001s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(90 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-0.8666666666666667s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(120 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-0.7583333333333333s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(150 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-0.65s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(180 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-0.5416666666666666s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(210 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-0.43333333333333335s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(240 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-0.325s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(270 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-0.21666666666666667s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(300 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "-0.10833333333333334s", repeatCount: "indefinite" }))), core.h("g", { transform: "rotate(330 50 50)" }, core.h("rect", { x: "47", y: "22.5", rx: "9.4", ry: "4.5", width: "6", height: "15", fill: "#677179" }, core.h("animate", { attributeName: "opacity", values: "1;0", keyTimes: "0;1", dur: "1.3s", begin: "0s", repeatCount: "indefinite" }))))))
            :
                core.h("div", null)));
    }
    static get style() { return "#pie-spinner-container{display:-ms-flexbox;display:flex;-ms-flex-align:center;align-items:center;-ms-flex-pack:center;justify-content:center;width:100%}#pie-spinner{max-width:50%}"; }
};

const PieStimulusLayout = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
        // shows if resizing is activated or not
        this.allowedResize = false;
        // shows if user is currently resizing the component,
        this.isResizing = false;
        // the initial X-coordinate of the mouse cursor.
        this.initialX = 0;
        //  flex values of the left component
        this.initialLeftFlex = 0.5;
        //  flex values of the right component as the user resizes them
        this.initialRightFlex = 0.5;
    }
    static handleElements() {
        const el = document.querySelector('#pie-stimulus-container');
        if (el) {
            const boundRect = el.getBoundingClientRect();
            el.style.height = `calc(100vh - ${boundRect.top}px`;
        }
    }
    componentDidRender() {
        PieStimulusLayout.handleElements();
    }
    componentDidLoad() {
        PieStimulusLayout.handleElements();
        if (this.allowedResize) {
            this.resizer.addEventListener('mousedown', (e) => {
                this.isResizing = true;
                this.initialX = e.clientX;
            });
            window.addEventListener('mousemove', this.handleMouseMove.bind(this));
            this.resizer.addEventListener('mouseup', this.handleMouseUp.bind(this));
        }
        this.readMoreButton.addEventListener("click", () => this.handleReadMore());
    }
    componentDidUpdate() {
        PieStimulusLayout.handleElements();
    }
    componentDidUnload() {
        if (this.allowedResize) {
            window.removeEventListener('mousemove', this.handleMouseMove.bind(this));
            this.resizer.removeEventListener('mouseup', this.handleMouseUp.bind(this));
        }
    }
    handleReadMore() {
        if (this.readMoreButton.innerHTML.includes('Read More')) {
            this.stimulus.style.flex = '1';
            this.readMoreButton.innerHTML = `Read Less <span class=arrow-up>&#9650;</span>`;
            this.stimulus.style.overflow = 'auto';
            this.stimulus.classList.remove('truncated');
        }
        else {
            this.readMoreButton.innerHTML = `Read More <span class=arrow-down>&#9660;</span>`;
            this.stimulus.style.flex = '0 0 30%';
            this.stimulus.style.overflow = 'hidden';
            this.stimulus.scrollTop = 0;
            this.stimulus.classList.add('truncated');
        }
    }
    handleMouseMove(e) {
        if (this.isResizing && this.allowedResize) {
            const deltaX = e.clientX - this.initialX;
            const containerWidth = this.resizer.parentElement.clientWidth;
            const newLeftFlex = (containerWidth * this.initialLeftFlex + deltaX) / containerWidth;
            const newRightFlex = 1 - newLeftFlex;
            if (newLeftFlex >= 0 && newRightFlex >= 0 && newLeftFlex <= 1 && newRightFlex <= 1) {
                this.initialLeftFlex = newLeftFlex;
                this.initialRightFlex = newRightFlex;
                // Update the initial cursor position
                this.initialX = e.clientX;
            }
        }
    }
    handleMouseUp() {
        this.isResizing = false;
    }
    // add style when the containers are resizable
    getStyle(flex) {
        const viewPortWidth = window.innerWidth;
        // avoid this style for viewports < 1020
        if (this.allowedResize && viewPortWidth > 1020) {
            return { flex: `${flex}`, transition: this.isResizing ? 'none' : 'flex 0.1s ease' };
        }
        return {};
    }
    render() {
        return (core.h("div", { id: "pie-stimulus-container" }, core.h("div", { id: "stimulus", class: "truncated", ref: (el) => this.stimulus = el, style: this.getStyle(this.initialLeftFlex) }, core.h("slot", { name: 'stimulus' })), core.h("button", { id: "read-more", ref: (el) => this.readMoreButton = el }, "Read More", core.h("span", { class: "arrow-down" }, "\u25BC")), this.allowedResize &&
            core.h("div", { id: "resizer", ref: (el) => (this.resizer = el) }), core.h("div", { id: "item", style: this.getStyle(this.initialRightFlex) }, core.h("slot", { name: 'item' }))));
    }
    static get style() { return "#pie-stimulus-container{display:-ms-flexbox;display:-webkit-flex;display:flex;-webkit-flex-direction:row;-ms-flex-direction:row;flex-direction:row;-webkit-flex-wrap:nowrap;-ms-flex-wrap:nowrap;flex-wrap:nowrap;-webkit-justify-content:flex-start;-ms-flex-pack:start;justify-content:flex-start;-webkit-align-content:stretch;-ms-flex-line-pack:stretch;align-content:stretch;-webkit-align-items:flex-start;-ms-flex-align:start;align-items:flex-start;overflow:hidden}#pie-stimulus-container #stimulus{-webkit-order:0;-ms-flex-order:0;order:0;-webkit-flex:1;-ms-flex:1;flex:1;-webkit-align-self:auto;-ms-flex-item-align:auto;align-self:auto;height:100%;overflow:scroll}#pie-stimulus-container #read-more{display:none}#pie-stimulus-container #item{-webkit-order:0;-ms-flex-order:0;order:0;-webkit-flex:1;-ms-flex:1;flex:1;-webkit-align-self:auto;-ms-flex-item-align:auto;align-self:auto;height:100%;overflow:scroll}#pie-stimulus-container #resizer{width:5px;cursor:ew-resize;background-color:transparent;height:100%;-webkit-transition:width .2s ease-in-out,background-color .2s ease-in-out;transition:width .2s ease-in-out,background-color .2s ease-in-out}#pie-stimulus-container #resizer:active,#pie-stimulus-container #resizer:hover{background-color:#add8e6;cursor:ew-resize}\@media screen and (max-width:1020px){#pie-stimulus-container{-ms-flex-direction:column;flex-direction:column}#resizer{display:none}#pie-stimulus-container #stimulus{width:100%;overflow:hidden;position:relative;-ms-flex:0 0 30%;flex:0 0 30%}#pie-stimulus-container #item{width:100%;overflow:auto;-ms-flex:1;flex:1}#pie-stimulus-container #stimulusPlayer{opacity:1;-webkit-transition:opacity .3s;transition:opacity .3s}#pie-stimulus-container #stimulus.truncated:after{content:\"\";position:absolute;bottom:0;right:0;left:0;height:50px;background:-webkit-gradient(linear,left top,left bottom,from(hsla(0,0%,100%,0)),to(#fff));background:linear-gradient(hsla(0,0%,100%,0),#fff)}#pie-stimulus-container #read-more{display:block;margin-top:10px;cursor:pointer;-ms-flex-item-align:center;align-self:center;border:none;background-color:transparent;color:#6495ed}}"; }
};

exports.error_message = ErrorMessage;
exports.pie_api_player = PieApiPlayer;
exports.pie_player = Player;
exports.pie_spinner = PieSpinner;
exports.pie_stimulus_layout = PieStimulusLayout;
