'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const core = require('./core-8e67a5c9.js');
const version$1 = require('./version-52296af9.js');
require('./_commonjsHelpers-681a8719.js');
require('./index-a93bde69.js');
require('./isSymbol-9b1c8e84.js');
const utils = require('./utils-601945ce.js');
const get$1 = require('./get-ad43ac3d.js');

// Unique ID creation requires a high quality random # generator. In the browser we therefore
// require the crypto API and do not support built-in fallback to lower quality random number
// generators (like Math.random()).
let getRandomValues;
const rnds8 = new Uint8Array(16);
function rng() {
  // lazy load so that environments that need to polyfill have a chance to do so
  if (!getRandomValues) {
    // getRandomValues needs to be invoked in a context where "this" is a Crypto implementation.
    getRandomValues = typeof crypto !== 'undefined' && crypto.getRandomValues && crypto.getRandomValues.bind(crypto);

    if (!getRandomValues) {
      throw new Error('crypto.getRandomValues() not supported. See https://github.com/uuidjs/uuid#getrandomvalues-not-supported');
    }
  }

  return getRandomValues(rnds8);
}

const REGEX = /^(?:[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}|00000000-0000-0000-0000-000000000000)$/i;

function validate(uuid) {
  return typeof uuid === 'string' && REGEX.test(uuid);
}

/**
 * Convert array of 16 byte values to UUID string format of the form:
 * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
 */

const byteToHex = [];

for (let i = 0; i < 256; ++i) {
  byteToHex.push((i + 0x100).toString(16).slice(1));
}

function unsafeStringify(arr, offset = 0) {
  // Note: Be careful editing this code!  It's been tuned for performance
  // and works in ways you may not expect. See https://github.com/uuidjs/uuid/pull/434
  return (byteToHex[arr[offset + 0]] + byteToHex[arr[offset + 1]] + byteToHex[arr[offset + 2]] + byteToHex[arr[offset + 3]] + '-' + byteToHex[arr[offset + 4]] + byteToHex[arr[offset + 5]] + '-' + byteToHex[arr[offset + 6]] + byteToHex[arr[offset + 7]] + '-' + byteToHex[arr[offset + 8]] + byteToHex[arr[offset + 9]] + '-' + byteToHex[arr[offset + 10]] + byteToHex[arr[offset + 11]] + byteToHex[arr[offset + 12]] + byteToHex[arr[offset + 13]] + byteToHex[arr[offset + 14]] + byteToHex[arr[offset + 15]]).toLowerCase();
}

function stringify(arr, offset = 0) {
  const uuid = unsafeStringify(arr, offset); // Consistency check for valid UUID.  If this throws, it's likely due to one
  // of the following:
  // - One or more input array values don't map to a hex octet (leading to
  // "undefined" in the uuid)
  // - Invalid input values for the RFC `version` or `variant` fields

  if (!validate(uuid)) {
    throw TypeError('Stringified UUID is invalid');
  }

  return uuid;
}

//
// Inspired by https://github.com/LiosK/UUID.js
// and http://docs.python.org/library/uuid.html

let _nodeId;

let _clockseq; // Previous uuid creation time


let _lastMSecs = 0;
let _lastNSecs = 0; // See https://github.com/uuidjs/uuid for API details

function v1(options, buf, offset) {
  let i = buf && offset || 0;
  const b = buf || new Array(16);
  options = options || {};
  let node = options.node || _nodeId;
  let clockseq = options.clockseq !== undefined ? options.clockseq : _clockseq; // node and clockseq need to be initialized to random values if they're not
  // specified.  We do this lazily to minimize issues related to insufficient
  // system entropy.  See #189

  if (node == null || clockseq == null) {
    const seedBytes = options.random || (options.rng || rng)();

    if (node == null) {
      // Per 4.5, create and 48-bit node id, (47 random bits + multicast bit = 1)
      node = _nodeId = [seedBytes[0] | 0x01, seedBytes[1], seedBytes[2], seedBytes[3], seedBytes[4], seedBytes[5]];
    }

    if (clockseq == null) {
      // Per 4.2.2, randomize (14 bit) clockseq
      clockseq = _clockseq = (seedBytes[6] << 8 | seedBytes[7]) & 0x3fff;
    }
  } // UUID timestamps are 100 nano-second units since the Gregorian epoch,
  // (1582-10-15 00:00).  JSNumbers aren't precise enough for this, so
  // time is handled internally as 'msecs' (integer milliseconds) and 'nsecs'
  // (100-nanoseconds offset from msecs) since unix epoch, 1970-01-01 00:00.


  let msecs = options.msecs !== undefined ? options.msecs : Date.now(); // Per 4.2.1.2, use count of uuid's generated during the current clock
  // cycle to simulate higher resolution clock

  let nsecs = options.nsecs !== undefined ? options.nsecs : _lastNSecs + 1; // Time since last uuid creation (in msecs)

  const dt = msecs - _lastMSecs + (nsecs - _lastNSecs) / 10000; // Per 4.2.1.2, Bump clockseq on clock regression

  if (dt < 0 && options.clockseq === undefined) {
    clockseq = clockseq + 1 & 0x3fff;
  } // Reset nsecs if clock regresses (new clockseq) or we've moved onto a new
  // time interval


  if ((dt < 0 || msecs > _lastMSecs) && options.nsecs === undefined) {
    nsecs = 0;
  } // Per 4.2.1.2 Throw error if too many uuids are requested


  if (nsecs >= 10000) {
    throw new Error("uuid.v1(): Can't create more than 10M uuids/sec");
  }

  _lastMSecs = msecs;
  _lastNSecs = nsecs;
  _clockseq = clockseq; // Per 4.1.4 - Convert from unix epoch to Gregorian epoch

  msecs += 12219292800000; // `time_low`

  const tl = ((msecs & 0xfffffff) * 10000 + nsecs) % 0x100000000;
  b[i++] = tl >>> 24 & 0xff;
  b[i++] = tl >>> 16 & 0xff;
  b[i++] = tl >>> 8 & 0xff;
  b[i++] = tl & 0xff; // `time_mid`

  const tmh = msecs / 0x100000000 * 10000 & 0xfffffff;
  b[i++] = tmh >>> 8 & 0xff;
  b[i++] = tmh & 0xff; // `time_high_and_version`

  b[i++] = tmh >>> 24 & 0xf | 0x10; // include version

  b[i++] = tmh >>> 16 & 0xff; // `clock_seq_hi_and_reserved` (Per 4.2.2 - include variant)

  b[i++] = clockseq >>> 8 | 0x80; // `clock_seq_low`

  b[i++] = clockseq & 0xff; // `node`

  for (let n = 0; n < 6; ++n) {
    b[i + n] = node[n];
  }

  return buf || unsafeStringify(b);
}

function parse(uuid) {
  if (!validate(uuid)) {
    throw TypeError('Invalid UUID');
  }

  let v;
  const arr = new Uint8Array(16); // Parse ########-....-....-....-............

  arr[0] = (v = parseInt(uuid.slice(0, 8), 16)) >>> 24;
  arr[1] = v >>> 16 & 0xff;
  arr[2] = v >>> 8 & 0xff;
  arr[3] = v & 0xff; // Parse ........-####-....-....-............

  arr[4] = (v = parseInt(uuid.slice(9, 13), 16)) >>> 8;
  arr[5] = v & 0xff; // Parse ........-....-####-....-............

  arr[6] = (v = parseInt(uuid.slice(14, 18), 16)) >>> 8;
  arr[7] = v & 0xff; // Parse ........-....-....-####-............

  arr[8] = (v = parseInt(uuid.slice(19, 23), 16)) >>> 8;
  arr[9] = v & 0xff; // Parse ........-....-....-....-############
  // (Use "/" to avoid 32-bit truncation when bit-shifting high-order bytes)

  arr[10] = (v = parseInt(uuid.slice(24, 36), 16)) / 0x10000000000 & 0xff;
  arr[11] = v / 0x100000000 & 0xff;
  arr[12] = v >>> 24 & 0xff;
  arr[13] = v >>> 16 & 0xff;
  arr[14] = v >>> 8 & 0xff;
  arr[15] = v & 0xff;
  return arr;
}

function stringToBytes(str) {
  str = unescape(encodeURIComponent(str)); // UTF8 escape

  const bytes = [];

  for (let i = 0; i < str.length; ++i) {
    bytes.push(str.charCodeAt(i));
  }

  return bytes;
}

const DNS = '6ba7b810-9dad-11d1-80b4-00c04fd430c8';
const URL = '6ba7b811-9dad-11d1-80b4-00c04fd430c8';
function v35(name, version, hashfunc) {
  function generateUUID(value, namespace, buf, offset) {
    var _namespace;

    if (typeof value === 'string') {
      value = stringToBytes(value);
    }

    if (typeof namespace === 'string') {
      namespace = parse(namespace);
    }

    if (((_namespace = namespace) === null || _namespace === void 0 ? void 0 : _namespace.length) !== 16) {
      throw TypeError('Namespace must be array-like (16 iterable integer values, 0-255)');
    } // Compute hash of namespace and value, Per 4.3
    // Future: Use spread syntax when supported on all platforms, e.g. `bytes =
    // hashfunc([...namespace, ... value])`


    let bytes = new Uint8Array(16 + value.length);
    bytes.set(namespace);
    bytes.set(value, namespace.length);
    bytes = hashfunc(bytes);
    bytes[6] = bytes[6] & 0x0f | version;
    bytes[8] = bytes[8] & 0x3f | 0x80;

    if (buf) {
      offset = offset || 0;

      for (let i = 0; i < 16; ++i) {
        buf[offset + i] = bytes[i];
      }

      return buf;
    }

    return unsafeStringify(bytes);
  } // Function#name is not settable on some platforms (#270)


  try {
    generateUUID.name = name; // eslint-disable-next-line no-empty
  } catch (err) {} // For CommonJS default export support


  generateUUID.DNS = DNS;
  generateUUID.URL = URL;
  return generateUUID;
}

/*
 * Browser-compatible JavaScript MD5
 *
 * Modification of JavaScript MD5
 * https://github.com/blueimp/JavaScript-MD5
 *
 * Copyright 2011, Sebastian Tschan
 * https://blueimp.net
 *
 * Licensed under the MIT license:
 * https://opensource.org/licenses/MIT
 *
 * Based on
 * A JavaScript implementation of the RSA Data Security, Inc. MD5 Message
 * Digest Algorithm, as defined in RFC 1321.
 * Version 2.2 Copyright (C) Paul Johnston 1999 - 2009
 * Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
 * Distributed under the BSD License
 * See http://pajhome.org.uk/crypt/md5 for more info.
 */
function md5(bytes) {
  if (typeof bytes === 'string') {
    const msg = unescape(encodeURIComponent(bytes)); // UTF8 escape

    bytes = new Uint8Array(msg.length);

    for (let i = 0; i < msg.length; ++i) {
      bytes[i] = msg.charCodeAt(i);
    }
  }

  return md5ToHexEncodedArray(wordsToMd5(bytesToWords(bytes), bytes.length * 8));
}
/*
 * Convert an array of little-endian words to an array of bytes
 */


function md5ToHexEncodedArray(input) {
  const output = [];
  const length32 = input.length * 32;
  const hexTab = '0123456789abcdef';

  for (let i = 0; i < length32; i += 8) {
    const x = input[i >> 5] >>> i % 32 & 0xff;
    const hex = parseInt(hexTab.charAt(x >>> 4 & 0x0f) + hexTab.charAt(x & 0x0f), 16);
    output.push(hex);
  }

  return output;
}
/**
 * Calculate output length with padding and bit length
 */


function getOutputLength(inputLength8) {
  return (inputLength8 + 64 >>> 9 << 4) + 14 + 1;
}
/*
 * Calculate the MD5 of an array of little-endian words, and a bit length.
 */


function wordsToMd5(x, len) {
  /* append padding */
  x[len >> 5] |= 0x80 << len % 32;
  x[getOutputLength(len) - 1] = len;
  let a = 1732584193;
  let b = -271733879;
  let c = -1732584194;
  let d = 271733878;

  for (let i = 0; i < x.length; i += 16) {
    const olda = a;
    const oldb = b;
    const oldc = c;
    const oldd = d;
    a = md5ff(a, b, c, d, x[i], 7, -680876936);
    d = md5ff(d, a, b, c, x[i + 1], 12, -389564586);
    c = md5ff(c, d, a, b, x[i + 2], 17, 606105819);
    b = md5ff(b, c, d, a, x[i + 3], 22, -1044525330);
    a = md5ff(a, b, c, d, x[i + 4], 7, -176418897);
    d = md5ff(d, a, b, c, x[i + 5], 12, 1200080426);
    c = md5ff(c, d, a, b, x[i + 6], 17, -1473231341);
    b = md5ff(b, c, d, a, x[i + 7], 22, -45705983);
    a = md5ff(a, b, c, d, x[i + 8], 7, 1770035416);
    d = md5ff(d, a, b, c, x[i + 9], 12, -1958414417);
    c = md5ff(c, d, a, b, x[i + 10], 17, -42063);
    b = md5ff(b, c, d, a, x[i + 11], 22, -1990404162);
    a = md5ff(a, b, c, d, x[i + 12], 7, 1804603682);
    d = md5ff(d, a, b, c, x[i + 13], 12, -40341101);
    c = md5ff(c, d, a, b, x[i + 14], 17, -1502002290);
    b = md5ff(b, c, d, a, x[i + 15], 22, 1236535329);
    a = md5gg(a, b, c, d, x[i + 1], 5, -165796510);
    d = md5gg(d, a, b, c, x[i + 6], 9, -1069501632);
    c = md5gg(c, d, a, b, x[i + 11], 14, 643717713);
    b = md5gg(b, c, d, a, x[i], 20, -373897302);
    a = md5gg(a, b, c, d, x[i + 5], 5, -701558691);
    d = md5gg(d, a, b, c, x[i + 10], 9, 38016083);
    c = md5gg(c, d, a, b, x[i + 15], 14, -660478335);
    b = md5gg(b, c, d, a, x[i + 4], 20, -405537848);
    a = md5gg(a, b, c, d, x[i + 9], 5, 568446438);
    d = md5gg(d, a, b, c, x[i + 14], 9, -1019803690);
    c = md5gg(c, d, a, b, x[i + 3], 14, -187363961);
    b = md5gg(b, c, d, a, x[i + 8], 20, 1163531501);
    a = md5gg(a, b, c, d, x[i + 13], 5, -1444681467);
    d = md5gg(d, a, b, c, x[i + 2], 9, -51403784);
    c = md5gg(c, d, a, b, x[i + 7], 14, 1735328473);
    b = md5gg(b, c, d, a, x[i + 12], 20, -1926607734);
    a = md5hh(a, b, c, d, x[i + 5], 4, -378558);
    d = md5hh(d, a, b, c, x[i + 8], 11, -2022574463);
    c = md5hh(c, d, a, b, x[i + 11], 16, 1839030562);
    b = md5hh(b, c, d, a, x[i + 14], 23, -35309556);
    a = md5hh(a, b, c, d, x[i + 1], 4, -1530992060);
    d = md5hh(d, a, b, c, x[i + 4], 11, 1272893353);
    c = md5hh(c, d, a, b, x[i + 7], 16, -155497632);
    b = md5hh(b, c, d, a, x[i + 10], 23, -1094730640);
    a = md5hh(a, b, c, d, x[i + 13], 4, 681279174);
    d = md5hh(d, a, b, c, x[i], 11, -358537222);
    c = md5hh(c, d, a, b, x[i + 3], 16, -722521979);
    b = md5hh(b, c, d, a, x[i + 6], 23, 76029189);
    a = md5hh(a, b, c, d, x[i + 9], 4, -640364487);
    d = md5hh(d, a, b, c, x[i + 12], 11, -421815835);
    c = md5hh(c, d, a, b, x[i + 15], 16, 530742520);
    b = md5hh(b, c, d, a, x[i + 2], 23, -995338651);
    a = md5ii(a, b, c, d, x[i], 6, -198630844);
    d = md5ii(d, a, b, c, x[i + 7], 10, 1126891415);
    c = md5ii(c, d, a, b, x[i + 14], 15, -1416354905);
    b = md5ii(b, c, d, a, x[i + 5], 21, -57434055);
    a = md5ii(a, b, c, d, x[i + 12], 6, 1700485571);
    d = md5ii(d, a, b, c, x[i + 3], 10, -1894986606);
    c = md5ii(c, d, a, b, x[i + 10], 15, -1051523);
    b = md5ii(b, c, d, a, x[i + 1], 21, -2054922799);
    a = md5ii(a, b, c, d, x[i + 8], 6, 1873313359);
    d = md5ii(d, a, b, c, x[i + 15], 10, -30611744);
    c = md5ii(c, d, a, b, x[i + 6], 15, -1560198380);
    b = md5ii(b, c, d, a, x[i + 13], 21, 1309151649);
    a = md5ii(a, b, c, d, x[i + 4], 6, -145523070);
    d = md5ii(d, a, b, c, x[i + 11], 10, -1120210379);
    c = md5ii(c, d, a, b, x[i + 2], 15, 718787259);
    b = md5ii(b, c, d, a, x[i + 9], 21, -343485551);
    a = safeAdd(a, olda);
    b = safeAdd(b, oldb);
    c = safeAdd(c, oldc);
    d = safeAdd(d, oldd);
  }

  return [a, b, c, d];
}
/*
 * Convert an array bytes to an array of little-endian words
 * Characters >255 have their high-byte silently ignored.
 */


function bytesToWords(input) {
  if (input.length === 0) {
    return [];
  }

  const length8 = input.length * 8;
  const output = new Uint32Array(getOutputLength(length8));

  for (let i = 0; i < length8; i += 8) {
    output[i >> 5] |= (input[i / 8] & 0xff) << i % 32;
  }

  return output;
}
/*
 * Add integers, wrapping at 2^32. This uses 16-bit operations internally
 * to work around bugs in some JS interpreters.
 */


function safeAdd(x, y) {
  const lsw = (x & 0xffff) + (y & 0xffff);
  const msw = (x >> 16) + (y >> 16) + (lsw >> 16);
  return msw << 16 | lsw & 0xffff;
}
/*
 * Bitwise rotate a 32-bit number to the left.
 */


function bitRotateLeft(num, cnt) {
  return num << cnt | num >>> 32 - cnt;
}
/*
 * These functions implement the four basic operations the algorithm uses.
 */


function md5cmn(q, a, b, x, s, t) {
  return safeAdd(bitRotateLeft(safeAdd(safeAdd(a, q), safeAdd(x, t)), s), b);
}

function md5ff(a, b, c, d, x, s, t) {
  return md5cmn(b & c | ~b & d, a, b, x, s, t);
}

function md5gg(a, b, c, d, x, s, t) {
  return md5cmn(b & d | c & ~d, a, b, x, s, t);
}

function md5hh(a, b, c, d, x, s, t) {
  return md5cmn(b ^ c ^ d, a, b, x, s, t);
}

function md5ii(a, b, c, d, x, s, t) {
  return md5cmn(c ^ (b | ~d), a, b, x, s, t);
}

const v3 = v35('v3', 0x30, md5);

const randomUUID = typeof crypto !== 'undefined' && crypto.randomUUID && crypto.randomUUID.bind(crypto);
const native = {
  randomUUID
};

function v4(options, buf, offset) {
  if (native.randomUUID && !buf && !options) {
    return native.randomUUID();
  }

  options = options || {};
  const rnds = options.random || (options.rng || rng)(); // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`

  rnds[6] = rnds[6] & 0x0f | 0x40;
  rnds[8] = rnds[8] & 0x3f | 0x80; // Copy bytes to buffer, if provided

  if (buf) {
    offset = offset || 0;

    for (let i = 0; i < 16; ++i) {
      buf[offset + i] = rnds[i];
    }

    return buf;
  }

  return unsafeStringify(rnds);
}

// Adapted from Chris Veness' SHA1 code at
// http://www.movable-type.co.uk/scripts/sha1.html
function f(s, x, y, z) {
  switch (s) {
    case 0:
      return x & y ^ ~x & z;

    case 1:
      return x ^ y ^ z;

    case 2:
      return x & y ^ x & z ^ y & z;

    case 3:
      return x ^ y ^ z;
  }
}

function ROTL(x, n) {
  return x << n | x >>> 32 - n;
}

function sha1(bytes) {
  const K = [0x5a827999, 0x6ed9eba1, 0x8f1bbcdc, 0xca62c1d6];
  const H = [0x67452301, 0xefcdab89, 0x98badcfe, 0x10325476, 0xc3d2e1f0];

  if (typeof bytes === 'string') {
    const msg = unescape(encodeURIComponent(bytes)); // UTF8 escape

    bytes = [];

    for (let i = 0; i < msg.length; ++i) {
      bytes.push(msg.charCodeAt(i));
    }
  } else if (!Array.isArray(bytes)) {
    // Convert Array-like to Array
    bytes = Array.prototype.slice.call(bytes);
  }

  bytes.push(0x80);
  const l = bytes.length / 4 + 2;
  const N = Math.ceil(l / 16);
  const M = new Array(N);

  for (let i = 0; i < N; ++i) {
    const arr = new Uint32Array(16);

    for (let j = 0; j < 16; ++j) {
      arr[j] = bytes[i * 64 + j * 4] << 24 | bytes[i * 64 + j * 4 + 1] << 16 | bytes[i * 64 + j * 4 + 2] << 8 | bytes[i * 64 + j * 4 + 3];
    }

    M[i] = arr;
  }

  M[N - 1][14] = (bytes.length - 1) * 8 / Math.pow(2, 32);
  M[N - 1][14] = Math.floor(M[N - 1][14]);
  M[N - 1][15] = (bytes.length - 1) * 8 & 0xffffffff;

  for (let i = 0; i < N; ++i) {
    const W = new Uint32Array(80);

    for (let t = 0; t < 16; ++t) {
      W[t] = M[i][t];
    }

    for (let t = 16; t < 80; ++t) {
      W[t] = ROTL(W[t - 3] ^ W[t - 8] ^ W[t - 14] ^ W[t - 16], 1);
    }

    let a = H[0];
    let b = H[1];
    let c = H[2];
    let d = H[3];
    let e = H[4];

    for (let t = 0; t < 80; ++t) {
      const s = Math.floor(t / 20);
      const T = ROTL(a, 5) + f(s, b, c, d) + e + K[s] + W[t] >>> 0;
      e = d;
      d = c;
      c = ROTL(b, 30) >>> 0;
      b = a;
      a = T;
    }

    H[0] = H[0] + a >>> 0;
    H[1] = H[1] + b >>> 0;
    H[2] = H[2] + c >>> 0;
    H[3] = H[3] + d >>> 0;
    H[4] = H[4] + e >>> 0;
  }

  return [H[0] >> 24 & 0xff, H[0] >> 16 & 0xff, H[0] >> 8 & 0xff, H[0] & 0xff, H[1] >> 24 & 0xff, H[1] >> 16 & 0xff, H[1] >> 8 & 0xff, H[1] & 0xff, H[2] >> 24 & 0xff, H[2] >> 16 & 0xff, H[2] >> 8 & 0xff, H[2] & 0xff, H[3] >> 24 & 0xff, H[3] >> 16 & 0xff, H[3] >> 8 & 0xff, H[3] & 0xff, H[4] >> 24 & 0xff, H[4] >> 16 & 0xff, H[4] >> 8 & 0xff, H[4] & 0xff];
}

const v5 = v35('v5', 0x50, sha1);

const nil = '00000000-0000-0000-0000-000000000000';

function version(uuid) {
  if (!validate(uuid)) {
    throw TypeError('Invalid UUID');
  }

  return parseInt(uuid.slice(14, 15), 16);
}

class PieApiImageSupport extends version$1.NewRelicEnabledClient {
    constructor(token, graphql, versionedID, versionedIDGenerator) {
        super();
        this.token = token;
        this.graphql = graphql;
        this.versionedID = versionedID;
        this.versionedIDGenerator = versionedIDGenerator;
        // Create a FetchUtils instance with tracking info
        this.fetchUtils = new version$1.FetchUtils({
            graphqlEndpoint: this.graphql,
            hasToken: !!this.token,
            hasVersionedID: !!this.versionedID
        });
    }
    getTrackingBaseAttributes() {
        return {
            graphqlEndpoint: this.graphql,
            hasToken: !!this.token,
            hasVersionedID: !!this.versionedID,
            networkInfo: this.getNetworkInfo()
        };
    }
    async insert(file, done) {
        var _a, _b;
        try {
            const info = await this.createUploadRequest(file.type);
            const uploadResponse = await this.fetchUtils.uploadFile(info.signedURL, file, "image-upload");
            if (uploadResponse.ok) {
                done(null, info.publicURL);
            }
            else {
                const error = new Error(`Upload failed (url ${info.signedURL})`);
                if ((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable()) {
                    this.newRelic.noticeError(error, { signedUrl: info.signedURL, status: uploadResponse.status });
                }
                done(error, null);
            }
        }
        catch (e) {
            const error = new Error(`Upload failed: ${e.message || 'Unknown error'}`);
            if ((_b = this.newRelic) === null || _b === void 0 ? void 0 : _b.isAvailable()) {
                this.newRelic.noticeError(error, { message: e.message || 'Unknown error' });
            }
            done(error, null);
        }
    }
    async createUploadRequest(contentType) {
        var _a;
        // The new pie api made media ID required for performance (previously ID was optional).
        // This means we have to treat this edge case:
        //  If the item is newly created and doesn't have an ID yet (was not saved in the db yet),
        //  we need to generate the ID, upload the media with that ID and also make sure to use the
        //  generated ID in pie-api-author for other item-related actions (eg: upload other medias, save item)
        if (!this.versionedID && this.versionedIDGenerator) {
            this.versionedID = this.versionedIDGenerator();
        }
        const query = `
      mutation RequestMediaUpload($id: ID!, $contentType: String!) {
        requestMediaUpload(id: $id, contentType: $contentType) {
          signedURL
          publicURL
        }
      }
    `;
        const variables = {
            id: this.versionedID || 'default-id',
            contentType
        };
        const response = await this.fetchUtils.executeGraphQL(this.graphql, this.token, query, variables);
        const data = get$1.get(response, "data.requestMediaUpload");
        if (!data) {
            const errors = get$1.get(response, "errors", []); // Provide default empty array
            const uploadError = errors && errors.length ? errors.find(e => e.path && e.path.includes && e.path.includes("requestMediaUpload")) : null;
            if ((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable()) {
                this.newRelic.noticeError(uploadError || new Error("Unknown error"), { errors: errors });
            }
            throw uploadError
                ? new Error(uploadError.message)
                : new Error(`Unknown gql error: ${JSON.stringify(response)}`);
        }
        return data;
    }
    async delete(src, done) {
        var _a, _b;
        if (!src) {
            done();
            return;
        }
        const mut = `
      mutation RequestMediaDelete($publicURL: String!) {
        requestMediaDelete(publicURL: $publicURL) {
          signedURL
        }
      }
    `;
        try {
            const response = await this.fetchUtils.executeGraphQL(this.graphql, this.token, mut, { publicURL: src });
            const info = get$1.get(response, "data.requestMediaDelete");
            if (!info) {
                const errors = get$1.get(response, "errors", []); // Provide default empty array
                const deleteError = (errors && errors.length) ? errors.find(e => e.path && e.path.includes && e.path.includes("requestMediaDelete")) : null;
                const errorMessage = deleteError ? deleteError.message : "no info in json?";
                let error = new Error(errorMessage);
                if ((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable()) {
                    this.newRelic.noticeError(error, { errors: errors });
                }
                throw error;
            }
            // Use the specialized delete method for DELETE operations
            const deleteResponse = await this.fetchUtils.deleteWithSignedUrl(info.signedURL, "image-delete");
            if (deleteResponse.ok) {
                done();
            }
            else {
                let error = new Error("DELETE failed");
                if ((_b = this.newRelic) === null || _b === void 0 ? void 0 : _b.isAvailable()) {
                    this.newRelic.noticeError(error, {});
                }
                throw error;
            }
        }
        catch (e) {
            done(e);
        }
    }
}

class PieApiSoundSupport extends version$1.NewRelicEnabledClient {
    constructor(token, graphql, versionedID, versionedIDGenerator) {
        super();
        this.token = token;
        this.graphql = graphql;
        this.versionedID = versionedID;
        this.versionedIDGenerator = versionedIDGenerator;
        // Create a FetchUtils instance with tracking info
        this.fetchUtils = new version$1.FetchUtils({
            graphqlEndpoint: this.graphql,
            hasToken: !!this.token,
            hasVersionedID: !!this.versionedID
        });
    }
    getTrackingBaseAttributes() {
        return {
            graphqlEndpoint: this.graphql,
            hasToken: !!this.token,
            hasVersionedID: !!this.versionedID,
            networkInfo: this.getNetworkInfo()
        };
    }
    async insert(file, done) {
        var _a, _b;
        try {
            const typeParsed = file.type.replace('x-', '');
            const info = await this.createUploadRequest(typeParsed);
            const uploadResponse = await this.fetchUtils.uploadFile(info.signedURL, file, "sound-upload");
            if (uploadResponse.ok) {
                done(null, info.publicURL);
            }
            else {
                const error = new Error(`Upload failed (url ${info.signedURL})`);
                if ((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable()) {
                    this.newRelic.noticeError(error, { signedUrl: info.signedURL, status: uploadResponse.status });
                }
                done(error, null);
            }
        }
        catch (e) {
            const error = new Error(`Upload failed: ${e.message}`);
            if ((_b = this.newRelic) === null || _b === void 0 ? void 0 : _b.isAvailable()) {
                this.newRelic.noticeError(error, { message: e.message });
            }
            done(error, null);
        }
    }
    async createUploadRequest(contentType) {
        var _a;
        // The new pie api made media ID required for performance (previously ID was optional).
        // This means we have to treat this edge case:
        //  If the item is newly created and doesn't have an ID yet (was not saved in the db yet),
        //  we need to generate the ID, upload the media with that ID and also make sure to use the
        //  generated ID in pie-api-author for other item-related actions (eg: upload other medias, save item)
        if (!this.versionedID && this.versionedIDGenerator) {
            this.versionedID = this.versionedIDGenerator();
        }
        const query = `
      mutation RequestMediaUpload($id: ID!, $contentType: String!) {
        requestMediaUpload(id: $id, contentType: $contentType) {
        signedURL
        publicURL
      }
    }`;
        const response = await this.fetchUtils.executeGraphQL(this.graphql, this.token, query, {
            id: this.versionedID,
            contentType
        });
        const data = get$1.get(response, "data.requestMediaUpload");
        if (!data) {
            const errors = get$1.get(response, "errors");
            const uploadError = errors.find(e => e.path.includes("requestMediaUpload"));
            if ((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable()) {
                this.newRelic.noticeError(uploadError, { errors: errors });
            }
            throw uploadError
                ? new Error(uploadError.message)
                : new Error(`Unknown gql error: ${JSON.stringify(response)}`);
        }
        return data;
    }
    async delete(src, done) {
        var _a, _b;
        if (!src) {
            return;
        }
        const mut = `
      mutation RequestMediaDelete($publicURL: String!) {
        requestMediaDelete(publicURL: $publicURL) {
        signedURL
      }
    }`;
        try {
            const response = await this.fetchUtils.executeGraphQL(this.graphql, this.token, mut, { publicURL: src });
            const info = get$1.get(response, "data.requestMediaDelete");
            if (!info) {
                const errors = get$1.get(response, "errors");
                const deleteError = (errors || []).find(e => e.path.includes("requestMediaDelete"));
                let error = new Error(deleteError ? deleteError.message : "no info in json?");
                if ((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable()) {
                    this.newRelic.noticeError(error, { errors: errors });
                }
                throw error;
            }
            // Use the specialized delete method for DELETE operations
            const deleteResponse = await this.fetchUtils.deleteWithSignedUrl(info.signedURL, "sound-delete");
            if (deleteResponse.ok) {
                done();
            }
            else {
                let error = new Error("DELETE failed");
                if ((_b = this.newRelic) === null || _b === void 0 ? void 0 : _b.isAvailable()) {
                    this.newRelic.noticeError(error, {});
                }
                throw error;
            }
        }
        catch (e) {
            done(e);
        }
    }
}

const log = () => ({});
// const log: any = console.log.bind(console, "[pie-versions]");
let versions;
/**
 * Use these as a fallback if all else fails. To be removed.
 */
const LegacyPieVersions = {
    "@pie-element/categorize": "6.5.4",
    "@pie-element/charting": "2.5.4",
    "@pie-element/drag-in-the-blank": "3.7.4",
    "@pie-element/drawing-response": "3.5.3",
    "@pie-element/ebsr": "6.3.4",
    "@pie-element/explicit-constructed-response": "3.7.4",
    "@pie-element/extended-text-entry": "6.3.3",
    "@pie-element/graph-lines": "4.0.7",
    "@pie-element/graphing": "3.4.4",
    "@pie-element/hotspot": "4.7.4",
    "@pie-element/image-cloze-association": "3.4.4",
    "@pie-element/inline-choice": "2.10.9",
    "@pie-element/inline-dropdown": "3.5.4",
    "@pie-element/match": "4.5.4",
    "@pie-element/match-list": "1.11.4",
    "@pie-element/math-inline": "5.4.4",
    "@pie-element/multiple-choice": "5.5.4",
    "@pie-element/number-line": "5.9.4",
    "@pie-element/placement-ordering": "7.3.4",
    "@pie-element/point-intercept": "4.5.8",
    "@pie-element/rubric": "1.5.2",
    "@pie-element/ruler": "3.11.3",
    "@pie-element/select-text": "6.3.4",
    "@pie-element/text-entry": "5.0.7",
};
const noOlderThan = (n, minutes) => {
    const diff = new Date().getTime() - n;
    log("age in millis", diff);
    return diff < minutes * 1000 * 60;
};
const KEY = "__pie_api_components_author_versions__";
const localVersions = () => {
    try {
        const local = localStorage.getItem(KEY);
        const o = JSON.parse(local);
        if (!o) {
            return;
        }
        if (noOlderThan(o.createdAt, 60)) {
            return o.versions;
        }
    }
    catch (e) {
        console.error(e.message);
    }
};
const get = async (token, apiUrl) => {
    try {
        log("get, token", token, "apiUrl:", apiUrl);
        if (!versions) {
            const local = localVersions();
            log("get - local:", local);
            if (local) {
                versions = local;
            }
            else {
                // console.log("get - loadBestVersions ", apiUrl);
                const r = await version$1.loadBestVersions(token, apiUrl);
                versions = r.reduce((acc, nv) => {
                    acc[nv.pie] = nv.version;
                    return acc;
                }, {});
                log("versions back from server:", versions);
                localStorage.setItem(KEY, JSON.stringify({ versions, createdAt: new Date().getTime() }));
            }
        }
        log("get - return versions");
        return versions;
    }
    catch (e) {
        console.warn(`[pie-versions:get] ${apiUrl} - Error loading best versions - returning fallback versions, this will throw an error in future`);
        console.error(e.message);
        return LegacyPieVersions;
    }
};

const PieApiAuthor = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
        /**
         * Choose 'dev' , 'stage' or 'prod' environments for back end.
         */
        this.host = "prod";
        /**
         * used to automatically re-fetch the bundle (in case we get a 503); defaults to false
         */
        this.reFetchBundle = false;
        /**
         * For development purposes only - allows alternate api endpoints to be provided.
         */
        this.apiConfig = version$1.PLAYER_HOST_CONFIG;
        /**
         * Adds a preview view which will render the content in another tab as it may appear to a student or instructor.
         */
        this.addPreview = false;
        /**
         * Enable screen recording with logrocket.
         * Note: not to be used in production unless for temporary use to track down a specific user problem.
         */
        this.recordScreen = false;
        /**
         * Enable exception tracking.
         */
        this.exceptions = false;
        /**
         * If true, the `@version` tag for elements in the configuration will be ignored and a static version for that
         * interaction type determined by the current release of this pie-api-author component will be used.
         */
        this.lockVersions = false;
        this.generateUUID = () => {
            // This function is a callback sent to PieApiImageSupport and PieApiSoundSupport, to treat a specific edge case:
            //  If the item is newly created and doesn't have an ID yet (was not saved in the db yet),
            //  we need to generate the ID, upload the media with that ID and also make sure to use the
            //  generated ID in pie-api-author for other item-related actions (eg: upload other medias, save item)
            this.contentId = v4();
            // Store the generated id
            return this.contentId;
        };
        this.contentLoaded = core.createEvent(this, "contentLoaded", 7);
        this.contentIdValidated = core.createEvent(this, "contentIdValidated", 7);
        this.recordingStarted = core.createEvent(this, "recordingStarted", 7);
        this.doc = core.getContext(this, "document");
    }
    async watchHostEnvironment(newVal) {
        if (newVal) {
            this.apiConfig.host = version$1.HOST_DEFAULTS[newVal].apiUrl;
            this.apiConfig.bundleBaseUrl = version$1.HOST_DEFAULTS[newVal].bundleBaseUrl;
            if (this.token) {
                await get(this.token, this.apiConfig.host);
            }
        }
    }
    /**
     * Utility method to add a `@pie-element/rubric` section to an item config when creating an item should be used before setting the config.
     *
     * @deprecated this method is for temporary use, will be removed at next major release
     *
     * @param config the item config to mutate
     * @param rubricModel optional model to pass to `@pie-element/rubric`
     */
    async addRubricToConfig(config, rubricModel) {
        console.warn("This function is deprecated. Use complex-rubric functionality instead.");
        if (!rubricModel) {
            rubricModel = {
                id: "rubric",
                element: "pie-rubric",
                points: ["", "", "", ""],
                excludeZero: false,
            };
        }
        const configPieContent = utils.pieContentFromConfig(config);
        get$1.addPackageToContent(configPieContent, "@pie-element/rubric", rubricModel);
        const updatedConfig = get$1.addRubric(configPieContent);
        return updatedConfig;
    }
    getOverrides() {
        let overrides = undefined;
        try {
            if (typeof this.overrides === "string") {
                overrides = JSON.parse(this.overrides);
            }
            else {
                if (this.overrides) {
                    overrides = this.overrides;
                }
            }
        }
        catch (e) { }
        return overrides;
    }
    /**
     * Loads a content item from `pie-api` host into the view.
     *
     * @param contentId the unique versioned identifer `id@version` for the content item to load. If this param
     * is not provided and `content-id` attribute has a value, that value will be used to load the content item.
     */
    async load(contentId) {
        if (!this.token) {
            return;
        }
        const idToLoad = contentId ? contentId : this.contentId;
        if (idToLoad) {
            const contentData = await version$1.loadFromPieApi(this.token, idToLoad, this.apiConfig.host, this.getOverrides());
            this.contentConfig = contentData.config;
            const stringVersionId = version$1.versionedIdtoString({
                id: contentData.id,
                version: contentData.version,
            });
            this.contentId = stringVersionId;
            this.contentLoaded.emit({
                id: this.contentId,
                config: this.contentConfig,
            });
        }
    }
    /**
     * Creates a new content item from the currently loaded contents.
     *
     * @param preReleaseType - see preRelaseType for the `save` method
     */
    async create(preReleaseType = "premajor") {
        const cc = this.shouldLock()
            ? version$1.lockConfigVersions(this.contentConfig, this.versions, {})
            : version$1.unlockConfigVersions(this.contentConfig);
        return version$1.saveContentItem(this.token, this.contentId || null, preReleaseType, cc, this.collectionId ? [this.collectionId] : null, this.apiConfig.host);
    }
    /**
     * Saves an update to an existing pre-released content item.
     * The content item version must be in pre-release state.
     *
     * the `preReleaseType` param determines what kind of version update, if any, will be performed.
     *
     * `null` - (default) writes over existing pre-release content, , e.g.`1.0.0-draft.0 -> 1.0.0-draft.0`\n
     * `prerelease` - writes in increment to the prerelease version, e.g. `1.0.0-draft.0 -> 1.0.0-draft.1`\n
     * `premajor` - creates a new major release from version, e.g. `1.0.0-draft.1 -> 2.0.0-draft.0`\n
     * `preminor` - creates a new minor release from version, e.g. `1.0.0-draft.1 -> 1.1.0-draft.0`\n
     * `prepatch` - creates a new patch release from version, e.g. `1.0.0-draft.1 -> 1.0.1-draft.0`\m
     *
     * @param preReleaseType - the type of update to perform
     */
    async save(preReleaseType = null) {
        let newVersionedId;
        if (this.contentConfig) {
            const cc = this.shouldLock()
                ? version$1.lockConfigVersions(this.contentConfig, this.versions, {})
                : version$1.unlockConfigVersions(this.contentConfig);
            newVersionedId = await version$1.saveContentItem(this.token, this.contentId, preReleaseType, cc, this.collectionId ? [this.collectionId] : null, this.apiConfig.host);
            if (newVersionedId) {
                this.contentId = newVersionedId;
                return newVersionedId;
            }
            else {
                return Promise.resolve(null);
            }
        }
        else {
            return Promise.resolve(null);
        }
    }
    version() {
        return Promise.resolve(version$1.VERSION);
    }
    /**
     * this method is for having the possibility to access validateModels from player <br/><br/> It returns the model of the item and errors field which contains the errors returned by validate function in controller. It is an asynchronous method, so it needs to be called with await
     *
     */
    validate() {
        if (this.pieConfigEl) {
            return this.pieConfigEl.validateModels();
        }
        return null;
    }
    /**
     * Release a pre-release version of the content currently loaded in the authoring view.
     *
     * Throws an exception if the version of the current Content Item does not have a pre-release tag e.g.`1.0.1-draft`
     *
     *  example:
     * (`contentId = 'abc@1.0.1-draft`)
     *
     *  `this.release() => 'abc@1.0.1`'
     *
     */
    async release() {
        return version$1.releaseContentItem(this.token, this.contentId, this.apiConfig.host);
    }
    /**
     * TODO
     * Gets a list of all versions that exist for this Content Item.
     */
    getVersions() {
        return Promise.resolve([""]);
    }
    async onTokenChange() {
        if (this.token) {
            await get(this.token, this.apiConfig.host);
        }
        this.load(this.contentId);
    }
    async watchContentId(newVal) {
        const info = version$1.idInfo(newVal);
        this.contentIdValidated.emit({ valid: !!info, status: info });
    }
    shouldLock() {
        return !this.overrides && this.lockVersions;
    }
    watchRecordScreen(newVal, oldVal) {
        if (newVal && newVal !== oldVal) {
            if (this.newRelic && this.recordScreenIdentity) {
                // Set user identity for New Relic session
                this.newRelic.setUserId(this.recordScreenIdentity);
                this.recordingStarted.emit("Session recording started with New Relic");
            }
        }
    }
    getImageSupport() {
        return new PieApiImageSupport(this.token, this.apiConfig.host + "/graphql", this.contentId, this.generateUUID);
    }
    getUploadSoundSupport() {
        return new PieApiSoundSupport(this.token, this.apiConfig.host + "/graphql", this.contentId, this.generateUUID);
    }
    async componentWillLoad() {
        if (this.token) {
            get(this.token, this.apiConfig.host);
        }
        this.newRelic = new version$1.NewRelicInstrumentationService(this.doc);
        this.newRelic.setApplicationVersion(`pie-api-components@${version$1.VERSION}`);
        this.newRelic.setCustomAttribute('componentType', 'pie-api-author');
        if (this.recordScreen) {
            this.watchRecordScreen(this.recordScreen, false);
        }
        this.watchHostEnvironment(this.host);
        this.versions = await get(this.token, this.apiConfig.host);
    }
    connectedCallback() {
        /**
         * Some stenciljs bug w/ this binding in event listeners, need to bind it correctly.
         */
        this.onModelUpdated = ((e) => {
            console.log("! this:", this);
            const unlocked = version$1.unlockConfigVersions(e.detail);
            this.contentConfig = unlocked;
        }).bind(this);
        if (!this.versions) {
            get(this.token, this.apiConfig.host).then((v) => (this.versions = v));
        }
    }
    componentDidLoad() {
        if (this.pieConfigEl) {
            this.pieConfigEl.VERSION = version$1.VERSION;
        }
    }
    getConfigParsed(sanctioned) {
        let configSettings = this.configSettings;
        if (this.configSettings) {
            const elName = Object.keys(this.configSettings)[0];
            configSettings = Object.assign(Object.assign({}, this.configSettings), { [elName]: Object.assign({ pieApi: {
                        token: this.token,
                        host: this.apiConfig.host,
                    } }, this.configSettings[elName]) });
        }
        else if (sanctioned.elements) {
            const value = Object.keys(sanctioned.elements).map((key) => sanctioned.elements[key])[0];
            const elName = `@${value.split("@")[1]}`;
            configSettings = {
                [elName]: {
                    pieApi: {
                        token: this.token,
                        host: this.apiConfig.host,
                    },
                },
            };
        }
        return configSettings;
    }
    render() {
        if (this.contentConfig && this.versions) {
            const is = this.getImageSupport();
            const us = this.getUploadSoundSupport();
            const sanctioned = version$1.lockConfigVersions(this.contentConfig, this.versions, this.getOverrides());
            const configSettings = this.getConfigParsed(sanctioned);
            return (core.h("pie-author", { onModelUpdated: this.onModelUpdated, bundleHost: this.host, defaultComplexRubricModel: this.defaultComplexRubricModel, config: sanctioned, configSettings: configSettings, addPreview: this.addPreview, imageSupport: is, uploadSoundSupport: us, ref: (el) => (this.pieConfigEl = el), isInsidePieApiAuthor: true, bundleEndpoints: this.bundleEndpoints, reFetchBundle: this.reFetchBundle }));
        }
        else {
            return core.h("div", null, "not loaded");
        }
    }
    get el() { return core.getElement(this); }
    static get watchers() { return {
        "host": ["watchHostEnvironment"],
        "token": ["onTokenChange"],
        "contentId": ["watchContentId"],
        "recordScreen": ["watchRecordScreen"]
    }; }
    static get style() { return ".pie-api-player img {\n  max-width: inherit;\n  max-height: inherit;\n}\n\n/* These styles are used in the body content of an item -- they also appear in the item body editor. */\n\n.book-title-k5 {\n  text-decoration: underline;\n}\n\n.book-title {\n  font-style: italic;\n}\n\n.block-quote {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.text-block {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.short-quote {\n  font-weight: bold;\n}\n\nspan.short-quote:before {\n  content: open-quote;\n}\n\nspan.short-quote:after {\n  content: close-quote;\n}\n\n.word-callout {\n  text-decoration: underline;\n}\n\n.relative-emphasis {\n  text-transform: uppercase;\n}\n\n.content-emphasis {\n  font-weight: bold;\n  color: red;\n}\n\nspan.passage-title:before {\n  content: open-quote;\n}\n\nspan.passage-title:after {\n  content: close-quote;\n}\n\n.proper-name {\n  font-style: italic;\n}\n\n.variable {\n  font-style: italic;\n}\n\n.equation-block {\n  text-align: center;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  padding: 2px;\n  display: block;\n}\n\n.embedded-error {\n  font-style: italic;\n}\n\n/* PASSAGES */\n\ndiv.passage-title,\ndiv.passage-subtitle {\n  font-weight: bold;\n  text-align: center;\n}\n\ndiv.passage-author {\n  text-align: center;\n}\n\n.p-number {\n  float: left;\n  font-size: 0.85em;\n}\n\n.numbered-paragraph {\n  margin-left: 36px;\n}\n\n/* ch5680 - also temporary kds content styles that need to be externalized */\n/* Inspect Styles */\n\n\@media print {\n  .kds-noprint {\n    display: none;\n  }\n\n  .noprint {\n    display: none;\n  }\n}\n\n.kds-absolute {\n  border-left: 1px solid black;\n  border-right: 1px solid black;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.abs {\n  border-left: #000000 1px solid;\n  border-right: #000000 1px solid;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.kds-border-1 {\n  border: 1px solid black;\n}\n\n.kds-border-2 {\n  border: 2px solid black;\n}\n\n.kds-center {\n  margin: 0px;\n  text-align: center;\n}\n\n.center {\n  text-align: center;\n  margin: 0;\n}\n\n.kds-exponent {\n  font-size: 85%;\n  margin-left: -0.4em;\n  position: relative;\n  top: -1.4em;\n}\n\nsup.frac {\n  position: relative;\n  top: -1.4em;\n  font-size: 70%;\n  margin-left: -0.4em;\n}\n\n.kds-fillin {\n  border: 1px solid black;\n  display: inline-block;\n  margin: 0em 0.2em;\n  padding-left: 1em;\n}\n\n.fillin {\n  margin: 0em 0.2em;\n  padding-left: 1em;\n  border: #000000 1px solid;\n  display: inline-block;\n  *display: inline;\n}\n\n.kds-font-larger {\n  font-size: larger;\n}\n\n.kds-font-smaller {\n  font-size: smaller;\n}\n\ntable.kds-fraction {\n  border: 0px;\n  display: inline-block;\n  font-size: 85%;\n  margin: 0px;\n  padding: 0px;\n  vertical-align: middle;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-numerator {\n  border-bottom: 1px solid black;\n  border-left: 0px;\n  border-right: 0px;\n  border-top: 0px;\n  text-align: center;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-denominator {\n  border: 0px;\n  text-align: center;\n}\n\ntable.frac {\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n  font-size: 85%;\n  border: 0px;\n  margin: 0px;\n  padding: 0px;\n}\n\n.frac .nu {\n  border-left: 0px;\n  border-top: 0px;\n  border-right: 0px;\n  border-bottom: #000000 1px solid;\n  text-align: center;\n}\n\n.frac .de {\n  border: 0px;\n  text-align: center;\n}\n\n.kds-hanging-indent {\n  margin: 0px;\n  padding-left: 3em;\n  text-indent: -3em;\n}\n\n.kds-image-caption {\n  font-family: Verdana, Arial, sans-serif;\n  text-align: center;\n}\n\n.kds-image-left {\n  float: left;\n  margin-right: 5px;\n}\n\n.kds-image-right {\n  float: right;\n  margin-left: 5px;\n}\n\n.kds-indent {\n  text-indent: 3em;\n}\n\n.indent {\n  text-indent: 3em;\n}\n\n.kds-nowrap {\n  display: inline-block;\n  white-space: nowrap;\n}\n\n.kds-overline {\n  text-decoration: overline;\n}\n\n.kds-parentheses {\n  font-family: \"Arial Narrow\", Arial, sans-serif;\n  font-size: 300%;\n  font-weight: normal;\n  position: relative;\n  top: 8px;\n}\n\nb.frac {\n  font-weight: normal;\n  font-size: 300%;\n  font-family: arial narrow;\n  position: relative;\n  top: +8px;\n}\n\n.kds-pi {\n  font-family: \"Times New Roman\", serif;\n  font-size: 1.2em;\n}\n\n.newradical {\n  border-collapse: collapse;\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n}\n\n.newradical td {\n  border: 0px;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical td.vinculum {\n  border-top: 1px solid black;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical div.exp {\n  font-size: 0.7em;\n  position: relative;\n  left: 7px;\n  bottom: 5px;\n}\n\n.kds-underline {\n  text-decoration: underline;\n}\n\n.under {\n  text-decoration: underline;\n}\n\n.kds-verdana2t {\n  border: 1px solid white;\n  text-align: left;\n  text-decoration: none;\n  vertical-align: top;\n}\n\n.Verdana2t {\n  vertical-align: top;\n  text-align: left;\n  text-decoration: none;\n  border: 1px solid white;\n}\n\n.kds-whole-number {\n  display: inline-block;\n  font-size: 120%;\n  margin: 0em 0em 0em 0.2em;\n}\n\n.whole {\n  display: inline-block;\n  *display: inline;\n  margin: 0em 0em 0em 0.2em;\n  font-size: 120%;\n}\n\ntable.KdsTable01 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable01 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > th.bold {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td.bold {\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable02 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable02 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n}\n\ntable.KdsTable02 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n}\n\n/* LEARNOSITY OVERRIDES */\n.lrn_feature h3 {\n  margin-top: 0 !important;\n}\n\n/* PIE OVERRIDES */\n\n#stimulus {\n  width: 50%;\n  float: left;\n}\n\n#item {\n  width: 50%;\n  float: left;\n}\n\n/* bootstrap table styles */\n.table {\n  width: 100%;\n  margin-bottom: 1em;\n  background-color: transparent\n}\n\nth {\n  text-align: inherit;\n}\n\n.table td,.table th {\n  padding: .75em;\n  vertical-align: top;\n  border-top: 1px solid #dee2e6\n}\n\n.table thead th {\n  vertical-align: bottom;\n  border-bottom: 2px solid #dee2e6\n}\n\n.table-bordered {\n  border: 1px solid #dee2e6\n}\n\n.table-bordered td,.table-bordered th {\n  border: 1px solid #dee2e6\n}\n\n.table-bordered thead td,.table-bordered thead th {\n  border-bottom-width: 2px\n}"; }
};

exports.pie_api_author = PieApiAuthor;
