'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const core = require('./core-8e67a5c9.js');

const PieStimulusLayout = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
        // shows if resizing is activated or not
        this.allowedResize = false;
        // shows if user is currently resizing the component,
        this.isResizing = false;
        // the initial X-coordinate of the mouse cursor.
        this.initialX = 0;
        //  flex values of the left component
        this.initialLeftFlex = 0.5;
        //  flex values of the right component as the user resizes them
        this.initialRightFlex = 0.5;
    }
    static handleElements() {
        const el = document.querySelector('#pie-stimulus-container');
        if (el) {
            const boundRect = el.getBoundingClientRect();
            el.style.height = `calc(100vh - ${boundRect.top}px`;
        }
    }
    componentDidRender() {
        PieStimulusLayout.handleElements();
    }
    componentDidLoad() {
        PieStimulusLayout.handleElements();
        if (this.allowedResize) {
            this.resizer.addEventListener('mousedown', (e) => {
                this.isResizing = true;
                this.initialX = e.clientX;
            });
            window.addEventListener('mousemove', this.handleMouseMove.bind(this));
            this.resizer.addEventListener('mouseup', this.handleMouseUp.bind(this));
        }
        this.readMoreButton.addEventListener("click", () => this.handleReadMore());
    }
    componentDidUpdate() {
        PieStimulusLayout.handleElements();
    }
    componentDidUnload() {
        if (this.allowedResize) {
            window.removeEventListener('mousemove', this.handleMouseMove.bind(this));
            this.resizer.removeEventListener('mouseup', this.handleMouseUp.bind(this));
        }
    }
    handleReadMore() {
        if (this.readMoreButton.innerHTML.includes('Read More')) {
            this.stimulus.style.flex = '1';
            this.readMoreButton.innerHTML = `Read Less <span class=arrow-up>&#9650;</span>`;
            this.stimulus.style.overflow = 'auto';
            this.stimulus.classList.remove('truncated');
        }
        else {
            this.readMoreButton.innerHTML = `Read More <span class=arrow-down>&#9660;</span>`;
            this.stimulus.style.flex = '0 0 30%';
            this.stimulus.style.overflow = 'hidden';
            this.stimulus.scrollTop = 0;
            this.stimulus.classList.add('truncated');
        }
    }
    handleMouseMove(e) {
        if (this.isResizing && this.allowedResize) {
            const deltaX = e.clientX - this.initialX;
            const containerWidth = this.resizer.parentElement.clientWidth;
            const newLeftFlex = (containerWidth * this.initialLeftFlex + deltaX) / containerWidth;
            const newRightFlex = 1 - newLeftFlex;
            if (newLeftFlex >= 0 && newRightFlex >= 0 && newLeftFlex <= 1 && newRightFlex <= 1) {
                this.initialLeftFlex = newLeftFlex;
                this.initialRightFlex = newRightFlex;
                // Update the initial cursor position
                this.initialX = e.clientX;
            }
        }
    }
    handleMouseUp() {
        this.isResizing = false;
    }
    // add style when the containers are resizable
    getStyle(flex) {
        const viewPortWidth = window.innerWidth;
        // avoid this style for viewports < 1020
        if (this.allowedResize && viewPortWidth > 1020) {
            return { flex: `${flex}`, transition: this.isResizing ? 'none' : 'flex 0.1s ease' };
        }
        return {};
    }
    render() {
        return (core.h("div", { id: "pie-stimulus-container" }, core.h("div", { id: "stimulus", class: "truncated", ref: (el) => this.stimulus = el, style: this.getStyle(this.initialLeftFlex) }, core.h("slot", { name: 'stimulus' })), core.h("button", { id: "read-more", ref: (el) => this.readMoreButton = el }, "Read More", core.h("span", { class: "arrow-down" }, "\u25BC")), this.allowedResize &&
            core.h("div", { id: "resizer", ref: (el) => (this.resizer = el) }), core.h("div", { id: "item", style: this.getStyle(this.initialRightFlex) }, core.h("slot", { name: 'item' }))));
    }
    static get style() { return "#pie-stimulus-container {\n  display: -ms-flexbox;\n  display: -webkit-flex;\n  display: flex;\n  -webkit-flex-direction: row;\n  -ms-flex-direction: row;\n  flex-direction: row;\n  -webkit-flex-wrap: nowrap;\n  -ms-flex-wrap: nowrap;\n  flex-wrap: nowrap;\n  -webkit-justify-content: flex-start;\n  -ms-flex-pack: start;\n  justify-content: flex-start;\n  -webkit-align-content: stretch;\n  -ms-flex-line-pack: stretch;\n  align-content: stretch;\n  -webkit-align-items: flex-start;\n  -ms-flex-align: start;\n  align-items: flex-start;\n  overflow: hidden;\n  }\n\n#pie-stimulus-container #stimulus {\n  -webkit-order: 0;\n  -ms-flex-order: 0;\n  order: 0;\n  -webkit-flex: 1;\n  -ms-flex: 1;\n  flex: 1;\n  -webkit-align-self: auto;\n  -ms-flex-item-align: auto;\n  align-self: auto;\n  height: 100%;\n  overflow: scroll;\n  }\n\n#pie-stimulus-container #read-more {\n  display: none;\n}\n\n#pie-stimulus-container #item {\n  -webkit-order: 0;\n  -ms-flex-order: 0;\n  order: 0;\n  -webkit-flex: 1;\n  -ms-flex: 1;\n  flex: 1;\n  -webkit-align-self: auto;\n  -ms-flex-item-align: auto;\n  align-self: auto;\n  height: 100%;\n  overflow: scroll;\n  }\n\n#pie-stimulus-container #resizer {\n  width: 5px;\n  cursor: ew-resize;\n  background-color: transparent;\n  height: 100%;\n  -webkit-transition: width 0.2s ease-in-out, background-color 0.2s ease-in-out;\n  transition: width 0.2s ease-in-out, background-color 0.2s ease-in-out;\n}\n\n#pie-stimulus-container #resizer:hover {\n  background-color: lightblue;\n  cursor: ew-resize;\n}\n\n#pie-stimulus-container #resizer:active {\n  background-color: lightblue;\n  cursor: ew-resize;\n}\n\n\n\@media screen and (max-width: 1020px) {\n  #pie-stimulus-container {\n    -ms-flex-direction: column;\n    flex-direction: column;\n  }\n\n  #resizer {\n    display: none;\n  }\n\n  #pie-stimulus-container #stimulus {\n    width: 100%;\n    overflow: hidden;\n    position: relative;\n    -ms-flex:0 0 30%;\n    flex:0 0 30%; /* stimulus represents 30% of the available space */\n  }\n  #pie-stimulus-container #item {\n    width: 100%;\n    overflow: auto;\n    -ms-flex:1;\n    flex:1;\n  }\n  #pie-stimulus-container #stimulusPlayer {\n    opacity: 1;\n    -webkit-transition: opacity 0.3s;\n    transition: opacity 0.3s;\n  }\n\n  #pie-stimulus-container #stimulus.truncated::after {\n    content: \"\";\n    position: absolute;\n    bottom: 0;\n    right: 0;\n    left: 0;\n    height: 50px; /* Adjust the height of the fade-out portion */\n    background: -webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0)), to(white));\n    background: linear-gradient(rgba(255, 255, 255, 0), white);\n  }\n\n  #pie-stimulus-container #read-more {\n    display: block;\n    margin-top: 10px;\n    cursor: pointer;\n    -ms-flex-item-align: center;\n    align-self: center;\n    border: none;\n    background-color: transparent;\n    color: cornflowerblue;\n  }\n\n}"; }
};

exports.pie_stimulus_layout = PieStimulusLayout;
