import { makeGuid, isPreRelease } from "../util";
export const importItemGql = (id, collections) => {
    const collectionsGql = collections && collections instanceof Array && collections.length > 0
        ? `collectionIds: ${collections && JSON.stringify(collections)}`
        : "";
    return `
    mutation importContentItem($config: JSON) {
      importContentItem(
        input: {
          id: "${id ? id.id : makeGuid()}"
          ${collectionsGql}
          config: $config
        }
      ) {
        id
        name
        version {
          major
          minor
          patch
          prerelease
        }
      }
    }`;
};
export const saveContentItemGql = (id) => {
    const version = id && id.version
        ? `version: {
  major: ${id ? id.version.major : 0}
  minor: ${id ? id.version.minor : 0}
  patch: ${id ? id.version.patch : 0},
  prerelease: {
    tag: "${id ? id.version.prerelease.tag : "draft"}"
    version: ${id ? id.version.prerelease.version : 0}
  }
}`
        : `version: {
    major: 0
    minor: 0
    patch: 0,
    prerelease: {
      tag: "draft"
      version: 0
    }
  }`;
    return `
  mutation save($config: JSON) {
    saveContentItem(
        id: "${id ? id.id : null}"
        ${version}
        contentItem: {
          config: $config
        }
    ) {
      id
      version {
        major
        minor
        patch
        prerelease
      }
    }
  }
  `;
};
export const saveContentItemRevisionGql = (id) => {
    const version = id.version
        ? `version: {
      major: ${id ? id.version.major : 0}
      minor: ${id ? id.version.minor : 0}
      patch: ${id ? id.version.patch : 0},
      prerelease: {
        tag: "${id ? id.version.prerelease.tag : "draft"}"
        version: ${id ? id.version.prerelease.version : 0}
      }
    }`
        : "";
    return `
  mutation saveContentItemRevision($config: JSON) {
    saveContentItemRevision(
      id: "${id ? id.id : null}"
      ${version}
      contentItem: {
        config: $config
      }
    ) {
      id
      version {
        major
        minor
        patch
        prerelease
      }
    }
  }`;
};
export const releaseContentItemGql = (id) => {
    if (isPreRelease(id)) {
        const version = `version: {
        major: ${id.version.major}
        minor: ${id.version.minor}
        patch: ${id.version.patch},
        prerelease: {
          tag: "${id.version.prerelease.tag}"
          version: ${id.version.prerelease.version}
        }
      }`;
        return `
      mutation releaseContentItem {
        releaseContentItem(
          id: "${id.id}"
          ${version}
          )
      {
        id
        version {
          major
          minor
          patch
          prerelease
        }
      }
    }
    `;
    }
    else {
        throw new Error("item version not present or item version is not a pre-release version");
    }
};
