const resHeaders = {
    "Access-Control-Allow-Origin": "*",
    "Access-Control-Allow-Methods": "DELETE, POST, GET, OPTIONS",
    "Access-Control-Allow-Headers": "Content-Type, Access-Control-Allow-Headers, Authorization, X-Requested-With",
};
const bodify = (s) => {
    if (Buffer.isBuffer(s)) {
        return s;
    }
    return typeof s === "string" ? s : JSON.stringify(s);
};
export class PageRequestInterceptor {
    constructor(page) {
        this.interceptors = new Array();
        page.on("request", (request) => {
            let handled = false;
            this.interceptors.forEach((icp) => {
                const gm = this.getGqlOperation(request, icp.gqlOperationMatch);
                if (request.url().match(icp.match) || (gm && !handled)) {
                    handled = true;
                    let bodyResponse;
                    if (gm) {
                        bodyResponse = gm.body;
                    }
                    else {
                        bodyResponse = icp.body ? icp.body : "";
                    }
                    const body = bodify(bodyResponse);
                    request.respond({
                        status: icp.status ? icp.status : 200,
                        headers: icp.headers ? icp.headers : resHeaders,
                        contentType: icp.contenType ? icp.match : "application/javascript",
                        body,
                    });
                }
            });
            if (!handled) {
                request.continue();
            }
        });
        page.setRequestInterception(true);
    }
    addInterceptor(interceptor) {
        this.interceptors.push(interceptor);
    }
    setInterceptor(interceptor) {
        const toChange = this.interceptors.find((icp) => icp.match === interceptor.match);
        if (!toChange) {
            this.interceptors.push(interceptor);
        }
        else {
            toChange.match = interceptor.match;
            toChange.body = interceptor.body;
            toChange.contenType = interceptor.contenType;
            toChange.headers = interceptor.headers;
            toChange.status = interceptor.status;
            toChange.gqlOperationMatch = interceptor.gqlOperationMatch;
        }
    }
    getGqlOperation(request, operationMatches) {
        try {
            const rBody = request.postData();
            return (rBody &&
                operationMatches &&
                operationMatches.find((opm) => !!rBody.match(opm.match)));
        }
        catch (err) {
            console.dir(err);
            console.log(`[graphql] got error parsing`);
            return false;
        }
    }
}
