
const HOST = {
  stage: {
    mode: 'stage',
    apiUrl: 'https://stage.pie-api.io/services',
    token:'TOKEN_HERE'
  },
  dev: {
    mode: 'dev',
    apiUrl: 'https://develop.pie-api.io/services',
    token:'TOKEN_HERE'
  }
}

const api = HOST.stage;


export const findItems = async (
  token,
  whereParams,
  apiUrl
) => {
  const query = `
  query contentItems($where: MetaDataSearchInput!){
    contentItems(where: $where) {
      edges {
        node {
          contentItem {
            id
            config
          }
        }
      }
    }
  }
  `
  const res = await executeGql(token, query, {where: whereParams}, apiUrl);
  return res && res.data ? res.data : null;
};


const executeGql = async (token, query, variables, apiUrl) => {
  const graphQlApi = apiUrl + '/graphql';
  try {
    const res = await fetch(graphQlApi, {
      method: 'POST',
      mode: 'cors',
      headers: {
        'Content-Type': 'application/json',
        'Accept-Encoding': 'gzip, deflate, br',
        Authorization: `Bearer ${token}`,
        Accept: 'application/json'
      },
      body: JSON.stringify({
        query,
        variables
      })
    });
    const jsonBody = await res.json();
    return jsonBody;
  } catch (err) {
    throw(new Error('error with server request'));
  }
};

const addPlayer = (item, div) => {
  const header = document.createElement("p");
  header.textContent = `${item.id}`;
  div.appendChild(header);
  const player = document.createElement('pie-player');
  player.bundleHost = api.mode;
  player.addCorrectResponse = true;
  player.config = item.config;
  player.env = {mode: 'view'};
  div.appendChild(player);
}

// alternative, that loads by id - less efficient
const addApiPlayer = (item, div) => {
  const player = document.createElement('pie-api-player');
  player.token = api.token;
  player.host = api.mode;
  player.preview = true;
  player.initialMode = 'view';
  player.itemId = item.id;
  player.renderStimulus = false;
  div.appendChild(player);
}
  const itemList = document.getElementById('itemList');
  const res = findItems(api.token, {pies_some:["@pie-element/multiple-choice"]}, api.apiUrl).then(res => {
    res.contentItems.edges.forEach(edge => {

     
      addPlayer(edge.node.contentItem, itemList);
      // addApiPlayer(edge.node.contentItem, itemList);

    });
  });



   
