import { loadBestVersions } from "./clients/graphql";
const log = () => ({});
// const log: any = console.log.bind(console, "[pie-versions]");
let versions;
/**
 * Use these as a fallback if all else fails. To be removed.
 */
const LegacyPieVersions = {
    "@pie-element/categorize": "6.5.4",
    "@pie-element/charting": "2.5.4",
    "@pie-element/drag-in-the-blank": "3.7.4",
    "@pie-element/drawing-response": "3.5.3",
    "@pie-element/ebsr": "6.3.4",
    "@pie-element/explicit-constructed-response": "3.7.4",
    "@pie-element/extended-text-entry": "6.3.3",
    "@pie-element/graph-lines": "4.0.7",
    "@pie-element/graphing": "3.4.4",
    "@pie-element/hotspot": "4.7.4",
    "@pie-element/image-cloze-association": "3.4.4",
    "@pie-element/inline-choice": "2.10.9",
    "@pie-element/inline-dropdown": "3.5.4",
    "@pie-element/match": "4.5.4",
    "@pie-element/match-list": "1.11.4",
    "@pie-element/math-inline": "5.4.4",
    "@pie-element/multiple-choice": "5.5.4",
    "@pie-element/number-line": "5.9.4",
    "@pie-element/placement-ordering": "7.3.4",
    "@pie-element/point-intercept": "4.5.8",
    "@pie-element/rubric": "1.5.2",
    "@pie-element/ruler": "3.11.3",
    "@pie-element/select-text": "6.3.4",
    "@pie-element/text-entry": "5.0.7",
};
export const noOlderThan = (n, minutes) => {
    const diff = new Date().getTime() - n;
    log("age in millis", diff);
    return diff < minutes * 1000 * 60;
};
export const KEY = "__pie_api_components_author_versions__";
const localVersions = () => {
    try {
        const local = localStorage.getItem(KEY);
        const o = JSON.parse(local);
        if (!o) {
            return;
        }
        if (noOlderThan(o.createdAt, 60)) {
            return o.versions;
        }
    }
    catch (e) {
        console.error(e.message);
    }
};
export const get = async (token, apiUrl) => {
    try {
        log("get, token", token, "apiUrl:", apiUrl);
        if (!versions) {
            const local = localVersions();
            log("get - local:", local);
            if (local) {
                versions = local;
            }
            else {
                // console.log("get - loadBestVersions ", apiUrl);
                const r = await loadBestVersions(token, apiUrl);
                versions = r.reduce((acc, nv) => {
                    acc[nv.pie] = nv.version;
                    return acc;
                }, {});
                log("versions back from server:", versions);
                localStorage.setItem(KEY, JSON.stringify({ versions, createdAt: new Date().getTime() }));
            }
        }
        log("get - return versions");
        return versions;
    }
    catch (e) {
        console.warn(`[pie-versions:get] ${apiUrl} - Error loading best versions - returning fallback versions, this will throw an error in future`);
        console.error(e.message);
        return LegacyPieVersions;
    }
};
