var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { c as cloneDeep } from './rubric-utils-61de23c8.js';
import { c as createCommonjsModule, p as parseNpm } from './utils-e506e807.js';
/**
 * New Relic Instrumentation Service
 *
 * A TypeScript service that provides a wrapper around New Relic browser APIs.
 * This service uses lazy initialization to access New Relic only when needed,
 * ensuring that it works even if the host application initializes New Relic
 * after this service is instantiated - or not at all.
 */
var NewRelicInstrumentationService = /** @class */ (function () {
    /**
     * Creates an instance of NewRelicInstrumentationService.
     * @param document - The Document object, used to access the window
     */
    function NewRelicInstrumentationService(document) {
        this.document = document;
    }
    /**
     * Gets the New Relic instance from the window object
     * This is called lazily when needed, not at initialization time
     * @returns The New Relic instance or undefined if not available
     */
    NewRelicInstrumentationService.prototype.getNewRelic = function () {
        try {
            var window = this.document.defaultView;
            return window && window.NREUM ? window.NREUM : undefined;
        }
        catch (error) {
            console.warn('Failed to access New Relic:', error);
            return undefined;
        }
    };
    /**
     * Check if New Relic is currently available
     * @returns boolean indicating whether New Relic is available
     */
    NewRelicInstrumentationService.prototype.isAvailable = function () {
        return !!this.getNewRelic();
    };
    // ==============================
    // Traces and Events Methods
    // ==============================
    /**
     * Add a page action (custom event) to New Relic
     * @param eventName - Name of the event
     * @param attributes - Attributes associated with the event
     */
    NewRelicInstrumentationService.prototype.addPageAction = function (eventName, attributes) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.addPageAction(eventName, attributes);
        }
    };
    /**
     * Add information to an in-progress session trace
     * @param attribute - The attribute to add to the trace
     */
    NewRelicInstrumentationService.prototype.addToTrace = function (attribute) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.addToTrace(attribute);
        }
    };
    /**
     * Record a time point when the page is finished according to custom criteria
     */
    NewRelicInstrumentationService.prototype.finished = function () {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.finished();
        }
    };
    /**
     * Add a custom attribute to subsequent events on the page
     * @param key - The attribute name
     * @param value - The attribute value
     */
    NewRelicInstrumentationService.prototype.setCustomAttribute = function (key, value) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setCustomAttribute(key, value);
        }
    };
    /**
     * Add application version information to subsequent events
     * @param version - The application version
     */
    NewRelicInstrumentationService.prototype.setApplicationVersion = function (version) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setApplicationVersion(version);
        }
    };
    /**
     * Group page views by setting a page name
     * @param name - The page name
     */
    NewRelicInstrumentationService.prototype.setPageViewName = function (name) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setPageViewName(name);
        }
    };
    /**
     * Add user ID information to subsequent events on the page
     * @param userId - The user ID
     */
    NewRelicInstrumentationService.prototype.setUserId = function (userId) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setUserId(userId);
        }
    };
    /**
     * Capture a single browser log event
     * @param message - The log message
     * @param level - Log level (optional)
     * @param context - Additional context (optional)
     */
    NewRelicInstrumentationService.prototype.log = function (message, level, context) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.log(message, level, context);
        }
    };
    /**
     * Wrap an existing logger to capture messages as log events
     * @param logger - The logger to wrap
     */
    NewRelicInstrumentationService.prototype.wrapLogger = function (logger) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.wrapLogger(logger);
        }
    };
    // ==============================
    // Error Reporting Methods
    // ==============================
    /**
     * Tag errors with your app's version information
     * @param releaseName - The release name/version
     */
    NewRelicInstrumentationService.prototype.addRelease = function (releaseName) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.addRelease(releaseName);
        }
    };
    /**
     * Capture a caught or handled error without disrupting the app's operation
     * @param error - The error object
     * @param customAttributes - Optional custom attributes
     */
    NewRelicInstrumentationService.prototype.noticeError = function (error, customAttributes) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.noticeError(error, customAttributes);
        }
    };
    /**
     * Set a custom error handler to ignore specific errors
     * @param errorHandler - Function that returns true if the error should be ignored
     */
    NewRelicInstrumentationService.prototype.setErrorHandler = function (errorHandler) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.setErrorHandler(errorHandler);
        }
    };
    // ==============================
    // SPA Tracking Methods
    // ==============================
    /**
     * Return a new API object bound to the current interaction
     * If New Relic is not available, returns a no-op implementation
     */
    NewRelicInstrumentationService.prototype.interaction = function () {
        var _this = this;
        var newrelic = this.getNewRelic();
        if (newrelic) {
            return newrelic.interaction();
        }
        // Return a no-op implementation to prevent errors
        return {
            actionText: function () { return _this; },
            createTracer: function () { return function () {
            }; },
            end: function () {
            },
            getContext: function () { return ({}); },
            ignore: function () {
            },
            onEnd: function () { return _this; },
            save: function () { return _this; },
            setAttribute: function () { return _this; },
            setCurrentRouteName: function () { return _this; }
        };
    };
    /**
     * Set the actionText of a SPA interaction
     * @param text - The text of the HTML element clicked
     */
    NewRelicInstrumentationService.prototype.setInteractionActionText = function (text) {
        var interaction = this.interaction();
        interaction.actionText(text);
    };
    /**
     * End a SPA interaction
     */
    NewRelicInstrumentationService.prototype.endInteraction = function () {
        var interaction = this.interaction();
        interaction.end();
    };
    /**
     * Add a custom attribute to the current interaction
     * @param key - The attribute name
     * @param value - The attribute value
     */
    NewRelicInstrumentationService.prototype.setInteractionAttribute = function (key, value) {
        var interaction = this.interaction();
        interaction.setAttribute(key, value);
    };
    /**
     * Give SPA routes more accurate names than default names
     * @param routeName - The route name
     */
    NewRelicInstrumentationService.prototype.setCurrentRouteName = function (routeName) {
        var interaction = this.interaction();
        interaction.setCurrentRouteName(routeName);
    };
    /**
     * Ignore the current interaction
     */
    NewRelicInstrumentationService.prototype.ignoreInteraction = function () {
        var interaction = this.interaction();
        interaction.ignore();
    };
    /**
     * Save the current interaction
     */
    NewRelicInstrumentationService.prototype.saveInteraction = function () {
        var interaction = this.interaction();
        interaction.save();
    };
    // ==============================
    // Agent Control Methods
    // ==============================
    /**
     * Start harvesting one or more features
     * @param features - Array of features to start
     */
    NewRelicInstrumentationService.prototype.start = function (features) {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.start(features);
        }
    };
    /**
     * Manually start session replay collection
     */
    NewRelicInstrumentationService.prototype.recordReplay = function () {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.recordReplay();
        }
    };
    /**
     * Manually pause session replay collection
     */
    NewRelicInstrumentationService.prototype.pauseReplay = function () {
        var newrelic = this.getNewRelic();
        if (newrelic) {
            newrelic.pauseReplay();
        }
    };
    return NewRelicInstrumentationService;
}());
/**
 * Base class for clients that need New Relic instrumentation
 *
 * This abstract class provides common New Relic initialization and tracking
 * functionality that can be reused across different client implementations.
 */
var NewRelicEnabledClient = /** @class */ (function () {
    /**
     * Constructor initializes New Relic automatically
     */
    function NewRelicEnabledClient() {
        var _a;
        this.newRelic = null;
        // Initialize New Relic on construction
        try {
            this.newRelic = new NewRelicInstrumentationService(document);
            if (!((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable())) {
                console.warn('New Relic is initialized but not available');
                this.newRelic = null;
            }
        }
        catch (error) {
            console.warn('Failed to initialize New Relic instrumentation:', error);
            this.newRelic = null;
        }
    }
    /**
     * Track API activity or errors in New Relic
     */
    NewRelicEnabledClient.prototype.track = function (type, name, attributes, error) {
        if (!this.newRelic)
            return;
        var commonAttrs = Object.assign(Object.assign(Object.assign({}, attributes), { timestamp: new Date().toISOString() }), this.getTrackingBaseAttributes());
        if (type === 'activity') {
            this.newRelic.addPageAction(name, commonAttrs);
        }
        else if (error) {
            this.newRelic.noticeError(error, commonAttrs);
        }
    };
    /**
     * Get network information for diagnostics
     */
    NewRelicEnabledClient.prototype.getNetworkInfo = function () {
        var info = {
            online: typeof navigator !== 'undefined' && navigator.onLine
        };
        if (typeof navigator !== 'undefined' && 'connection' in navigator) {
            var connection = navigator.connection;
            if (connection) {
                info.effectiveType = connection.effectiveType || 'unknown';
                info.downlink = connection.downlink || 'unknown';
                info.rtt = connection.rtt || 'unknown';
                info.saveData = connection.saveData || false;
            }
        }
        return info;
    };
    return NewRelicEnabledClient;
}());
var jsonHeaders = function () { return ({
    Accept: "application/json",
    "Content-Type": "application/json",
}); };
var Mode;
(function (Mode) {
    Mode["view"] = "view";
    Mode["gather"] = "gather";
    Mode["evaluate"] = "evaluate";
})(Mode || (Mode = {}));
var Role;
(function (Role) {
    Role["student"] = "student";
    Role["instructor"] = "instructor";
})(Role || (Role = {}));
var __rest = (undefined && undefined.__rest) || function (s, e) {
    var t = {};
    for (var p in s)
        if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
            t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
// Default retry configuration values
var DEFAULT_RETRY_CONFIG = {
    maxRetryAttempts: 2,
    requestTimeoutMs: 10000,
    initialRetryDelayMs: 500,
    maxRetryDelayMs: 2000,
    trackPageActions: false
};
/**
 * Shared fetch utility class that provides retry capability
 * Can be used standalone or extended by other clients
 */
var FetchUtils = /** @class */ (function (_super) {
    __extends(FetchUtils, _super);
    function FetchUtils(trackingInfo, config) {
        if (trackingInfo === void 0) { trackingInfo = {}; }
        var _this = _super.call(this) || this;
        _this.trackingInfo = trackingInfo;
        _this.retryConfig = Object.assign(Object.assign({}, DEFAULT_RETRY_CONFIG), config);
        return _this;
    }
    /**
     * Implementation of the abstract method from NewRelicEnabledClient
     */
    FetchUtils.prototype.getTrackingBaseAttributes = function () {
        return Object.assign(Object.assign({}, this.trackingInfo), { networkInfo: this.getNetworkInfo() });
    };
    /**
     * Create a sanitized sample of request data that's safe to log
     */
    FetchUtils.prototype.createSanitizedDataSample = function (data) {
        if (!data || typeof data !== 'object')
            return {};
        var result = {};
        Object.keys(data).forEach(function (key) {
            var value = data[key];
            result[key] = typeof value === 'string'
                ? (value.length > 50 ? value.substring(0, 50) + '...' : value)
                : typeof value;
        });
        return result;
    };
    /**
     * Check if an error is a network error
     */
    FetchUtils.prototype.isNetworkError = function (error) {
        return (error.name === "AbortError" ||
            error instanceof TypeError ||
            error.message.includes("NetworkError") ||
            error.message.includes("Failed to fetch") ||
            error.message.includes("Network request failed") ||
            !(typeof navigator !== 'undefined' && navigator.onLine));
    };
    /**
     * Track API call start if trackPageActions is enabled
     */
    FetchUtils.prototype.trackOperationStart = function (endpointName, operationType, dataSample) {
        if (operationType === void 0) { operationType = 'request'; }
        if (dataSample === void 0) { dataSample = {}; }
        if (!this.retryConfig.trackPageActions || !this.newRelic)
            return;
        this.track('activity', "api_" + operationType + "_started", {
            endpoint: endpointName,
            dataSample: JSON.stringify(dataSample),
            timestamp: new Date().toISOString()
        });
    };
    /**
     * Track API call complete if trackPageActions is enabled
     */
    FetchUtils.prototype.trackOperationComplete = function (operationType, endpointName, startTime) {
        if (this.retryConfig.trackPageActions && operationType !== 'request') {
            this.track('activity', "api_" + operationType + "_complete", {
                endpoint: endpointName,
                duration: Date.now() - startTime,
                success: true
            });
        }
    };
    /**
     * Track API call completion
     */
    FetchUtils.prototype.trackRequestComplete = function (endpointName, attempt, response, duration, responseData) {
        // Always track retries and failures
        if (attempt > 0 || !response.ok) {
            // This will track retry attempts (success or failure) and all failures
            this.track('activity', "api_request_complete", {
                apiEndpoint: endpointName,
                attempt: attempt,
                status: response.status,
                duration: duration,
                success: response.ok,
                outcome: response.ok
                    ? "retry_success" // We know attempt > 0 here
                    : (attempt >= this.retryConfig.maxRetryAttempts ? "non_recoverable_failure" : "failure_will_retry")
            });
        }
        // Track successful first attempts when trackPageActions is enabled
        else if (this.retryConfig.trackPageActions) {
            // This is a first attempt success (attempt === 0 && response.ok)
            var responseSample = responseData
                ? this.createSanitizedDataSample(responseData)
                : {};
            this.track('activity', "api_request_complete", {
                endpoint: endpointName,
                status: response.status,
                duration: duration,
                outcome: "first_attempt_success",
                responseSample: JSON.stringify(responseSample)
            });
        }
    };
    /**
     * Track retry attempt
     */
    FetchUtils.prototype.trackRetryAttempt = function (endpointName, attempt, dataSample) {
        this.track('activity', "api_retry_attempt", Object.assign({ endpoint: endpointName, attempt: attempt }, (dataSample && { dataSample: JSON.stringify(dataSample) })));
    };
    /**
     * Track error
     */
    FetchUtils.prototype.trackError = function (error, attributes, responseInfo) {
        var errorType = responseInfo ? "api_error" : "api_final_error";
        var finalAttempt = attributes.finalAttempt, isNetworkError = attributes.isNetworkError, customAttributes = __rest(attributes
        /**
         * Transient (retryable) errors:
         * - 400 Request Timeout
         * - 429 Too Many Requests
         * - 500–502, 503, 504 Server Errors (sometimes recoverable)
         * - Network errors
         */
        , ["finalAttempt", "isNetworkError"]);
        /**
         * Transient (retryable) errors:
         * - 400 Request Timeout
         * - 429 Too Many Requests
         * - 500–502, 503, 504 Server Errors (sometimes recoverable)
         * - Network errors
         */
        var recoveryStatus = responseInfo && [408, 429, 500, 502, 503, 504].includes(responseInfo.status) || !!isNetworkError
            ? 'transient_recoverable'
            : 'non_recoverable';
        var errorData = Object.assign(Object.assign({}, customAttributes), { finalAttempt: !!finalAttempt, errorName: error.name, errorMessage: error.message, errorType: errorType,
            recoveryStatus: recoveryStatus, stackTrace: error.stack });
        if (responseInfo) {
            Object.assign(errorData, Object.assign({ status: responseInfo.status, statusText: responseInfo.statusText }, (responseInfo.responseData && { responseData: responseInfo.responseData })));
        }
        else {
            Object.assign(errorData, {
                isNetworkError: !!isNetworkError,
            });
        }
        if (recoveryStatus === 'transient_recoverable') {
            this.track('activity', 'transient_error', errorData, error);
        }
        else {
            this.track('error', errorType, errorData, error);
        }
    };
    /**
     * Calculate exponential backoff wait time for retries
     */
    FetchUtils.prototype.calculateBackoffTime = function (attempt) {
        return Math.min(this.retryConfig.initialRetryDelayMs * Math.pow(2, attempt), this.retryConfig.maxRetryDelayMs);
    };
    /**
     * Wait for network connectivity if offline
     */
    FetchUtils.prototype.waitForConnectivity = function (endpointName, attempt) {
        return __awaiter(this, void 0, void 0, function () {
            var waitTime;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!(typeof navigator !== 'undefined' && !navigator.onLine && typeof window !== 'undefined')) return [3 /*break*/, 2];
                        this.track('activity', "api_offline_retry", {
                            endpoint: endpointName,
                            attempt: attempt,
                            waitingForConnectivity: true
                        });
                        return [4 /*yield*/, new Promise(function (resolve) {
                                var onlineHandler = function () {
                                    window.removeEventListener('online', onlineHandler);
                                    resolve();
                                };
                                setTimeout(function () {
                                    window.removeEventListener('online', onlineHandler);
                                    resolve();
                                }, _this.retryConfig.maxRetryDelayMs);
                                window.addEventListener('online', onlineHandler);
                            })];
                    case 1:
                        _b.sent();
                        return [2 /*return*/];
                    case 2:
                        waitTime = this.calculateBackoffTime(attempt);
                        console.warn("Retrying " + endpointName + " (attempt " + (attempt + 1) + "/" + (this.retryConfig.maxRetryAttempts + 1) + ") after " + waitTime + "ms...");
                        this.track('activity', "api_retry_scheduled", {
                            endpoint: endpointName,
                            attempt: attempt,
                            waitTime: waitTime
                        });
                        return [4 /*yield*/, new Promise(function (resolve) { return setTimeout(resolve, waitTime); })];
                    case 3:
                        _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Core retry logic for any fetch operation
     */
    FetchUtils.prototype.executeWithRetry = function (operation, endpointName, operationType, dataSample) {
        if (operationType === void 0) { operationType = 'request'; }
        return __awaiter(this, void 0, void 0, function () {
            var startTime, attempt, result, error_1, typedError, isNetworkError;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        startTime = Date.now();
                        // Track start if enabled
                        this.trackOperationStart(endpointName, operationType, dataSample);
                        attempt = 0;
                        _b.label = 1;
                    case 1:
                        if (!(attempt <= this.retryConfig.maxRetryAttempts)) return [3 /*break*/, 7];
                        _b.label = 2;
                    case 2:
                        _b.trys.push([2, 4, , 6]);
                        // Track retry attempts
                        if (attempt > 0) {
                            this.trackRetryAttempt(endpointName, attempt, dataSample);
                        }
                        return [4 /*yield*/, operation(attempt)];
                    case 3:
                        result = _b.sent();
                        // If we're here, the operation succeeded
                        this.trackOperationComplete(operationType, endpointName, startTime);
                        return [2 /*return*/, result];
                    case 4:
                        error_1 = _b.sent();
                        typedError = error_1 instanceof Error ? error_1 : new Error(String(error_1));
                        isNetworkError = this.isNetworkError(typedError);
                        // If it's the last attempt or not a network error, throw
                        if (attempt >= this.retryConfig.maxRetryAttempts || !isNetworkError) {
                            this.trackError(typedError, {
                                endpoint: endpointName,
                                attempt: attempt,
                                duration: Date.now() - startTime,
                                finalAttempt: attempt === this.retryConfig.maxRetryAttempts,
                                isNetworkError: isNetworkError
                            });
                            throw typedError;
                        }
                        // Wait before retrying
                        return [4 /*yield*/, this.waitForConnectivity(endpointName, attempt)];
                    case 5:
                        // Wait before retrying
                        _b.sent();
                        return [3 /*break*/, 6];
                    case 6:
                        attempt++;
                        return [3 /*break*/, 1];
                    case 7: 
                    // Should never reach here
                    throw new Error("Operation " + endpointName + " failed after " + (this.retryConfig.maxRetryAttempts + 1) + " attempts.");
                }
            });
        });
    };
    /**
     * Make an API call with configurable retry logic
     *
     * @param url - Full URL to fetch
     * @param options - Request options (method, headers, body, etc.)
     * @param endpointName - Name of endpoint for tracking purposes
     * @returns A promise that resolves to the parsed response data
     */
    FetchUtils.prototype.fetchWithRetry = function (url, options, endpointName) {
        return __awaiter(this, void 0, void 0, function () {
            var timeoutId, controller, hasAbortController, requestBody, dataSample;
            var _this = this;
            return __generator(this, function (_b) {
                hasAbortController = typeof AbortController !== 'undefined';
                requestBody = options.body ?
                    (typeof options.body === 'string' ? JSON.parse(options.body) : options.body) :
                    undefined;
                dataSample = this.createSanitizedDataSample(requestBody);
                return [2 /*return*/, this.executeWithRetry(function (attempt) { return __awaiter(_this, void 0, void 0, function () {
                        var _a, startTime, response, responseData, responseText, e_1, e_2, error;
                        return __generator(this, function (_b) {
                            switch (_b.label) {
                                case 0:
                                    startTime = Date.now();
                                    // Clear any previous timeout
                                    if (timeoutId) {
                                        clearTimeout(timeoutId);
                                        timeoutId = undefined;
                                    }
                                    // Set up abort controller for timeouts (if available)
                                    if (hasAbortController) {
                                        controller = new AbortController();
                                        timeoutId = window.setTimeout(function () {
                                            var _a;
                                            (_a = controller) === null || _a === void 0 ? void 0 : _a.abort();
                                        }, this.retryConfig.requestTimeoutMs);
                                        // Add signal to options
                                        options.signal = controller.signal;
                                    }
                                    _b.label = 1;
                                case 1:
                                    _b.trys.push([1, , 11, 12]);
                                    return [4 /*yield*/, fetch(url, options)];
                                case 2:
                                    response = _b.sent();
                                    // Clear timeout after response
                                    if (timeoutId) {
                                        clearTimeout(timeoutId);
                                        timeoutId = undefined;
                                    }
                                    responseData = {};
                                    responseText = '';
                                    if (!(response.status !== 204)) return [3 /*break*/, 10];
                                    _b.label = 3;
                                case 3:
                                    _b.trys.push([3, 9, , 10]);
                                    if (!(typeof response.clone === 'function')) return [3 /*break*/, 5];
                                    return [4 /*yield*/, response.clone().text()];
                                case 4:
                                    responseText = _b.sent();
                                    return [3 /*break*/, 8];
                                case 5:
                                    _b.trys.push([5, 7, , 8]);
                                    return [4 /*yield*/, response.text()];
                                case 6:
                                    // Try to get text directly if possible
                                    responseText = _b.sent();
                                    return [3 /*break*/, 8];
                                case 7:
                                    e_1 = _b.sent();
                                    // If that fails too, try to access responseText directly (for mock responses)
                                    responseText = response.bodyUsed ? '' : response.body || '';
                                    return [3 /*break*/, 8];
                                case 8:
                                    if (responseText && responseText.trim()) {
                                        try {
                                            responseData = JSON.parse(responseText);
                                        }
                                        catch (e) {
                                            responseData = { error: "Parse error: " + e.message, rawResponse: responseText };
                                        }
                                    }
                                    return [3 /*break*/, 10];
                                case 9:
                                    e_2 = _b.sent();
                                    console.warn("Failed to parse response as JSON: " + e_2);
                                    return [3 /*break*/, 10];
                                case 10:
                                    this.trackRequestComplete(endpointName, attempt, response, Date.now() - startTime, responseData);
                                    if (response.ok) {
                                        return [2 /*return*/, responseData];
                                    }
                                    error = new Error("Call to " + endpointName + " failed. " + (((_a = responseData) === null || _a === void 0 ? void 0 : _a.error) || 'HTTP ' + response.status));
                                    if (attempt >= this.retryConfig.maxRetryAttempts) {
                                        this.trackError(error, {
                                            endpoint: endpointName,
                                            attempt: attempt,
                                            duration: Date.now() - startTime,
                                            finalAttempt: true,
                                            isNetworkError: false
                                        }, {
                                            status: response.status,
                                            statusText: response.statusText,
                                            responseData: typeof responseData === 'object' ? JSON.stringify(responseData) : responseText
                                        });
                                    }
                                    throw error;
                                case 11:
                                    // Clear any timeout
                                    if (timeoutId) {
                                        clearTimeout(timeoutId);
                                        timeoutId = undefined;
                                    }
                                    return [7 /*endfinally*/];
                                case 12: return [2 /*return*/];
                            }
                        });
                    }); }, endpointName, 'request', dataSample)];
            });
        });
    };
    /**
     * Convenience method for API calls with standard JSON headers and authorization
     */
    FetchUtils.prototype.callApi = function (url, method, token, data, customHeaders) {
        return __awaiter(this, void 0, void 0, function () {
            var headers, fetchOptions;
            return __generator(this, function (_b) {
                headers = Object.assign(Object.assign(Object.assign({}, jsonHeaders()), { "Authorization": "Bearer " + token, "x-date": new Date().getTime().toString(), "x-request-id": "" + Date.now() }), customHeaders);
                fetchOptions = {
                    method: method.toUpperCase(),
                    mode: "cors",
                    headers: headers,
                    body: data !== undefined ? JSON.stringify(data) : undefined,
                };
                return [2 /*return*/, this.fetchWithRetry(url, fetchOptions, method.toUpperCase() + " " + url)];
            });
        });
    };
    /**
     * Convenience method for GraphQL requests
     */
    FetchUtils.prototype.executeGraphQL = function (endpoint, token, query, variables) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_b) {
                return [2 /*return*/, this.callApi(endpoint, 'POST', token, {
                        query: query,
                        variables: variables
                    }, {
                        "Accept-Encoding": "gzip, deflate, br",
                        "Accept": "application/json"
                    })];
            });
        });
    };
    /**
     * Convenience method for file uploads with direct signed URLs
     */
    FetchUtils.prototype.uploadFile = function (signedURL, file, endpointName) {
        if (endpointName === void 0) { endpointName = "file-upload"; }
        return __awaiter(this, void 0, void 0, function () {
            var options;
            var _this = this;
            return __generator(this, function (_b) {
                options = {
                    method: "PUT",
                    body: file,
                    headers: {
                        "Content-Type": file.type,
                        "Content-Length": file.size.toString()
                    }
                };
                // For file uploads, we want the raw response object, not parsed JSON
                return [2 /*return*/, this.executeWithRetry(function () { return __awaiter(_this, void 0, void 0, function () {
                        var response;
                        return __generator(this, function (_b) {
                            switch (_b.label) {
                                case 0: return [4 /*yield*/, fetch(signedURL, options)];
                                case 1:
                                    response = _b.sent();
                                    if (response.ok) {
                                        return [2 /*return*/, response];
                                    }
                                    // Handle error response
                                    throw new Error("Upload to " + endpointName + " failed. HTTP " + response.status);
                            }
                        });
                    }); }, endpointName, 'upload')];
            });
        });
    };
    /**
     * Convenience method for DELETE operations with signed URLs
     */
    FetchUtils.prototype.deleteWithSignedUrl = function (signedURL, endpointName) {
        if (endpointName === void 0) { endpointName = "resource-delete"; }
        return __awaiter(this, void 0, void 0, function () {
            var options;
            var _this = this;
            return __generator(this, function (_b) {
                options = {
                    method: "DELETE"
                };
                // For DELETE operations, we want the raw response object
                return [2 /*return*/, this.executeWithRetry(function () { return __awaiter(_this, void 0, void 0, function () {
                        var response;
                        return __generator(this, function (_b) {
                            switch (_b.label) {
                                case 0: return [4 /*yield*/, fetch(signedURL, options)];
                                case 1:
                                    response = _b.sent();
                                    if (response.ok) {
                                        return [2 /*return*/, response];
                                    }
                                    // Handle error response
                                    throw new Error("Delete operation to " + endpointName + " failed. HTTP " + response.status);
                            }
                        });
                    }); }, endpointName, 'delete')];
            });
        });
    };
    return FetchUtils;
}(NewRelicEnabledClient));
// TODO [Eelco] I really dislike how this is hard coded. Why not just have a simple mechanism that allows you to
// set the api and bundle urls directly, and use that throughout the code base?
var HOST_DEFAULTS = {
    prod: {
        apiUrl: 'https://api.pie-api.com/api',
        bundleBaseUrl: 'https://storage.googleapis.com/pie-pits-prod/bundles/'
    },
    stage: {
        apiUrl: 'https://api.dev.pie-api.com/api',
        bundleBaseUrl: 'https://storage.googleapis.com/pie-pits-prod/bundles/'
    },
    dev: {
        apiUrl: 'https://api.dev.pie-api.com/api',
        bundleBaseUrl: 'https://storage.googleapis.com/pie-pits-prod/bundles/'
    },
    qa: {
        apiUrl: 'https://api.dev.pie-api.com/api',
        bundleBaseUrl: 'https://storage.googleapis.com/pie-pits-prod/bundles/'
    },
    local: {
        apiUrl: 'http://localhost:4000/api',
        bundleBaseUrl: 'https://storage.googleapis.com/pie-pits-prod/bundles/'
    }
};
var PLAYER_HOST_CONFIG = {
    host: HOST_DEFAULTS.prod.apiUrl,
    bundleBaseUrl: HOST_DEFAULTS.prod.bundleBaseUrl,
    // load: "/loadPlayerJson/",
    load: "/player/load",
    saveSession: "/player/save",
    model: "/player/model",
    score: "/player/score"
};
var VERSION = '1.0.1';
var semver = createCommonjsModule(function (module, exports) {
    exports = module.exports = SemVer;
    var debug;
    /* istanbul ignore next */
    if (typeof process === 'object' &&
        process.env &&
        process.env.NODE_DEBUG &&
        /\bsemver\b/i.test(process.env.NODE_DEBUG)) {
        debug = function () {
            var args = Array.prototype.slice.call(arguments, 0);
            args.unshift('SEMVER');
            console.log.apply(console, args);
        };
    }
    else {
        debug = function () { };
    }
    // Note: this is the semver.org version of the spec that it implements
    // Not necessarily the package version of this code.
    exports.SEMVER_SPEC_VERSION = '2.0.0';
    var MAX_LENGTH = 256;
    var MAX_SAFE_INTEGER = Number.MAX_SAFE_INTEGER ||
        /* istanbul ignore next */ 9007199254740991;
    // Max safe segment length for coercion.
    var MAX_SAFE_COMPONENT_LENGTH = 16;
    // The actual regexps go on exports.re
    var re = exports.re = [];
    var src = exports.src = [];
    var R = 0;
    // The following Regular Expressions can be used for tokenizing,
    // validating, and parsing SemVer version strings.
    // ## Numeric Identifier
    // A single `0`, or a non-zero digit followed by zero or more digits.
    var NUMERICIDENTIFIER = R++;
    src[NUMERICIDENTIFIER] = '0|[1-9]\\d*';
    var NUMERICIDENTIFIERLOOSE = R++;
    src[NUMERICIDENTIFIERLOOSE] = '[0-9]+';
    // ## Non-numeric Identifier
    // Zero or more digits, followed by a letter or hyphen, and then zero or
    // more letters, digits, or hyphens.
    var NONNUMERICIDENTIFIER = R++;
    src[NONNUMERICIDENTIFIER] = '\\d*[a-zA-Z-][a-zA-Z0-9-]*';
    // ## Main Version
    // Three dot-separated numeric identifiers.
    var MAINVERSION = R++;
    src[MAINVERSION] = '(' + src[NUMERICIDENTIFIER] + ')\\.' +
        '(' + src[NUMERICIDENTIFIER] + ')\\.' +
        '(' + src[NUMERICIDENTIFIER] + ')';
    var MAINVERSIONLOOSE = R++;
    src[MAINVERSIONLOOSE] = '(' + src[NUMERICIDENTIFIERLOOSE] + ')\\.' +
        '(' + src[NUMERICIDENTIFIERLOOSE] + ')\\.' +
        '(' + src[NUMERICIDENTIFIERLOOSE] + ')';
    // ## Pre-release Version Identifier
    // A numeric identifier, or a non-numeric identifier.
    var PRERELEASEIDENTIFIER = R++;
    src[PRERELEASEIDENTIFIER] = '(?:' + src[NUMERICIDENTIFIER] +
        '|' + src[NONNUMERICIDENTIFIER] + ')';
    var PRERELEASEIDENTIFIERLOOSE = R++;
    src[PRERELEASEIDENTIFIERLOOSE] = '(?:' + src[NUMERICIDENTIFIERLOOSE] +
        '|' + src[NONNUMERICIDENTIFIER] + ')';
    // ## Pre-release Version
    // Hyphen, followed by one or more dot-separated pre-release version
    // identifiers.
    var PRERELEASE = R++;
    src[PRERELEASE] = '(?:-(' + src[PRERELEASEIDENTIFIER] +
        '(?:\\.' + src[PRERELEASEIDENTIFIER] + ')*))';
    var PRERELEASELOOSE = R++;
    src[PRERELEASELOOSE] = '(?:-?(' + src[PRERELEASEIDENTIFIERLOOSE] +
        '(?:\\.' + src[PRERELEASEIDENTIFIERLOOSE] + ')*))';
    // ## Build Metadata Identifier
    // Any combination of digits, letters, or hyphens.
    var BUILDIDENTIFIER = R++;
    src[BUILDIDENTIFIER] = '[0-9A-Za-z-]+';
    // ## Build Metadata
    // Plus sign, followed by one or more period-separated build metadata
    // identifiers.
    var BUILD = R++;
    src[BUILD] = '(?:\\+(' + src[BUILDIDENTIFIER] +
        '(?:\\.' + src[BUILDIDENTIFIER] + ')*))';
    // ## Full Version String
    // A main version, followed optionally by a pre-release version and
    // build metadata.
    // Note that the only major, minor, patch, and pre-release sections of
    // the version string are capturing groups.  The build metadata is not a
    // capturing group, because it should not ever be used in version
    // comparison.
    var FULL = R++;
    var FULLPLAIN = 'v?' + src[MAINVERSION] +
        src[PRERELEASE] + '?' +
        src[BUILD] + '?';
    src[FULL] = '^' + FULLPLAIN + '$';
    // like full, but allows v1.2.3 and =1.2.3, which people do sometimes.
    // also, 1.0.0alpha1 (prerelease without the hyphen) which is pretty
    // common in the npm registry.
    var LOOSEPLAIN = '[v=\\s]*' + src[MAINVERSIONLOOSE] +
        src[PRERELEASELOOSE] + '?' +
        src[BUILD] + '?';
    var LOOSE = R++;
    src[LOOSE] = '^' + LOOSEPLAIN + '$';
    var GTLT = R++;
    src[GTLT] = '((?:<|>)?=?)';
    // Something like "2.*" or "1.2.x".
    // Note that "x.x" is a valid xRange identifer, meaning "any version"
    // Only the first item is strictly required.
    var XRANGEIDENTIFIERLOOSE = R++;
    src[XRANGEIDENTIFIERLOOSE] = src[NUMERICIDENTIFIERLOOSE] + '|x|X|\\*';
    var XRANGEIDENTIFIER = R++;
    src[XRANGEIDENTIFIER] = src[NUMERICIDENTIFIER] + '|x|X|\\*';
    var XRANGEPLAIN = R++;
    src[XRANGEPLAIN] = '[v=\\s]*(' + src[XRANGEIDENTIFIER] + ')' +
        '(?:\\.(' + src[XRANGEIDENTIFIER] + ')' +
        '(?:\\.(' + src[XRANGEIDENTIFIER] + ')' +
        '(?:' + src[PRERELEASE] + ')?' +
        src[BUILD] + '?' +
        ')?)?';
    var XRANGEPLAINLOOSE = R++;
    src[XRANGEPLAINLOOSE] = '[v=\\s]*(' + src[XRANGEIDENTIFIERLOOSE] + ')' +
        '(?:\\.(' + src[XRANGEIDENTIFIERLOOSE] + ')' +
        '(?:\\.(' + src[XRANGEIDENTIFIERLOOSE] + ')' +
        '(?:' + src[PRERELEASELOOSE] + ')?' +
        src[BUILD] + '?' +
        ')?)?';
    var XRANGE = R++;
    src[XRANGE] = '^' + src[GTLT] + '\\s*' + src[XRANGEPLAIN] + '$';
    var XRANGELOOSE = R++;
    src[XRANGELOOSE] = '^' + src[GTLT] + '\\s*' + src[XRANGEPLAINLOOSE] + '$';
    // Coercion.
    // Extract anything that could conceivably be a part of a valid semver
    var COERCE = R++;
    src[COERCE] = '(?:^|[^\\d])' +
        '(\\d{1,' + MAX_SAFE_COMPONENT_LENGTH + '})' +
        '(?:\\.(\\d{1,' + MAX_SAFE_COMPONENT_LENGTH + '}))?' +
        '(?:\\.(\\d{1,' + MAX_SAFE_COMPONENT_LENGTH + '}))?' +
        '(?:$|[^\\d])';
    // Tilde ranges.
    // Meaning is "reasonably at or greater than"
    var LONETILDE = R++;
    src[LONETILDE] = '(?:~>?)';
    var TILDETRIM = R++;
    src[TILDETRIM] = '(\\s*)' + src[LONETILDE] + '\\s+';
    re[TILDETRIM] = new RegExp(src[TILDETRIM], 'g');
    var tildeTrimReplace = '$1~';
    var TILDE = R++;
    src[TILDE] = '^' + src[LONETILDE] + src[XRANGEPLAIN] + '$';
    var TILDELOOSE = R++;
    src[TILDELOOSE] = '^' + src[LONETILDE] + src[XRANGEPLAINLOOSE] + '$';
    // Caret ranges.
    // Meaning is "at least and backwards compatible with"
    var LONECARET = R++;
    src[LONECARET] = '(?:\\^)';
    var CARETTRIM = R++;
    src[CARETTRIM] = '(\\s*)' + src[LONECARET] + '\\s+';
    re[CARETTRIM] = new RegExp(src[CARETTRIM], 'g');
    var caretTrimReplace = '$1^';
    var CARET = R++;
    src[CARET] = '^' + src[LONECARET] + src[XRANGEPLAIN] + '$';
    var CARETLOOSE = R++;
    src[CARETLOOSE] = '^' + src[LONECARET] + src[XRANGEPLAINLOOSE] + '$';
    // A simple gt/lt/eq thing, or just "" to indicate "any version"
    var COMPARATORLOOSE = R++;
    src[COMPARATORLOOSE] = '^' + src[GTLT] + '\\s*(' + LOOSEPLAIN + ')$|^$';
    var COMPARATOR = R++;
    src[COMPARATOR] = '^' + src[GTLT] + '\\s*(' + FULLPLAIN + ')$|^$';
    // An expression to strip any whitespace between the gtlt and the thing
    // it modifies, so that `> 1.2.3` ==> `>1.2.3`
    var COMPARATORTRIM = R++;
    src[COMPARATORTRIM] = '(\\s*)' + src[GTLT] +
        '\\s*(' + LOOSEPLAIN + '|' + src[XRANGEPLAIN] + ')';
    // this one has to use the /g flag
    re[COMPARATORTRIM] = new RegExp(src[COMPARATORTRIM], 'g');
    var comparatorTrimReplace = '$1$2$3';
    // Something like `1.2.3 - 1.2.4`
    // Note that these all use the loose form, because they'll be
    // checked against either the strict or loose comparator form
    // later.
    var HYPHENRANGE = R++;
    src[HYPHENRANGE] = '^\\s*(' + src[XRANGEPLAIN] + ')' +
        '\\s+-\\s+' +
        '(' + src[XRANGEPLAIN] + ')' +
        '\\s*$';
    var HYPHENRANGELOOSE = R++;
    src[HYPHENRANGELOOSE] = '^\\s*(' + src[XRANGEPLAINLOOSE] + ')' +
        '\\s+-\\s+' +
        '(' + src[XRANGEPLAINLOOSE] + ')' +
        '\\s*$';
    // Star ranges basically just allow anything at all.
    var STAR = R++;
    src[STAR] = '(<|>)?=?\\s*\\*';
    // Compile to actual regexp objects.
    // All are flag-free, unless they were created above with a flag.
    for (var i = 0; i < R; i++) {
        debug(i, src[i]);
        if (!re[i]) {
            re[i] = new RegExp(src[i]);
        }
    }
    exports.parse = parse;
    function parse(version, options) {
        if (!options || typeof options !== 'object') {
            options = {
                loose: !!options,
                includePrerelease: false
            };
        }
        if (version instanceof SemVer) {
            return version;
        }
        if (typeof version !== 'string') {
            return null;
        }
        if (version.length > MAX_LENGTH) {
            return null;
        }
        var r = options.loose ? re[LOOSE] : re[FULL];
        if (!r.test(version)) {
            return null;
        }
        try {
            return new SemVer(version, options);
        }
        catch (er) {
            return null;
        }
    }
    exports.valid = valid;
    function valid(version, options) {
        var v = parse(version, options);
        return v ? v.version : null;
    }
    exports.clean = clean;
    function clean(version, options) {
        var s = parse(version.trim().replace(/^[=v]+/, ''), options);
        return s ? s.version : null;
    }
    exports.SemVer = SemVer;
    function SemVer(version, options) {
        if (!options || typeof options !== 'object') {
            options = {
                loose: !!options,
                includePrerelease: false
            };
        }
        if (version instanceof SemVer) {
            if (version.loose === options.loose) {
                return version;
            }
            else {
                version = version.version;
            }
        }
        else if (typeof version !== 'string') {
            throw new TypeError('Invalid Version: ' + version);
        }
        if (version.length > MAX_LENGTH) {
            throw new TypeError('version is longer than ' + MAX_LENGTH + ' characters');
        }
        if (!(this instanceof SemVer)) {
            return new SemVer(version, options);
        }
        debug('SemVer', version, options);
        this.options = options;
        this.loose = !!options.loose;
        var m = version.trim().match(options.loose ? re[LOOSE] : re[FULL]);
        if (!m) {
            throw new TypeError('Invalid Version: ' + version);
        }
        this.raw = version;
        // these are actually numbers
        this.major = +m[1];
        this.minor = +m[2];
        this.patch = +m[3];
        if (this.major > MAX_SAFE_INTEGER || this.major < 0) {
            throw new TypeError('Invalid major version');
        }
        if (this.minor > MAX_SAFE_INTEGER || this.minor < 0) {
            throw new TypeError('Invalid minor version');
        }
        if (this.patch > MAX_SAFE_INTEGER || this.patch < 0) {
            throw new TypeError('Invalid patch version');
        }
        // numberify any prerelease numeric ids
        if (!m[4]) {
            this.prerelease = [];
        }
        else {
            this.prerelease = m[4].split('.').map(function (id) {
                if (/^[0-9]+$/.test(id)) {
                    var num = +id;
                    if (num >= 0 && num < MAX_SAFE_INTEGER) {
                        return num;
                    }
                }
                return id;
            });
        }
        this.build = m[5] ? m[5].split('.') : [];
        this.format();
    }
    SemVer.prototype.format = function () {
        this.version = this.major + '.' + this.minor + '.' + this.patch;
        if (this.prerelease.length) {
            this.version += '-' + this.prerelease.join('.');
        }
        return this.version;
    };
    SemVer.prototype.toString = function () {
        return this.version;
    };
    SemVer.prototype.compare = function (other) {
        debug('SemVer.compare', this.version, this.options, other);
        if (!(other instanceof SemVer)) {
            other = new SemVer(other, this.options);
        }
        return this.compareMain(other) || this.comparePre(other);
    };
    SemVer.prototype.compareMain = function (other) {
        if (!(other instanceof SemVer)) {
            other = new SemVer(other, this.options);
        }
        return compareIdentifiers(this.major, other.major) ||
            compareIdentifiers(this.minor, other.minor) ||
            compareIdentifiers(this.patch, other.patch);
    };
    SemVer.prototype.comparePre = function (other) {
        if (!(other instanceof SemVer)) {
            other = new SemVer(other, this.options);
        }
        // NOT having a prerelease is > having one
        if (this.prerelease.length && !other.prerelease.length) {
            return -1;
        }
        else if (!this.prerelease.length && other.prerelease.length) {
            return 1;
        }
        else if (!this.prerelease.length && !other.prerelease.length) {
            return 0;
        }
        var i = 0;
        do {
            var a = this.prerelease[i];
            var b = other.prerelease[i];
            debug('prerelease compare', i, a, b);
            if (a === undefined && b === undefined) {
                return 0;
            }
            else if (b === undefined) {
                return 1;
            }
            else if (a === undefined) {
                return -1;
            }
            else if (a === b) {
                continue;
            }
            else {
                return compareIdentifiers(a, b);
            }
        } while (++i);
    };
    // preminor will bump the version up to the next minor release, and immediately
    // down to pre-release. premajor and prepatch work the same way.
    SemVer.prototype.inc = function (release, identifier) {
        switch (release) {
            case 'premajor':
                this.prerelease.length = 0;
                this.patch = 0;
                this.minor = 0;
                this.major++;
                this.inc('pre', identifier);
                break;
            case 'preminor':
                this.prerelease.length = 0;
                this.patch = 0;
                this.minor++;
                this.inc('pre', identifier);
                break;
            case 'prepatch':
                // If this is already a prerelease, it will bump to the next version
                // drop any prereleases that might already exist, since they are not
                // relevant at this point.
                this.prerelease.length = 0;
                this.inc('patch', identifier);
                this.inc('pre', identifier);
                break;
            // If the input is a non-prerelease version, this acts the same as
            // prepatch.
            case 'prerelease':
                if (this.prerelease.length === 0) {
                    this.inc('patch', identifier);
                }
                this.inc('pre', identifier);
                break;
            case 'major':
                // If this is a pre-major version, bump up to the same major version.
                // Otherwise increment major.
                // 1.0.0-5 bumps to 1.0.0
                // 1.1.0 bumps to 2.0.0
                if (this.minor !== 0 ||
                    this.patch !== 0 ||
                    this.prerelease.length === 0) {
                    this.major++;
                }
                this.minor = 0;
                this.patch = 0;
                this.prerelease = [];
                break;
            case 'minor':
                // If this is a pre-minor version, bump up to the same minor version.
                // Otherwise increment minor.
                // 1.2.0-5 bumps to 1.2.0
                // 1.2.1 bumps to 1.3.0
                if (this.patch !== 0 || this.prerelease.length === 0) {
                    this.minor++;
                }
                this.patch = 0;
                this.prerelease = [];
                break;
            case 'patch':
                // If this is not a pre-release version, it will increment the patch.
                // If it is a pre-release it will bump up to the same patch version.
                // 1.2.0-5 patches to 1.2.0
                // 1.2.0 patches to 1.2.1
                if (this.prerelease.length === 0) {
                    this.patch++;
                }
                this.prerelease = [];
                break;
            // This probably shouldn't be used publicly.
            // 1.0.0 "pre" would become 1.0.0-0 which is the wrong direction.
            case 'pre':
                if (this.prerelease.length === 0) {
                    this.prerelease = [0];
                }
                else {
                    var i = this.prerelease.length;
                    while (--i >= 0) {
                        if (typeof this.prerelease[i] === 'number') {
                            this.prerelease[i]++;
                            i = -2;
                        }
                    }
                    if (i === -1) {
                        // didn't increment anything
                        this.prerelease.push(0);
                    }
                }
                if (identifier) {
                    // 1.2.0-beta.1 bumps to 1.2.0-beta.2,
                    // 1.2.0-beta.fooblz or 1.2.0-beta bumps to 1.2.0-beta.0
                    if (this.prerelease[0] === identifier) {
                        if (isNaN(this.prerelease[1])) {
                            this.prerelease = [identifier, 0];
                        }
                    }
                    else {
                        this.prerelease = [identifier, 0];
                    }
                }
                break;
            default:
                throw new Error('invalid increment argument: ' + release);
        }
        this.format();
        this.raw = this.version;
        return this;
    };
    exports.inc = inc;
    function inc(version, release, loose, identifier) {
        if (typeof (loose) === 'string') {
            identifier = loose;
            loose = undefined;
        }
        try {
            return new SemVer(version, loose).inc(release, identifier).version;
        }
        catch (er) {
            return null;
        }
    }
    exports.diff = diff;
    function diff(version1, version2) {
        if (eq(version1, version2)) {
            return null;
        }
        else {
            var v1 = parse(version1);
            var v2 = parse(version2);
            var prefix = '';
            if (v1.prerelease.length || v2.prerelease.length) {
                prefix = 'pre';
                var defaultResult = 'prerelease';
            }
            for (var key in v1) {
                if (key === 'major' || key === 'minor' || key === 'patch') {
                    if (v1[key] !== v2[key]) {
                        return prefix + key;
                    }
                }
            }
            return defaultResult; // may be undefined
        }
    }
    exports.compareIdentifiers = compareIdentifiers;
    var numeric = /^[0-9]+$/;
    function compareIdentifiers(a, b) {
        var anum = numeric.test(a);
        var bnum = numeric.test(b);
        if (anum && bnum) {
            a = +a;
            b = +b;
        }
        return a === b ? 0
            : (anum && !bnum) ? -1
                : (bnum && !anum) ? 1
                    : a < b ? -1
                        : 1;
    }
    exports.rcompareIdentifiers = rcompareIdentifiers;
    function rcompareIdentifiers(a, b) {
        return compareIdentifiers(b, a);
    }
    exports.major = major;
    function major(a, loose) {
        return new SemVer(a, loose).major;
    }
    exports.minor = minor;
    function minor(a, loose) {
        return new SemVer(a, loose).minor;
    }
    exports.patch = patch;
    function patch(a, loose) {
        return new SemVer(a, loose).patch;
    }
    exports.compare = compare;
    function compare(a, b, loose) {
        return new SemVer(a, loose).compare(new SemVer(b, loose));
    }
    exports.compareLoose = compareLoose;
    function compareLoose(a, b) {
        return compare(a, b, true);
    }
    exports.rcompare = rcompare;
    function rcompare(a, b, loose) {
        return compare(b, a, loose);
    }
    exports.sort = sort;
    function sort(list, loose) {
        return list.sort(function (a, b) {
            return exports.compare(a, b, loose);
        });
    }
    exports.rsort = rsort;
    function rsort(list, loose) {
        return list.sort(function (a, b) {
            return exports.rcompare(a, b, loose);
        });
    }
    exports.gt = gt;
    function gt(a, b, loose) {
        return compare(a, b, loose) > 0;
    }
    exports.lt = lt;
    function lt(a, b, loose) {
        return compare(a, b, loose) < 0;
    }
    exports.eq = eq;
    function eq(a, b, loose) {
        return compare(a, b, loose) === 0;
    }
    exports.neq = neq;
    function neq(a, b, loose) {
        return compare(a, b, loose) !== 0;
    }
    exports.gte = gte;
    function gte(a, b, loose) {
        return compare(a, b, loose) >= 0;
    }
    exports.lte = lte;
    function lte(a, b, loose) {
        return compare(a, b, loose) <= 0;
    }
    exports.cmp = cmp;
    function cmp(a, op, b, loose) {
        switch (op) {
            case '===':
                if (typeof a === 'object')
                    a = a.version;
                if (typeof b === 'object')
                    b = b.version;
                return a === b;
            case '!==':
                if (typeof a === 'object')
                    a = a.version;
                if (typeof b === 'object')
                    b = b.version;
                return a !== b;
            case '':
            case '=':
            case '==':
                return eq(a, b, loose);
            case '!=':
                return neq(a, b, loose);
            case '>':
                return gt(a, b, loose);
            case '>=':
                return gte(a, b, loose);
            case '<':
                return lt(a, b, loose);
            case '<=':
                return lte(a, b, loose);
            default:
                throw new TypeError('Invalid operator: ' + op);
        }
    }
    exports.Comparator = Comparator;
    function Comparator(comp, options) {
        if (!options || typeof options !== 'object') {
            options = {
                loose: !!options,
                includePrerelease: false
            };
        }
        if (comp instanceof Comparator) {
            if (comp.loose === !!options.loose) {
                return comp;
            }
            else {
                comp = comp.value;
            }
        }
        if (!(this instanceof Comparator)) {
            return new Comparator(comp, options);
        }
        debug('comparator', comp, options);
        this.options = options;
        this.loose = !!options.loose;
        this.parse(comp);
        if (this.semver === ANY) {
            this.value = '';
        }
        else {
            this.value = this.operator + this.semver.version;
        }
        debug('comp', this);
    }
    var ANY = {};
    Comparator.prototype.parse = function (comp) {
        var r = this.options.loose ? re[COMPARATORLOOSE] : re[COMPARATOR];
        var m = comp.match(r);
        if (!m) {
            throw new TypeError('Invalid comparator: ' + comp);
        }
        this.operator = m[1];
        if (this.operator === '=') {
            this.operator = '';
        }
        // if it literally is just '>' or '' then allow anything.
        if (!m[2]) {
            this.semver = ANY;
        }
        else {
            this.semver = new SemVer(m[2], this.options.loose);
        }
    };
    Comparator.prototype.toString = function () {
        return this.value;
    };
    Comparator.prototype.test = function (version) {
        debug('Comparator.test', version, this.options.loose);
        if (this.semver === ANY) {
            return true;
        }
        if (typeof version === 'string') {
            version = new SemVer(version, this.options);
        }
        return cmp(version, this.operator, this.semver, this.options);
    };
    Comparator.prototype.intersects = function (comp, options) {
        if (!(comp instanceof Comparator)) {
            throw new TypeError('a Comparator is required');
        }
        if (!options || typeof options !== 'object') {
            options = {
                loose: !!options,
                includePrerelease: false
            };
        }
        var rangeTmp;
        if (this.operator === '') {
            rangeTmp = new Range(comp.value, options);
            return satisfies(this.value, rangeTmp, options);
        }
        else if (comp.operator === '') {
            rangeTmp = new Range(this.value, options);
            return satisfies(comp.semver, rangeTmp, options);
        }
        var sameDirectionIncreasing = (this.operator === '>=' || this.operator === '>') &&
            (comp.operator === '>=' || comp.operator === '>');
        var sameDirectionDecreasing = (this.operator === '<=' || this.operator === '<') &&
            (comp.operator === '<=' || comp.operator === '<');
        var sameSemVer = this.semver.version === comp.semver.version;
        var differentDirectionsInclusive = (this.operator === '>=' || this.operator === '<=') &&
            (comp.operator === '>=' || comp.operator === '<=');
        var oppositeDirectionsLessThan = cmp(this.semver, '<', comp.semver, options) &&
            ((this.operator === '>=' || this.operator === '>') &&
                (comp.operator === '<=' || comp.operator === '<'));
        var oppositeDirectionsGreaterThan = cmp(this.semver, '>', comp.semver, options) &&
            ((this.operator === '<=' || this.operator === '<') &&
                (comp.operator === '>=' || comp.operator === '>'));
        return sameDirectionIncreasing || sameDirectionDecreasing ||
            (sameSemVer && differentDirectionsInclusive) ||
            oppositeDirectionsLessThan || oppositeDirectionsGreaterThan;
    };
    exports.Range = Range;
    function Range(range, options) {
        if (!options || typeof options !== 'object') {
            options = {
                loose: !!options,
                includePrerelease: false
            };
        }
        if (range instanceof Range) {
            if (range.loose === !!options.loose &&
                range.includePrerelease === !!options.includePrerelease) {
                return range;
            }
            else {
                return new Range(range.raw, options);
            }
        }
        if (range instanceof Comparator) {
            return new Range(range.value, options);
        }
        if (!(this instanceof Range)) {
            return new Range(range, options);
        }
        this.options = options;
        this.loose = !!options.loose;
        this.includePrerelease = !!options.includePrerelease;
        // First, split based on boolean or ||
        this.raw = range;
        this.set = range.split(/\s*\|\|\s*/).map(function (range) {
            return this.parseRange(range.trim());
        }, this).filter(function (c) {
            // throw out any that are not relevant for whatever reason
            return c.length;
        });
        if (!this.set.length) {
            throw new TypeError('Invalid SemVer Range: ' + range);
        }
        this.format();
    }
    Range.prototype.format = function () {
        this.range = this.set.map(function (comps) {
            return comps.join(' ').trim();
        }).join('||').trim();
        return this.range;
    };
    Range.prototype.toString = function () {
        return this.range;
    };
    Range.prototype.parseRange = function (range) {
        var loose = this.options.loose;
        range = range.trim();
        // `1.2.3 - 1.2.4` => `>=1.2.3 <=1.2.4`
        var hr = loose ? re[HYPHENRANGELOOSE] : re[HYPHENRANGE];
        range = range.replace(hr, hyphenReplace);
        debug('hyphen replace', range);
        // `> 1.2.3 < 1.2.5` => `>1.2.3 <1.2.5`
        range = range.replace(re[COMPARATORTRIM], comparatorTrimReplace);
        debug('comparator trim', range, re[COMPARATORTRIM]);
        // `~ 1.2.3` => `~1.2.3`
        range = range.replace(re[TILDETRIM], tildeTrimReplace);
        // `^ 1.2.3` => `^1.2.3`
        range = range.replace(re[CARETTRIM], caretTrimReplace);
        // normalize spaces
        range = range.split(/\s+/).join(' ');
        // At this point, the range is completely trimmed and
        // ready to be split into comparators.
        var compRe = loose ? re[COMPARATORLOOSE] : re[COMPARATOR];
        var set = range.split(' ').map(function (comp) {
            return parseComparator(comp, this.options);
        }, this).join(' ').split(/\s+/);
        if (this.options.loose) {
            // in loose mode, throw out any that are not valid comparators
            set = set.filter(function (comp) {
                return !!comp.match(compRe);
            });
        }
        set = set.map(function (comp) {
            return new Comparator(comp, this.options);
        }, this);
        return set;
    };
    Range.prototype.intersects = function (range, options) {
        if (!(range instanceof Range)) {
            throw new TypeError('a Range is required');
        }
        return this.set.some(function (thisComparators) {
            return thisComparators.every(function (thisComparator) {
                return range.set.some(function (rangeComparators) {
                    return rangeComparators.every(function (rangeComparator) {
                        return thisComparator.intersects(rangeComparator, options);
                    });
                });
            });
        });
    };
    // Mostly just for testing and legacy API reasons
    exports.toComparators = toComparators;
    function toComparators(range, options) {
        return new Range(range, options).set.map(function (comp) {
            return comp.map(function (c) {
                return c.value;
            }).join(' ').trim().split(' ');
        });
    }
    // comprised of xranges, tildes, stars, and gtlt's at this point.
    // already replaced the hyphen ranges
    // turn into a set of JUST comparators.
    function parseComparator(comp, options) {
        debug('comp', comp, options);
        comp = replaceCarets(comp, options);
        debug('caret', comp);
        comp = replaceTildes(comp, options);
        debug('tildes', comp);
        comp = replaceXRanges(comp, options);
        debug('xrange', comp);
        comp = replaceStars(comp, options);
        debug('stars', comp);
        return comp;
    }
    function isX(id) {
        return !id || id.toLowerCase() === 'x' || id === '*';
    }
    // ~, ~> --> * (any, kinda silly)
    // ~2, ~2.x, ~2.x.x, ~>2, ~>2.x ~>2.x.x --> >=2.0.0 <3.0.0
    // ~2.0, ~2.0.x, ~>2.0, ~>2.0.x --> >=2.0.0 <2.1.0
    // ~1.2, ~1.2.x, ~>1.2, ~>1.2.x --> >=1.2.0 <1.3.0
    // ~1.2.3, ~>1.2.3 --> >=1.2.3 <1.3.0
    // ~1.2.0, ~>1.2.0 --> >=1.2.0 <1.3.0
    function replaceTildes(comp, options) {
        return comp.trim().split(/\s+/).map(function (comp) {
            return replaceTilde(comp, options);
        }).join(' ');
    }
    function replaceTilde(comp, options) {
        var r = options.loose ? re[TILDELOOSE] : re[TILDE];
        return comp.replace(r, function (_, M, m, p, pr) {
            debug('tilde', comp, _, M, m, p, pr);
            var ret;
            if (isX(M)) {
                ret = '';
            }
            else if (isX(m)) {
                ret = '>=' + M + '.0.0 <' + (+M + 1) + '.0.0';
            }
            else if (isX(p)) {
                // ~1.2 == >=1.2.0 <1.3.0
                ret = '>=' + M + '.' + m + '.0 <' + M + '.' + (+m + 1) + '.0';
            }
            else if (pr) {
                debug('replaceTilde pr', pr);
                ret = '>=' + M + '.' + m + '.' + p + '-' + pr +
                    ' <' + M + '.' + (+m + 1) + '.0';
            }
            else {
                // ~1.2.3 == >=1.2.3 <1.3.0
                ret = '>=' + M + '.' + m + '.' + p +
                    ' <' + M + '.' + (+m + 1) + '.0';
            }
            debug('tilde return', ret);
            return ret;
        });
    }
    // ^ --> * (any, kinda silly)
    // ^2, ^2.x, ^2.x.x --> >=2.0.0 <3.0.0
    // ^2.0, ^2.0.x --> >=2.0.0 <3.0.0
    // ^1.2, ^1.2.x --> >=1.2.0 <2.0.0
    // ^1.2.3 --> >=1.2.3 <2.0.0
    // ^1.2.0 --> >=1.2.0 <2.0.0
    function replaceCarets(comp, options) {
        return comp.trim().split(/\s+/).map(function (comp) {
            return replaceCaret(comp, options);
        }).join(' ');
    }
    function replaceCaret(comp, options) {
        debug('caret', comp, options);
        var r = options.loose ? re[CARETLOOSE] : re[CARET];
        return comp.replace(r, function (_, M, m, p, pr) {
            debug('caret', comp, _, M, m, p, pr);
            var ret;
            if (isX(M)) {
                ret = '';
            }
            else if (isX(m)) {
                ret = '>=' + M + '.0.0 <' + (+M + 1) + '.0.0';
            }
            else if (isX(p)) {
                if (M === '0') {
                    ret = '>=' + M + '.' + m + '.0 <' + M + '.' + (+m + 1) + '.0';
                }
                else {
                    ret = '>=' + M + '.' + m + '.0 <' + (+M + 1) + '.0.0';
                }
            }
            else if (pr) {
                debug('replaceCaret pr', pr);
                if (M === '0') {
                    if (m === '0') {
                        ret = '>=' + M + '.' + m + '.' + p + '-' + pr +
                            ' <' + M + '.' + m + '.' + (+p + 1);
                    }
                    else {
                        ret = '>=' + M + '.' + m + '.' + p + '-' + pr +
                            ' <' + M + '.' + (+m + 1) + '.0';
                    }
                }
                else {
                    ret = '>=' + M + '.' + m + '.' + p + '-' + pr +
                        ' <' + (+M + 1) + '.0.0';
                }
            }
            else {
                debug('no pr');
                if (M === '0') {
                    if (m === '0') {
                        ret = '>=' + M + '.' + m + '.' + p +
                            ' <' + M + '.' + m + '.' + (+p + 1);
                    }
                    else {
                        ret = '>=' + M + '.' + m + '.' + p +
                            ' <' + M + '.' + (+m + 1) + '.0';
                    }
                }
                else {
                    ret = '>=' + M + '.' + m + '.' + p +
                        ' <' + (+M + 1) + '.0.0';
                }
            }
            debug('caret return', ret);
            return ret;
        });
    }
    function replaceXRanges(comp, options) {
        debug('replaceXRanges', comp, options);
        return comp.split(/\s+/).map(function (comp) {
            return replaceXRange(comp, options);
        }).join(' ');
    }
    function replaceXRange(comp, options) {
        comp = comp.trim();
        var r = options.loose ? re[XRANGELOOSE] : re[XRANGE];
        return comp.replace(r, function (ret, gtlt, M, m, p, pr) {
            debug('xRange', comp, ret, gtlt, M, m, p, pr);
            var xM = isX(M);
            var xm = xM || isX(m);
            var xp = xm || isX(p);
            var anyX = xp;
            if (gtlt === '=' && anyX) {
                gtlt = '';
            }
            if (xM) {
                if (gtlt === '>' || gtlt === '<') {
                    // nothing is allowed
                    ret = '<0.0.0';
                }
                else {
                    // nothing is forbidden
                    ret = '*';
                }
            }
            else if (gtlt && anyX) {
                // we know patch is an x, because we have any x at all.
                // replace X with 0
                if (xm) {
                    m = 0;
                }
                p = 0;
                if (gtlt === '>') {
                    // >1 => >=2.0.0
                    // >1.2 => >=1.3.0
                    // >1.2.3 => >= 1.2.4
                    gtlt = '>=';
                    if (xm) {
                        M = +M + 1;
                        m = 0;
                        p = 0;
                    }
                    else {
                        m = +m + 1;
                        p = 0;
                    }
                }
                else if (gtlt === '<=') {
                    // <=0.7.x is actually <0.8.0, since any 0.7.x should
                    // pass.  Similarly, <=7.x is actually <8.0.0, etc.
                    gtlt = '<';
                    if (xm) {
                        M = +M + 1;
                    }
                    else {
                        m = +m + 1;
                    }
                }
                ret = gtlt + M + '.' + m + '.' + p;
            }
            else if (xm) {
                ret = '>=' + M + '.0.0 <' + (+M + 1) + '.0.0';
            }
            else if (xp) {
                ret = '>=' + M + '.' + m + '.0 <' + M + '.' + (+m + 1) + '.0';
            }
            debug('xRange return', ret);
            return ret;
        });
    }
    // Because * is AND-ed with everything else in the comparator,
    // and '' means "any version", just remove the *s entirely.
    function replaceStars(comp, options) {
        debug('replaceStars', comp, options);
        // Looseness is ignored here.  star is always as loose as it gets!
        return comp.trim().replace(re[STAR], '');
    }
    // This function is passed to string.replace(re[HYPHENRANGE])
    // M, m, patch, prerelease, build
    // 1.2 - 3.4.5 => >=1.2.0 <=3.4.5
    // 1.2.3 - 3.4 => >=1.2.0 <3.5.0 Any 3.4.x will do
    // 1.2 - 3.4 => >=1.2.0 <3.5.0
    function hyphenReplace($0, from, fM, fm, fp, fpr, fb, to, tM, tm, tp, tpr, tb) {
        if (isX(fM)) {
            from = '';
        }
        else if (isX(fm)) {
            from = '>=' + fM + '.0.0';
        }
        else if (isX(fp)) {
            from = '>=' + fM + '.' + fm + '.0';
        }
        else {
            from = '>=' + from;
        }
        if (isX(tM)) {
            to = '';
        }
        else if (isX(tm)) {
            to = '<' + (+tM + 1) + '.0.0';
        }
        else if (isX(tp)) {
            to = '<' + tM + '.' + (+tm + 1) + '.0';
        }
        else if (tpr) {
            to = '<=' + tM + '.' + tm + '.' + tp + '-' + tpr;
        }
        else {
            to = '<=' + to;
        }
        return (from + ' ' + to).trim();
    }
    // if ANY of the sets match ALL of its comparators, then pass
    Range.prototype.test = function (version) {
        if (!version) {
            return false;
        }
        if (typeof version === 'string') {
            version = new SemVer(version, this.options);
        }
        for (var i = 0; i < this.set.length; i++) {
            if (testSet(this.set[i], version, this.options)) {
                return true;
            }
        }
        return false;
    };
    function testSet(set, version, options) {
        for (var i = 0; i < set.length; i++) {
            if (!set[i].test(version)) {
                return false;
            }
        }
        if (version.prerelease.length && !options.includePrerelease) {
            // Find the set of versions that are allowed to have prereleases
            // For example, ^1.2.3-pr.1 desugars to >=1.2.3-pr.1 <2.0.0
            // That should allow `1.2.3-pr.2` to pass.
            // However, `1.2.4-alpha.notready` should NOT be allowed,
            // even though it's within the range set by the comparators.
            for (i = 0; i < set.length; i++) {
                debug(set[i].semver);
                if (set[i].semver === ANY) {
                    continue;
                }
                if (set[i].semver.prerelease.length > 0) {
                    var allowed = set[i].semver;
                    if (allowed.major === version.major &&
                        allowed.minor === version.minor &&
                        allowed.patch === version.patch) {
                        return true;
                    }
                }
            }
            // Version has a -pre, but it's not one of the ones we like.
            return false;
        }
        return true;
    }
    exports.satisfies = satisfies;
    function satisfies(version, range, options) {
        try {
            range = new Range(range, options);
        }
        catch (er) {
            return false;
        }
        return range.test(version);
    }
    exports.maxSatisfying = maxSatisfying;
    function maxSatisfying(versions, range, options) {
        var max = null;
        var maxSV = null;
        try {
            var rangeObj = new Range(range, options);
        }
        catch (er) {
            return null;
        }
        versions.forEach(function (v) {
            if (rangeObj.test(v)) {
                // satisfies(v, range, options)
                if (!max || maxSV.compare(v) === -1) {
                    // compare(max, v, true)
                    max = v;
                    maxSV = new SemVer(max, options);
                }
            }
        });
        return max;
    }
    exports.minSatisfying = minSatisfying;
    function minSatisfying(versions, range, options) {
        var min = null;
        var minSV = null;
        try {
            var rangeObj = new Range(range, options);
        }
        catch (er) {
            return null;
        }
        versions.forEach(function (v) {
            if (rangeObj.test(v)) {
                // satisfies(v, range, options)
                if (!min || minSV.compare(v) === 1) {
                    // compare(min, v, true)
                    min = v;
                    minSV = new SemVer(min, options);
                }
            }
        });
        return min;
    }
    exports.minVersion = minVersion;
    function minVersion(range, loose) {
        range = new Range(range, loose);
        var minver = new SemVer('0.0.0');
        if (range.test(minver)) {
            return minver;
        }
        minver = new SemVer('0.0.0-0');
        if (range.test(minver)) {
            return minver;
        }
        minver = null;
        for (var i = 0; i < range.set.length; ++i) {
            var comparators = range.set[i];
            comparators.forEach(function (comparator) {
                // Clone to avoid manipulating the comparator's semver object.
                var compver = new SemVer(comparator.semver.version);
                switch (comparator.operator) {
                    case '>':
                        if (compver.prerelease.length === 0) {
                            compver.patch++;
                        }
                        else {
                            compver.prerelease.push(0);
                        }
                        compver.raw = compver.format();
                    /* fallthrough */
                    case '':
                    case '>=':
                        if (!minver || gt(minver, compver)) {
                            minver = compver;
                        }
                        break;
                    case '<':
                    case '<=':
                        /* Ignore maximum versions */
                        break;
                    /* istanbul ignore next */
                    default:
                        throw new Error('Unexpected operation: ' + comparator.operator);
                }
            });
        }
        if (minver && range.test(minver)) {
            return minver;
        }
        return null;
    }
    exports.validRange = validRange;
    function validRange(range, options) {
        try {
            // Return '*' instead of '' so that truthiness works.
            // This will throw if it's invalid anyway
            return new Range(range, options).range || '*';
        }
        catch (er) {
            return null;
        }
    }
    // Determine if version is less than all the versions possible in the range
    exports.ltr = ltr;
    function ltr(version, range, options) {
        return outside(version, range, '<', options);
    }
    // Determine if version is greater than all the versions possible in the range.
    exports.gtr = gtr;
    function gtr(version, range, options) {
        return outside(version, range, '>', options);
    }
    exports.outside = outside;
    function outside(version, range, hilo, options) {
        version = new SemVer(version, options);
        range = new Range(range, options);
        var gtfn, ltefn, ltfn, comp, ecomp;
        switch (hilo) {
            case '>':
                gtfn = gt;
                ltefn = lte;
                ltfn = lt;
                comp = '>';
                ecomp = '>=';
                break;
            case '<':
                gtfn = lt;
                ltefn = gte;
                ltfn = gt;
                comp = '<';
                ecomp = '<=';
                break;
            default:
                throw new TypeError('Must provide a hilo val of "<" or ">"');
        }
        // If it satisifes the range it is not outside
        if (satisfies(version, range, options)) {
            return false;
        }
        // From now on, variable terms are as if we're in "gtr" mode.
        // but note that everything is flipped for the "ltr" function.
        for (var i = 0; i < range.set.length; ++i) {
            var comparators = range.set[i];
            var high = null;
            var low = null;
            comparators.forEach(function (comparator) {
                if (comparator.semver === ANY) {
                    comparator = new Comparator('>=0.0.0');
                }
                high = high || comparator;
                low = low || comparator;
                if (gtfn(comparator.semver, high.semver, options)) {
                    high = comparator;
                }
                else if (ltfn(comparator.semver, low.semver, options)) {
                    low = comparator;
                }
            });
            // If the edge version comparator has a operator then our version
            // isn't outside it
            if (high.operator === comp || high.operator === ecomp) {
                return false;
            }
            // If the lowest version comparator has an operator and our version
            // is less than it then it isn't higher than the range
            if ((!low.operator || low.operator === comp) &&
                ltefn(version, low.semver)) {
                return false;
            }
            else if (low.operator === ecomp && ltfn(version, low.semver)) {
                return false;
            }
        }
        return true;
    }
    exports.prerelease = prerelease;
    function prerelease(version, options) {
        var parsed = parse(version, options);
        return (parsed && parsed.prerelease.length) ? parsed.prerelease : null;
    }
    exports.intersects = intersects;
    function intersects(r1, r2, options) {
        r1 = new Range(r1, options);
        r2 = new Range(r2, options);
        return r1.intersects(r2);
    }
    exports.coerce = coerce;
    function coerce(version) {
        if (version instanceof SemVer) {
            return version;
        }
        if (typeof version !== 'string') {
            return null;
        }
        var match = version.match(re[COERCE]);
        if (match == null) {
            return null;
        }
        return parse(match[1] +
            '.' + (match[2] || '0') +
            '.' + (match[3] || '0'));
    }
});
var semver_4 = semver.parse;
var getVersionedId = function (versionedId) {
    if (!versionedId)
        return null;
    var _b = versionedId.split("@"), id = _b[0], version = _b[1];
    var parsed = semver_4(version);
    if (version && !parsed)
        return null; // badly formatted version
    if (!id)
        return null;
    var parsedVersion = parsed && parsed.version
        ? {
            major: parsed ? parsed.major : null,
            minor: parsed ? parsed.minor : null,
            patch: parsed ? parsed.patch : null,
            prerelease: {
                tag: parsed ? parsed.prerelease[0] : null,
                version: parsed ? parsed.prerelease[1] : null,
            },
        }
        : null;
    return {
        id: id,
        version: parsedVersion,
    };
};
var versionedIdtoString = function (vId) {
    try {
        if (!vId && vId.version)
            return null;
        var prTag = vId.version.prerelease && vId.version.prerelease.tag;
        var prVersion = vId.version.prerelease &&
            typeof vId.version.prerelease.version === "number"
            ? vId.version.prerelease.version.toString()
            : "";
        // prerelease must at least have tag
        var prString = prTag
            ? "-" + prTag + (prVersion ? "." + prVersion : "")
            : "";
        return vId.id + "@" + vId.version.major + "." + vId.version.minor + "." + vId.version.patch + prString;
    }
    catch (_a) {
        return null;
    }
};
var isPreRelease = function (id) {
    return !!(id &&
        id.version &&
        (id.version.major || id.version.major === 0) &&
        (id.version.minor || id.version.minor === 0) &&
        (id.version.patch || id.version.patch === 0) &&
        id.version.prerelease &&
        id.version.prerelease.tag);
};
var idInfo = function (id) {
    var vId = getVersionedId(id);
    return id && vId
        ? isPreRelease(getVersionedId(id))
            ? "pre-release"
            : "released"
        : void 0;
};
var lockConfigVersions = function (config, versions, overrides) {
    if (!versions) {
        console.warn("Error loading versions - dont change the config");
        return config;
    }
    var out = cloneDeep(config);
    var elements = out && out.elements;
    if (elements) {
        var keys = Object.keys(elements);
        keys.forEach(function (key) {
            var v = elements[key];
            var packageName = parseNpm(v).name;
            if (versions[packageName]) {
                var mk = Object.keys(overrides || {}).find(function (o) { return o === packageName || new RegExp(o).test(packageName); });
                var v_1 = mk ? overrides[mk] : versions[packageName];
                elements[key] = packageName + "@" + v_1;
            }
        });
    }
    return out;
};
var unlockConfigVersions = function (config) {
    var out = cloneDeep(config);
    var elements = out && out.elements;
    if (elements) {
        var keys = Object.keys(elements);
        keys.forEach(function (key) {
            var v = elements[key];
            var packageName = parseNpm(v).name;
            elements[key] = packageName + "@latest";
        });
    }
    return out;
};
var splitStimulusItemsBundle = function (bundleUrl) {
    if (bundleUrl && bundleUrl[0]) {
        var url = bundleUrl[0];
        var baseUrl = url.split('/bundles/')[0]; // Get the base URL
        var bundlePath = url.split('/bundles/')[1]; // Get the bundles part after /bundles/
        // /load endpoint always returns player.js bundle
        var bundleList = bundlePath.split('/player.js')[0]; // Separate bundles and file
        // backend joins bundles by +
        var components = bundleList.split('+'); // Split bundles by '+'
        // Group components for each new bundle
        var group1 = components.filter(function (bundle) { return !bundle.includes('@pie-element/passage'); });
        var group2 = components.filter(function (bundle) { return bundle.includes('@pie-element/passage'); });
        // Create new URLs
        var url1 = baseUrl + "/bundles/" + group1.join('+') + "/player.js";
        var url2 = baseUrl + "/bundles/" + group2.join('+') + "/player.js";
        return [[url1], [url2]];
    }
    return [[], []];
};
// Create a single shared instance of FetchUtils for GraphQL operations
var fetchUtils = new FetchUtils({ source: "graphql-client" });
var removeVersionFromId = function (id) {
    var idParts = id.split('@');
    return idParts[0];
};
var loadBestVersions = function (token, apiUrl) { return __awaiter(void 0, void 0, void 0, function () {
    var query, graphqlEndpoint, result, err_1;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                if (!token) {
                    console.log("no token - skip");
                    return [2 /*return*/];
                }
                query = "query {\n      findAllBestVersions {\n        pie\n        version\n      }\n    }";
                graphqlEndpoint = apiUrl + "/graphql";
                _b.label = 1;
            case 1:
                _b.trys.push([1, 3, , 4]);
                return [4 /*yield*/, fetchUtils.executeGraphQL(graphqlEndpoint, token, query)];
            case 2:
                result = _b.sent();
                if (result.errors) {
                    console.log("ERRORS:", result.errors);
                    throw new Error(result.errors[0].message);
                }
                return [2 /*return*/, result.data.findAllBestVersions];
            case 3:
                err_1 = _b.sent();
                console.log(err_1);
                throw new Error("error with server request " + query);
            case 4: return [2 /*return*/];
        }
    });
}); };
var saveContentItem = function (token, versionedID, releaseType, config, collections, apiUrl) {
    if (collections === void 0) { collections = []; }
    if (apiUrl === void 0) { apiUrl = HOST_DEFAULTS.prod.apiUrl; }
    return __awaiter(void 0, void 0, void 0, function () {
        var query, itemInput, variables, graphqlEndpoint, saveResult, err_2;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    versionedID = versionedID ? versionedID : null;
                    query = "\n    mutation SaveItem($input: SaveItemInput!) {\n      saveItem(input: $input) {\n        id\n        name\n        retired\n        published\n        searchMetaData\n        passage {\n          id\n          name\n          version {\n            major\n            minor\n            patch\n            prerelease\n          }\n        }\n        version {\n          major\n          minor\n          patch\n          prerelease\n        }\n        createdAt\n        updatedAt\n      }\n    }\n  ";
                    itemInput = {
                        config: config,
                        searchMetaData: {},
                    };
                    if (versionedID) {
                        // Do not send id with version
                        itemInput['id'] = removeVersionFromId(versionedID);
                    }
                    variables = {
                        input: {
                            item: itemInput,
                            releaseType: releaseType // (optional release type, defaults to ReleaseType.PRERELEASE)
                        },
                    };
                    if (collections) {
                        variables.input['collectionIds'] = collections; // (optional collection IDs)
                    }
                    graphqlEndpoint = apiUrl + "/graphql";
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, fetchUtils.executeGraphQL(graphqlEndpoint, token, query, variables)];
                case 2:
                    saveResult = _b.sent();
                    if (saveResult && saveResult.data.saveItem.id) {
                        return [2 /*return*/, versionedIdtoString({
                                id: saveResult.data.saveItem.id,
                                version: saveResult.data.saveItem.version,
                            })];
                    }
                    return [3 /*break*/, 4];
                case 3:
                    err_2 = _b.sent();
                    console.log(err_2);
                    throw new Error("error with server request " + query);
                case 4: return [2 /*return*/];
            }
        });
    });
};
var releaseContentItem = function (token, versionedID, apiUrl) {
    if (apiUrl === void 0) { apiUrl = HOST_DEFAULTS.prod.apiUrl; }
    return __awaiter(void 0, void 0, void 0, function () {
        var vId, graphqlEndpoint, query, res, err_3;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    vId = getVersionedId(versionedID);
                    graphqlEndpoint = apiUrl + "/graphql";
                    query = "\n    mutation ReleaseItem($id: VersionedID!) {\n      releaseItem(vId: $id) {\n        id\n        version {\n          major\n          minor\n          patch\n          prerelease\n        }\n      }\n    }\n  ";
                    if (!isPreRelease(vId)) return [3 /*break*/, 5];
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, fetchUtils.executeGraphQL(graphqlEndpoint, token, query, {
                            id: versionedID
                        })];
                case 2:
                    res = _b.sent();
                    if (res &&
                        res.data &&
                        res.data.releaseItem &&
                        res.data.releaseItem.id) {
                        return [2 /*return*/, versionedIdtoString({
                                id: res.data.releaseItem.id,
                                version: res.data.releaseItem.version,
                            })];
                    }
                    return [3 /*break*/, 4];
                case 3:
                    err_3 = _b.sent();
                    console.log(err_3);
                    throw new Error("error with server request " + query);
                case 4: return [3 /*break*/, 6];
                case 5: return [2 /*return*/, Promise.reject("id does not have a pre-release version")];
                case 6: return [2 /*return*/];
            }
        });
    });
};
var loadFromPieApi = function (token, versionedID, apiUrl, overrides) {
    if (apiUrl === void 0) { apiUrl = HOST_DEFAULTS.prod.apiUrl; }
    return __awaiter(void 0, void 0, void 0, function () {
        var graphqlEndpoint, query, res, err_4;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    graphqlEndpoint = apiUrl + "/graphql";
                    query = "\n    query Item($id: VersionedID!, $opts: ItemLoadOptions) {\n      item(vId: $id, opts: $opts) {\n        id\n        name\n        config\n        searchMetaData\n        version {\n          major\n          minor\n          patch\n          prerelease\n        }\n        passage {\n          id\n          name\n          config\n          searchMetaData\n          version {\n            major\n            minor\n            patch\n            prerelease\n          }\n        }\n      }\n    }\n  ";
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, fetchUtils.executeGraphQL(graphqlEndpoint, token, query, {
                            id: versionedID,
                            opts: {
                                overrides: overrides
                            }
                        })];
                case 2:
                    res = _b.sent();
                    return [2 /*return*/, res && res.data ? res.data.item : null];
                case 3:
                    err_4 = _b.sent();
                    console.log(err_4);
                    throw new Error("error with server request " + query);
                case 4: return [2 /*return*/];
            }
        });
    });
};
export { FetchUtils as F, HOST_DEFAULTS as H, Mode as M, NewRelicEnabledClient as N, PLAYER_HOST_CONFIG as P, Role as R, VERSION as V, NewRelicInstrumentationService as a, loadBestVersions as b, lockConfigVersions as c, saveContentItem as d, idInfo as i, loadFromPieApi as l, releaseContentItem as r, splitStimulusItemsBundle as s, unlockConfigVersions as u, versionedIdtoString as v };
