import { r as registerInstance, c as createEvent, d as getContext, h, e as getElement } from './core-43f43fa0.js';
import './_commonjsHelpers-91036208.js';
import { p as parseNpm } from './index-801c8326.js';
import { t as isArrayLike, i as isArray, f as isBuffer, j as isTypedArray, o as isArguments, d as getTag, u as isPrototype, v as baseKeys, k as isObjectLike, w as baseGetTag, x as getPrototype, S as Symbol, y as arrayPush, z as defineProperty, A as copyObject, q as getAllKeysIn, B as baseClone, b as isObject, n as isIndex, e as eq, c as cloneDeep } from './isSymbol-f32647bc.js';
import { p as pieContentFromConfig, d as createTag } from './utils-77e24cb0.js';
import { e as baseGet, c as castPath, t as toKey, f as arrayMap, h as complexRubricChecks, r as removeComplexRubricFromMarkup, C as COMPLEX_RUBRIC, a as addPackageToContent, j as addModelToContent, k as addComplexRubric, b as addRubric, l as addMultiTraitRubric } from './get-b9d5808f.js';
import { t as toNumber } from './toNumber-e03abca3.js';
import { l as lib_3, a as lib_2, b as lib_5, c as lib_4, d as lib_1 } from './index-0c37ca4e.js';
import { b as baseIsEqual, i as identity, P as PieLoader, V as VERSION, D as DEFAULT_ENDPOINTS, m as mathRenderingAccessible$1 } from './version-a0ee00c5.js';

/** `Object#toString` result references. */
var mapTag = '[object Map]',
    setTag = '[object Set]';

/** Used for built-in method references. */
var objectProto = Object.prototype;

/** Used to check objects for own properties. */
var hasOwnProperty = objectProto.hasOwnProperty;

/**
 * Checks if `value` is an empty object, collection, map, or set.
 *
 * Objects are considered empty if they have no own enumerable string keyed
 * properties.
 *
 * Array-like values such as `arguments` objects, arrays, buffers, strings, or
 * jQuery-like collections are considered empty if they have a `length` of `0`.
 * Similarly, maps and sets are considered empty if they have a `size` of `0`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is empty, else `false`.
 * @example
 *
 * _.isEmpty(null);
 * // => true
 *
 * _.isEmpty(true);
 * // => true
 *
 * _.isEmpty(1);
 * // => true
 *
 * _.isEmpty([1, 2, 3]);
 * // => false
 *
 * _.isEmpty({ 'a': 1 });
 * // => false
 */
function isEmpty(value) {
  if (value == null) {
    return true;
  }
  if (isArrayLike(value) &&
      (isArray(value) || typeof value == 'string' || typeof value.splice == 'function' ||
        isBuffer(value) || isTypedArray(value) || isArguments(value))) {
    return !value.length;
  }
  var tag = getTag(value);
  if (tag == mapTag || tag == setTag) {
    return !value.size;
  }
  if (isPrototype(value)) {
    return !baseKeys(value).length;
  }
  for (var key in value) {
    if (hasOwnProperty.call(value, key)) {
      return false;
    }
  }
  return true;
}

var isEmpty_1 = isEmpty;

/**
 * Performs a deep comparison between two values to determine if they are
 * equivalent.
 *
 * **Note:** This method supports comparing arrays, array buffers, booleans,
 * date objects, error objects, maps, numbers, `Object` objects, regexes,
 * sets, strings, symbols, and typed arrays. `Object` objects are compared
 * by their own, not inherited, enumerable properties. Functions and DOM
 * nodes are compared by strict equality, i.e. `===`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to compare.
 * @param {*} other The other value to compare.
 * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
 * @example
 *
 * var object = { 'a': 1 };
 * var other = { 'a': 1 };
 *
 * _.isEqual(object, other);
 * // => true
 *
 * object === other;
 * // => false
 */
function isEqual(value, other) {
  return baseIsEqual(value, other);
}

var isEqual_1 = isEqual;

/**
 * Gets the last element of `array`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Array
 * @param {Array} array The array to query.
 * @returns {*} Returns the last element of `array`.
 * @example
 *
 * _.last([1, 2, 3]);
 * // => 3
 */
function last(array) {
  var length = array == null ? 0 : array.length;
  return length ? array[length - 1] : undefined;
}

var last_1 = last;

/**
 * The base implementation of `_.slice` without an iteratee call guard.
 *
 * @private
 * @param {Array} array The array to slice.
 * @param {number} [start=0] The start position.
 * @param {number} [end=array.length] The end position.
 * @returns {Array} Returns the slice of `array`.
 */
function baseSlice(array, start, end) {
  var index = -1,
      length = array.length;

  if (start < 0) {
    start = -start > length ? 0 : (length + start);
  }
  end = end > length ? length : end;
  if (end < 0) {
    end += length;
  }
  length = start > end ? 0 : ((end - start) >>> 0);
  start >>>= 0;

  var result = Array(length);
  while (++index < length) {
    result[index] = array[index + start];
  }
  return result;
}

var _baseSlice = baseSlice;

/**
 * Gets the parent value at `path` of `object`.
 *
 * @private
 * @param {Object} object The object to query.
 * @param {Array} path The path to get the parent value of.
 * @returns {*} Returns the parent value.
 */
function parent(object, path) {
  return path.length < 2 ? object : baseGet(object, _baseSlice(path, 0, -1));
}

var _parent = parent;

/**
 * The base implementation of `_.unset`.
 *
 * @private
 * @param {Object} object The object to modify.
 * @param {Array|string} path The property path to unset.
 * @returns {boolean} Returns `true` if the property is deleted, else `false`.
 */
function baseUnset(object, path) {
  path = castPath(path, object);
  object = _parent(object, path);
  return object == null || delete object[toKey(last_1(path))];
}

var _baseUnset = baseUnset;

/** `Object#toString` result references. */
var objectTag = '[object Object]';

/** Used for built-in method references. */
var funcProto = Function.prototype,
    objectProto$1 = Object.prototype;

/** Used to resolve the decompiled source of functions. */
var funcToString = funcProto.toString;

/** Used to check objects for own properties. */
var hasOwnProperty$1 = objectProto$1.hasOwnProperty;

/** Used to infer the `Object` constructor. */
var objectCtorString = funcToString.call(Object);

/**
 * Checks if `value` is a plain object, that is, an object created by the
 * `Object` constructor or one with a `[[Prototype]]` of `null`.
 *
 * @static
 * @memberOf _
 * @since 0.8.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a plain object, else `false`.
 * @example
 *
 * function Foo() {
 *   this.a = 1;
 * }
 *
 * _.isPlainObject(new Foo);
 * // => false
 *
 * _.isPlainObject([1, 2, 3]);
 * // => false
 *
 * _.isPlainObject({ 'x': 0, 'y': 0 });
 * // => true
 *
 * _.isPlainObject(Object.create(null));
 * // => true
 */
function isPlainObject(value) {
  if (!isObjectLike(value) || baseGetTag(value) != objectTag) {
    return false;
  }
  var proto = getPrototype(value);
  if (proto === null) {
    return true;
  }
  var Ctor = hasOwnProperty$1.call(proto, 'constructor') && proto.constructor;
  return typeof Ctor == 'function' && Ctor instanceof Ctor &&
    funcToString.call(Ctor) == objectCtorString;
}

var isPlainObject_1 = isPlainObject;

/**
 * Used by `_.omit` to customize its `_.cloneDeep` use to only clone plain
 * objects.
 *
 * @private
 * @param {*} value The value to inspect.
 * @param {string} key The key of the property to inspect.
 * @returns {*} Returns the uncloned value or `undefined` to defer cloning to `_.cloneDeep`.
 */
function customOmitClone(value) {
  return isPlainObject_1(value) ? undefined : value;
}

var _customOmitClone = customOmitClone;

/** Built-in value references. */
var spreadableSymbol = Symbol ? Symbol.isConcatSpreadable : undefined;

/**
 * Checks if `value` is a flattenable `arguments` object or array.
 *
 * @private
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is flattenable, else `false`.
 */
function isFlattenable(value) {
  return isArray(value) || isArguments(value) ||
    !!(spreadableSymbol && value && value[spreadableSymbol]);
}

var _isFlattenable = isFlattenable;

/**
 * The base implementation of `_.flatten` with support for restricting flattening.
 *
 * @private
 * @param {Array} array The array to flatten.
 * @param {number} depth The maximum recursion depth.
 * @param {boolean} [predicate=isFlattenable] The function invoked per iteration.
 * @param {boolean} [isStrict] Restrict to values that pass `predicate` checks.
 * @param {Array} [result=[]] The initial result value.
 * @returns {Array} Returns the new flattened array.
 */
function baseFlatten(array, depth, predicate, isStrict, result) {
  var index = -1,
      length = array.length;

  predicate || (predicate = _isFlattenable);
  result || (result = []);

  while (++index < length) {
    var value = array[index];
    if (depth > 0 && predicate(value)) {
      if (depth > 1) {
        // Recursively flatten arrays (susceptible to call stack limits).
        baseFlatten(value, depth - 1, predicate, isStrict, result);
      } else {
        arrayPush(result, value);
      }
    } else if (!isStrict) {
      result[result.length] = value;
    }
  }
  return result;
}

var _baseFlatten = baseFlatten;

/**
 * Flattens `array` a single level deep.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Array
 * @param {Array} array The array to flatten.
 * @returns {Array} Returns the new flattened array.
 * @example
 *
 * _.flatten([1, [2, [3, [4]], 5]]);
 * // => [1, 2, [3, [4]], 5]
 */
function flatten(array) {
  var length = array == null ? 0 : array.length;
  return length ? _baseFlatten(array, 1) : [];
}

var flatten_1 = flatten;

/**
 * A faster alternative to `Function#apply`, this function invokes `func`
 * with the `this` binding of `thisArg` and the arguments of `args`.
 *
 * @private
 * @param {Function} func The function to invoke.
 * @param {*} thisArg The `this` binding of `func`.
 * @param {Array} args The arguments to invoke `func` with.
 * @returns {*} Returns the result of `func`.
 */
function apply(func, thisArg, args) {
  switch (args.length) {
    case 0: return func.call(thisArg);
    case 1: return func.call(thisArg, args[0]);
    case 2: return func.call(thisArg, args[0], args[1]);
    case 3: return func.call(thisArg, args[0], args[1], args[2]);
  }
  return func.apply(thisArg, args);
}

var _apply = apply;

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeMax = Math.max;

/**
 * A specialized version of `baseRest` which transforms the rest array.
 *
 * @private
 * @param {Function} func The function to apply a rest parameter to.
 * @param {number} [start=func.length-1] The start position of the rest parameter.
 * @param {Function} transform The rest array transform.
 * @returns {Function} Returns the new function.
 */
function overRest(func, start, transform) {
  start = nativeMax(start === undefined ? (func.length - 1) : start, 0);
  return function() {
    var args = arguments,
        index = -1,
        length = nativeMax(args.length - start, 0),
        array = Array(length);

    while (++index < length) {
      array[index] = args[start + index];
    }
    index = -1;
    var otherArgs = Array(start + 1);
    while (++index < start) {
      otherArgs[index] = args[index];
    }
    otherArgs[start] = transform(array);
    return _apply(func, this, otherArgs);
  };
}

var _overRest = overRest;

/**
 * Creates a function that returns `value`.
 *
 * @static
 * @memberOf _
 * @since 2.4.0
 * @category Util
 * @param {*} value The value to return from the new function.
 * @returns {Function} Returns the new constant function.
 * @example
 *
 * var objects = _.times(2, _.constant({ 'a': 1 }));
 *
 * console.log(objects);
 * // => [{ 'a': 1 }, { 'a': 1 }]
 *
 * console.log(objects[0] === objects[1]);
 * // => true
 */
function constant(value) {
  return function() {
    return value;
  };
}

var constant_1 = constant;

/**
 * The base implementation of `setToString` without support for hot loop shorting.
 *
 * @private
 * @param {Function} func The function to modify.
 * @param {Function} string The `toString` result.
 * @returns {Function} Returns `func`.
 */
var baseSetToString = !defineProperty ? identity : function(func, string) {
  return defineProperty(func, 'toString', {
    'configurable': true,
    'enumerable': false,
    'value': constant_1(string),
    'writable': true
  });
};

var _baseSetToString = baseSetToString;

/** Used to detect hot functions by number of calls within a span of milliseconds. */
var HOT_COUNT = 800,
    HOT_SPAN = 16;

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeNow = Date.now;

/**
 * Creates a function that'll short out and invoke `identity` instead
 * of `func` when it's called `HOT_COUNT` or more times in `HOT_SPAN`
 * milliseconds.
 *
 * @private
 * @param {Function} func The function to restrict.
 * @returns {Function} Returns the new shortable function.
 */
function shortOut(func) {
  var count = 0,
      lastCalled = 0;

  return function() {
    var stamp = nativeNow(),
        remaining = HOT_SPAN - (stamp - lastCalled);

    lastCalled = stamp;
    if (remaining > 0) {
      if (++count >= HOT_COUNT) {
        return arguments[0];
      }
    } else {
      count = 0;
    }
    return func.apply(undefined, arguments);
  };
}

var _shortOut = shortOut;

/**
 * Sets the `toString` method of `func` to return `string`.
 *
 * @private
 * @param {Function} func The function to modify.
 * @param {Function} string The `toString` result.
 * @returns {Function} Returns `func`.
 */
var setToString = _shortOut(_baseSetToString);

var _setToString = setToString;

/**
 * A specialized version of `baseRest` which flattens the rest array.
 *
 * @private
 * @param {Function} func The function to apply a rest parameter to.
 * @returns {Function} Returns the new function.
 */
function flatRest(func) {
  return _setToString(_overRest(func, undefined, flatten_1), func + '');
}

var _flatRest = flatRest;

/** Used to compose bitmasks for cloning. */
var CLONE_DEEP_FLAG = 1,
    CLONE_FLAT_FLAG = 2,
    CLONE_SYMBOLS_FLAG = 4;

/**
 * The opposite of `_.pick`; this method creates an object composed of the
 * own and inherited enumerable property paths of `object` that are not omitted.
 *
 * **Note:** This method is considerably slower than `_.pick`.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Object
 * @param {Object} object The source object.
 * @param {...(string|string[])} [paths] The property paths to omit.
 * @returns {Object} Returns the new object.
 * @example
 *
 * var object = { 'a': 1, 'b': '2', 'c': 3 };
 *
 * _.omit(object, ['a', 'c']);
 * // => { 'b': '2' }
 */
var omit = _flatRest(function(object, paths) {
  var result = {};
  if (object == null) {
    return result;
  }
  var isDeep = false;
  paths = arrayMap(paths, function(path) {
    path = castPath(path, object);
    isDeep || (isDeep = path.length > 1);
    return path;
  });
  copyObject(object, getAllKeysIn(object), result);
  if (isDeep) {
    result = baseClone(result, CLONE_DEEP_FLAG | CLONE_FLAT_FLAG | CLONE_SYMBOLS_FLAG, _customOmitClone);
  }
  var length = paths.length;
  while (length--) {
    _baseUnset(result, paths[length]);
  }
  return result;
});

var omit_1 = omit;

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeCeil = Math.ceil,
    nativeMax$1 = Math.max;

/**
 * The base implementation of `_.range` and `_.rangeRight` which doesn't
 * coerce arguments.
 *
 * @private
 * @param {number} start The start of the range.
 * @param {number} end The end of the range.
 * @param {number} step The value to increment or decrement by.
 * @param {boolean} [fromRight] Specify iterating from right to left.
 * @returns {Array} Returns the range of numbers.
 */
function baseRange(start, end, step, fromRight) {
  var index = -1,
      length = nativeMax$1(nativeCeil((end - start) / (step || 1)), 0),
      result = Array(length);

  while (length--) {
    result[fromRight ? length : ++index] = start;
    start += step;
  }
  return result;
}

var _baseRange = baseRange;

/**
 * Checks if the given arguments are from an iteratee call.
 *
 * @private
 * @param {*} value The potential iteratee value argument.
 * @param {*} index The potential iteratee index or key argument.
 * @param {*} object The potential iteratee object argument.
 * @returns {boolean} Returns `true` if the arguments are from an iteratee call,
 *  else `false`.
 */
function isIterateeCall(value, index, object) {
  if (!isObject(object)) {
    return false;
  }
  var type = typeof index;
  if (type == 'number'
        ? (isArrayLike(object) && isIndex(index, object.length))
        : (type == 'string' && index in object)
      ) {
    return eq(object[index], value);
  }
  return false;
}

var _isIterateeCall = isIterateeCall;

/** Used as references for various `Number` constants. */
var INFINITY = 1 / 0,
    MAX_INTEGER = 1.7976931348623157e+308;

/**
 * Converts `value` to a finite number.
 *
 * @static
 * @memberOf _
 * @since 4.12.0
 * @category Lang
 * @param {*} value The value to convert.
 * @returns {number} Returns the converted number.
 * @example
 *
 * _.toFinite(3.2);
 * // => 3.2
 *
 * _.toFinite(Number.MIN_VALUE);
 * // => 5e-324
 *
 * _.toFinite(Infinity);
 * // => 1.7976931348623157e+308
 *
 * _.toFinite('3.2');
 * // => 3.2
 */
function toFinite(value) {
  if (!value) {
    return value === 0 ? value : 0;
  }
  value = toNumber(value);
  if (value === INFINITY || value === -INFINITY) {
    var sign = (value < 0 ? -1 : 1);
    return sign * MAX_INTEGER;
  }
  return value === value ? value : 0;
}

var toFinite_1 = toFinite;

/**
 * Creates a `_.range` or `_.rangeRight` function.
 *
 * @private
 * @param {boolean} [fromRight] Specify iterating from right to left.
 * @returns {Function} Returns the new range function.
 */
function createRange(fromRight) {
  return function(start, end, step) {
    if (step && typeof step != 'number' && _isIterateeCall(start, end, step)) {
      end = step = undefined;
    }
    // Ensure the sign of `-0` is preserved.
    start = toFinite_1(start);
    if (end === undefined) {
      end = start;
      start = 0;
    } else {
      end = toFinite_1(end);
    }
    step = step === undefined ? (start < end ? 1 : -1) : toFinite_1(step);
    return _baseRange(start, end, step, fromRight);
  };
}

var _createRange = createRange;

/**
 * Creates an array of numbers (positive and/or negative) progressing from
 * `start` up to, but not including, `end`. A step of `-1` is used if a negative
 * `start` is specified without an `end` or `step`. If `end` is not specified,
 * it's set to `start` with `start` then set to `0`.
 *
 * **Note:** JavaScript follows the IEEE-754 standard for resolving
 * floating-point values which can produce unexpected results.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Util
 * @param {number} [start=0] The start of the range.
 * @param {number} end The end of the range.
 * @param {number} [step=1] The value to increment or decrement by.
 * @returns {Array} Returns the range of numbers.
 * @see _.inRange, _.rangeRight
 * @example
 *
 * _.range(4);
 * // => [0, 1, 2, 3]
 *
 * _.range(-4);
 * // => [0, -1, -2, -3]
 *
 * _.range(1, 5);
 * // => [1, 2, 3, 4]
 *
 * _.range(0, 20, 5);
 * // => [0, 5, 10, 15]
 *
 * _.range(0, -4, -1);
 * // => [0, -1, -2, -3]
 *
 * _.range(1, 4, 0);
 * // => [1, 1, 1]
 *
 * _.range(0);
 * // => []
 */
var range = _createRange();

var range_1 = range;

class DataURLImageSupport {
    insert(file, done, progressFn) {
        const reader = new FileReader();
        reader.onload = () => {
            const dataURL = reader.result;
            setTimeout(() => {
                done(null, dataURL.toString());
            }, 2000);
        };
        let progress = 0;
        progressFn(progress, 0, 100);
        range_1(1, 100).forEach(n => {
            setTimeout(() => {
                progressFn(n, n, 100);
            }, n * 20);
        });
        // if external asset support .. add reader.readAsArrayBuffer ..
        reader.readAsDataURL(file);
    }
    delete(src, done) {
        done();
    }
}

class DataURLUploadSoundSupport {
    insert(file, done, progressFn) {
        const reader = new FileReader();
        reader.onload = () => {
            const dataURL = reader.result;
            setTimeout(() => {
                done(null, dataURL.toString());
            }, 2000);
        };
        let progress = 0;
        progressFn(progress, 0, 100);
        range_1(1, 100).forEach(n => {
            setTimeout(() => {
                progressFn(n, n, 100);
            }, n * 20);
        });
        // if external asset support .. add reader.readAsArrayBuffer ..
        reader.readAsDataURL(file);
    }
    delete(src, done) {
        done();
    }
}

const Author = class {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        this._modelLoadedState = false;
        /**
         * Allows disabling of the default behaviour which is to look up and load the JS bundle that define the Custom Elements
         * used by the item config.
         * This if for advanced use cases when using the pie-player in a container that is managing loading of Custom Elements and Controllers.
         */
        this.disableBundler = false;
        /**
         * Adds a preview view which will render the content in another tab as it may appear to a student or instructor.
         */
        this.addPreview = false;
        this.elementsLoaded = false;
        /**
         * If pie-author is used inside pie-api-author component. Do not set it manually.
         */
        this.isInsidePieApiAuthor = false;
        this.pieLoader = new PieLoader(null, this.loaderConfig);
        this.fileInput = null;
        this.imageHandler = null;
        this.dialogOpened = false;
        this.changeHandled = false;
        this.componentLoaded = false;
        /** external providers can set this if they need to upload the assets to the cloud etc. by default we use data urls */
        this.imageSupport = new DataURLImageSupport();
        /** external providers can set this if they need to upload the assets to the cloud etc. by default we use data urls */
        this.uploadSoundSupport = new DataURLUploadSoundSupport();
        this.version = VERSION;
        /**
         * used in our demo environment to allow author to watch config settings and make updates
         * defaults to false (do not set it to true because it was not tested properly)
         * @type {boolean}
         * @private
         */
        this.canWatchConfigSettings = false;
        /**
         * used to automatically re-fetch the bundle (in case we get a 503)
         */
        this.reFetchBundle = false;
        this.isAdvancedItemConfig = (config) => config.pie;
        this.getElementByType = (elements, type) => {
            const result = [];
            for (const [key, value] of Object.entries(elements || {})) {
                if (value.startsWith(type)) {
                    result.push(key);
                }
            }
            return result;
        };
        this.getPieContentModelWithToggledComplexRubric = async ({ complexRubricCheckedValues, pieContentModel }) => {
            const { shouldAddComplexRubric, shouldRemoveComplexRubric, complexRubricElements } = complexRubricCheckedValues || {};
            if (shouldAddComplexRubric) {
                let clonedModel = cloneDeep(pieContentModel);
                // we have to convert the rubric to complex-rubric, so get the default values from it
                const rubricNames = this.getElementByType(pieContentModel.elements, "@pie-element/rubric");
                for (const rubricName of rubricNames) {
                    const rubric = ((pieContentModel && pieContentModel.models) || []).find(el => el.element === rubricName);
                    if (rubric) {
                        const simpleRubric = omit_1(rubric, ["id", "element"]);
                        this.defaultComplexRubricModel = {
                            rubrics: {
                                simpleRubric
                            }
                        };
                        clonedModel = this.removeRubricItemTypes(clonedModel, rubric.id, rubric.element);
                    }
                }
                // we add complex-rubric to config
                return this.addComplexRubric(clonedModel);
            }
            if (shouldRemoveComplexRubric) {
                // we remove complex-rubric from config
                return this.removeComplexRubricItemTypes(cloneDeep(pieContentModel), complexRubricElements);
            }
            return null;
        };
        this.getNewConfig = newConfig => {
            if (this.isAdvancedItemConfig(this.config)) {
                return Object.assign(Object.assign({}, this.config), { pie: newConfig });
            }
            return newConfig;
        };
        this.setModelsAndConfigurations = () => {
            if (this.pieContentModel && this.pieContentModel.models) {
                this.pieContentModel.models.map(model => {
                    let pieEl = this.el.querySelector(`[id='${model.id}']`);
                    !pieEl && (pieEl = this.el.querySelector(`[pie-id='${model.id}']`));
                    if (pieEl) {
                        const pieElName = pieEl.tagName.toLowerCase().split("-config")[0];
                        try {
                            const packageName = parseNpm(this.pieContentModel.elements[pieElName]).name;
                            pieEl.model = model;
                            if (this.configSettings && this.configSettings[packageName]) {
                                pieEl.configuration = this.configSettings[packageName];
                            }
                        }
                        catch (e) {
                            console.log(e.toString(), pieElName, this.pieContentModel.elements[pieElName]);
                        }
                    }
                });
            }
        };
        this.handleFileInputChange = (e) => {
            const input = e.target;
            this.dialogOpened = false;
            if (!this.imageHandler) {
                console.error("no image handler - but file input change triggered?");
                return;
            }
            this.changeHandled = true;
            const files = input.files;
            if (files.length < 1 || !files[0]) {
                this.imageHandler.cancel();
                this.imageHandler = null;
            }
            else {
                const file = files[0];
                this.imageHandler.fileChosen(file);
                this.fileInput.value = "";
                this.insertImage(file);
            }
        };
        this.handleWindowFocus = () => {
            if (!this.fileInput || !this.dialogOpened) {
                return;
            }
            this.dialogOpened = false;
            // needed in order for the fileInput to be populated with the file
            setTimeout(() => {
                if (this.fileInput.files.length === 0 && !this.changeHandled) {
                    // it means the user clicked cancel
                    this.imageHandler.cancel();
                    this.imageHandler = null;
                    this.changeHandled = true;
                }
            }, 500);
        };
        this.handleInsertImage = (e) => {
            console.log("[handleInsertImage]", e);
            this.imageHandler = e.detail;
            if (!e.detail.isPasted) {
                this.fileInput.click();
                this.dialogOpened = true;
                this.changeHandled = false;
            }
            else if (e.detail.getChosenFile) {
                // this is for images that were pasted into the editor (or dropped)
                // they also need to be uploaded, but the file input doesn't have to be used
                try {
                    const file = e.detail.getChosenFile();
                    if (file) {
                        this.insertImage(file);
                    }
                }
                catch (e) {
                    console.error(e);
                }
            }
        };
        this.handleDeleteImage = (e) => {
            console.log("[handleDeleteImage ..]", e);
            this.imageSupport.delete(e.detail.src, e.detail.done);
        };
        this.handleInsertSound = (e) => {
            console.log("[handleInsertSound]", e);
            this.uploadSoundSupport.insert(e.detail.fileChosen, e.detail.done);
        };
        this.handleDeleteSound = (e) => {
            console.log("[handleDeleteSound ..]", e);
            this.uploadSoundSupport.delete(e.detail.src, e.detail.done);
        };
        this.modelUpdated = createEvent(this, "modelUpdated", 7);
        this.modelLoaded = createEvent(this, "modelLoaded", 7);
        this.doc = getContext(this, "document");
    }
    async validateModels() {
        if (!this.pieContentModel || !this.pieContentModel.models) {
            console.error("No pie content model");
            return { hasErrors: false, validatedModels: {} };
        }
        return (this.pieContentModel.models || []).reduce((acc, model, index) => {
            let pieEl = this.el.querySelector(`[id='${model.id}']`);
            !pieEl && (pieEl = this.el.querySelector(`[pie-id='${model.id}']`));
            if (pieEl) {
                const pieElName = pieEl.tagName.toLowerCase().split("-config")[0];
                const controller = this.pieLoader.getController(pieElName);
                const packageName = parseNpm(this.pieContentModel.elements[pieElName])
                    .name;
                const configuration = (this.configSettings && this.configSettings[packageName]) || {};
                if (controller && controller.validate) {
                    // here we call controller.validate which returns an object with all the errors
                    const errors = controller.validate(model, configuration);
                    const errorsAreTheSame = isEqual_1(model && model.errors, errors);
                    // Only update models if anything changed to errors object, otherwise, we'll keep calling onModelUpdated and again validateModels
                    // for complex items (like ebsr, complex-rubric, who are using other items)
                    if (!errorsAreTheSame) {
                        // added this to ensure that this.pieContentModel.model is updated with the correct errors
                        // TODO: figure out what's actually happening and if there is a better way to make sure that this.pieContentModels.models is actually updated
                        this.pieContentModel.models[index] = Object.assign(Object.assign({}, model), { errors });
                        // here we can update the model in author, so we can set errors
                        pieEl.model = Object.assign(Object.assign({}, model), { errors });
                    }
                    // for ebsr
                    if (errors && errors.partA && errors.partB) {
                        acc.hasErrors =
                            acc.hasErrors ||
                                (!isEmpty_1(errors.partA) || !isEmpty_1(errors.partB));
                    }
                    else {
                        // here we return a boolean value if models are valid or not
                        acc.hasErrors = acc.hasErrors || !isEmpty_1(errors);
                    }
                    acc.validatedModels = Object.assign(Object.assign({}, acc.validatedModels), { [model.id]: Object.assign(Object.assign({}, model), { errors }) });
                }
                else {
                    acc.validatedModels = Object.assign(Object.assign({}, acc.validatedModels), { [model.id]: model });
                }
            }
            return acc;
        }, { hasErrors: false, validatedModels: {} });
    }
    insertImage(file) {
        this.imageSupport.insert(file, (e, src) => {
            if (e) {
                console.warn("error inserting image: ", e.message);
                console.error(e);
            }
            this.imageHandler.done(e, src);
            this.imageHandler = null;
        }, (percent, bytes, total) => this.imageHandler.progress(percent, bytes, total));
    }
    getRenderMarkup() {
        let markup = this.pieContentModel ? this.pieContentModel.markup : "";
        if (markup) {
            Object.keys(this.pieContentModel.elements).forEach(key => {
                markup = markup.split(key).join(key + "-config");
            });
            return markup;
        }
    }
    async watchAddPreview() {
        // if add preview changes, make sure to reset the configuration
        // this is not a common use case
        if (this.componentLoaded) {
            await this.watchConfig(this.config, {});
        }
    }
    async watchConfig(newValue, oldValue) {
        if (newValue && !isEqual_1(newValue, oldValue)) {
            try {
                this.elementsLoaded = false;
                this._modelLoadedState = false;
                const pieContentModel = pieContentFromConfig(newValue);
                const complexRubricCheckedValues = complexRubricChecks(pieContentModel, this.configSettings);
                const { shouldAddComplexRubric, shouldRemoveComplexRubric } = complexRubricCheckedValues || {};
                // if changes are needed
                if (shouldAddComplexRubric || shouldRemoveComplexRubric) {
                    this.pieContentModel = null;
                    const newConfig = await this.getPieContentModelWithToggledComplexRubric({
                        complexRubricCheckedValues,
                        pieContentModel
                    });
                    // and then we reset the config
                    if (newConfig) {
                        this.config = this.getNewConfig(newConfig);
                    }
                }
                else {
                    // if there's no change needed, we can proceed with updating the config and loading pie-elements
                    this.pieContentModel = pieContentModel;
                    this.addConfigTags(this.pieContentModel);
                    await this.loadPieElements();
                }
            }
            catch (error) {
                console.log(`ERROR ${error}`);
            }
        }
    }
    removeRubricItemTypes(pieContentModel, rubricId, rubricElement) {
        if (!rubricElement || !pieContentModel.models) {
            return pieContentModel;
        }
        // delete the rubric elements from elements object
        delete pieContentModel.elements[rubricElement];
        const { markupWithoutComplexRubric } = removeComplexRubricFromMarkup(pieContentModel, [rubricElement], this.doc);
        // delete the rubric nodes from markup
        pieContentModel.markup = markupWithoutComplexRubric;
        // delete the rubric models
        pieContentModel.models = (pieContentModel.models || []).filter(model => rubricId !== model.id);
        return pieContentModel;
    }
    async removeComplexRubricItemTypes(pieContentModel, rubricElements) {
        if (!rubricElements.length || !pieContentModel.models) {
            return pieContentModel;
        }
        // delete the complex-rubric elements from elements object
        rubricElements.forEach(rubricElementKey => delete pieContentModel.elements[rubricElementKey]);
        const { markupWithoutComplexRubric, deletedComplexRubricItemIds } = removeComplexRubricFromMarkup(pieContentModel, rubricElements, this.doc);
        // delete the complex-rubric nodes from markup
        pieContentModel.markup = markupWithoutComplexRubric;
        // delete the complex-rubric models
        pieContentModel.models = pieContentModel.models.filter(model => !deletedComplexRubricItemIds.includes(model.id));
        return pieContentModel;
    }
    async addComplexRubric(pieContentModel) {
        const complexRubricModel = Object.assign({ id: COMPLEX_RUBRIC, element: `pie-${COMPLEX_RUBRIC}` }, (this.defaultComplexRubricModel || {}));
        const packageName = `@pie-element/${COMPLEX_RUBRIC}`;
        // add complex-rubric in elements, IF it isn't there already
        let content = addPackageToContent(pieContentModel, packageName, complexRubricModel);
        // but if complex-rubric is already in elements, then just add it in the models
        if (!content) {
            content = addModelToContent(pieContentModel, createTag(packageName), complexRubricModel);
        }
        return addComplexRubric(content);
    }
    addConfigTags(c) {
        if (!c.markup && c.models) {
            const tags = c.models.map(model => {
                return `<${model.element}-config id="${model.id}"></${model.element}-config>`;
            });
            c.markup = tags.join("");
        }
    }
    async watchElementsLoaded(newValue, oldValue) {
        if (newValue && !oldValue) {
            await this.updateModels();
        }
    }
    async updateModels() {
        if (this.pieContentModel &&
            this.pieContentModel.elements &&
            this.pieContentModel.markup) {
            if (!this.pieContentModel.models) {
                this.pieContentModel.models = [];
            }
            const tempDiv = this.doc.createElement("div");
            tempDiv.innerHTML = this.pieContentModel.markup;
            const elsWithId = tempDiv.querySelectorAll("[id]");
            // set up a model for each pie defined in the markup
            elsWithId.forEach(el => {
                const pieElName = el.tagName.toLowerCase().split("-config")[0];
                // initialize emtpy model if this is a pie
                if (this.pieContentModel.elements[pieElName]) {
                    const elementId = el.getAttribute("id");
                    if (!this.pieContentModel.models.find(m => m.id === elementId)) {
                        const model = { id: elementId, element: pieElName };
                        this.pieContentModel.models.push(model);
                    }
                }
            });
            tempDiv.remove();
        }
        if (this.pieContentModel && this.pieContentModel.models) {
            this.setModelsAndConfigurations();
            if (this._modelLoadedState === false) {
                this._modelLoadedState = true;
                this.modelLoaded.emit(this.pieContentModel);
            }
        }
    }
    watchConfigSettings() {
        if (this.canWatchConfigSettings) {
            this.setModelsAndConfigurations();
        }
    }
    componentDidUnload() {
        this.el.removeEventListener(lib_3.TYPE, this.handleInsertImage);
        this.el.removeEventListener(lib_2.TYPE, this.handleDeleteImage);
        this.el.removeEventListener(lib_5.TYPE, this.handleInsertSound);
        this.el.removeEventListener(lib_4.TYPE, this.handleDeleteSound);
        if (this.fileInput) {
            this.fileInput.removeEventListener("change", this.handleFileInputChange);
        }
        window.removeEventListener("focus", this.handleWindowFocus);
    }
    async componentWillLoad() {
        if (this.config) {
            this.watchConfig(this.config, {});
        }
        // Note: cannot use the @Listen decorator as creates bundling problems due
        // to `.` in event name.
        this.el.addEventListener(lib_1.TYPE, async (e) => {
            // set the internal model
            // emit a content-item level event with the model
            let rubricChanged;
            if (this.pieContentModel && e.update) {
                this.pieContentModel.models.forEach(m => {
                    if (m.id === e.update.id && m.element === e.update.element) {
                        rubricChanged =
                            rubricChanged || m.rubricEnabled !== e.update.rubricEnabled;
                        Object.assign(m, e.update);
                        if (e.update.errors && !isEmpty_1(e.update.errors)) {
                            this.validateModels();
                        }
                    }
                });
            }
            if (this._modelLoadedState) {
                if (rubricChanged) {
                    if (this.isInsidePieApiAuthor) {
                        // if pie-author is used inside pie-api-author, then what we need is to
                        // calculate the new pieContentModel and make sure we emit the modelUpdated event with the updated pieContentModel
                        // first thing, we check what changed (if we have to add or remove complex-rubric)
                        const complexRubricCheckedValues = complexRubricChecks(this.pieContentModel, this.configSettings);
                        const { shouldAddComplexRubric, shouldRemoveComplexRubric } = complexRubricCheckedValues || {};
                        if (shouldAddComplexRubric || shouldRemoveComplexRubric) {
                            // then, if changes are needed, we make them and save the new contentModel
                            const newPieContentModel = await this.getPieContentModelWithToggledComplexRubric({
                                complexRubricCheckedValues,
                                pieContentModel: this.pieContentModel
                            });
                            // last thing to do is to make sure we send the updated pie content model to pie-api-author
                            // (which listens for this event) who will trigger watchConfig
                            if (newPieContentModel) {
                                this.modelUpdated.emit(newPieContentModel);
                            }
                        }
                    }
                    else {
                        // however, if we only use pie-player-components, just force the call to watchConfig
                        await this.watchConfig(this.config, {});
                    }
                }
                else {
                    // if rubricEnabled did not change, just behave as previously
                    // (pie-api-author listens for this event and triggers watchConfig)
                    this.modelUpdated.emit(this.pieContentModel);
                }
            }
        });
        this.el.addEventListener(lib_3.TYPE, this.handleInsertImage);
        this.el.addEventListener(lib_2.TYPE, this.handleDeleteImage);
        this.el.addEventListener(lib_5.TYPE, this.handleInsertSound);
        this.el.addEventListener(lib_4.TYPE, this.handleDeleteSound);
        window.addEventListener("focus", this.handleWindowFocus);
    }
    async componentDidLoad() {
        this.componentLoaded = true;
        await this.afterRender();
    }
    async componentDidUpdate() {
        await this.afterRender();
        if (this.fileInput) {
            this.fileInput.addEventListener("change", this.handleFileInputChange);
        }
    }
    async loadPieElements() {
        if (this.config && !this.disableBundler) {
            let endpoints = DEFAULT_ENDPOINTS.prod;
            if (this.bundleHost &&
                ["dev", "stage", "prod"].includes(this.bundleHost)) {
                endpoints = DEFAULT_ENDPOINTS[this.bundleHost];
            }
            // if bundleEndpoints are provided, use them instead of the default
            // used for proxies, etc.
            let forceBundleUrl = false;
            if (this.bundleEndpoints) {
                endpoints = this.bundleEndpoints;
                forceBundleUrl = true;
            }
            await this.pieLoader.loadCloudPies({
                content: this.pieContentModel,
                doc: this.doc,
                endpoints,
                useCdn: false,
                forceBundleUrl,
                reFetchBundle: this.reFetchBundle
            });
        }
    }
    renderMath() {
        mathRenderingAccessible$1.renderMath(this.el);
    }
    async afterRender() {
        if (this.pieContentModel &&
            this.pieContentModel.markup &&
            !this.elementsLoaded) {
            const elements = Object.keys(this.pieContentModel.elements).map(el => ({
                name: el,
                tag: `${el}-config`
            }));
            const loadedInfo = await this.pieLoader.elementsHaveLoaded(elements);
            if (loadedInfo.val &&
                !!loadedInfo.elements.find(el => this.pieContentModel.elements[el.name])) {
                this.elementsLoaded = true;
                this.renderMath();
            }
        }
    }
    /**
     * Utility method to add a `@pie-element/rubric` section to an item config when creating an item should be used before setting the config.
     *
     * @deprecated this method is for temporary use, will be removed at next major release
     *
     * @param config the item config to mutate
     * @param rubricModel
     */
    async addRubricToConfig(config, rubricModel) {
        console.warn("If you are using complex-rubric, stop using this function to prevent having duplicated rubrics.");
        if (!rubricModel) {
            rubricModel = {
                id: "rubric",
                element: "pie-rubric",
                points: ["", "", "", ""],
                maxPoints: 4,
                excludeZero: false
            };
        }
        const configPieContent = pieContentFromConfig(config);
        addPackageToContent(configPieContent, "@pie-element/rubric", rubricModel);
        return addRubric(configPieContent);
    }
    /**
     * Utility method to add a `@pie-element/multi-trait-rubric` section to an item config when creating an item should be used before setting the config.
     **
     * @param config the item config to mutate
     * @param multiTraitRubricModel
     */
    async addMultiTraitRubricToConfig(config, multiTraitRubricModel) {
        console.warn("If you are using complex-rubric, stop using this function to prevent having duplicated rubrics.");
        if (!multiTraitRubricModel) {
            multiTraitRubricModel = {
                id: "multi-trait-rubric",
                element: "pie-multi-trait-rubric",
                visibleToStudent: true,
                halfScoring: false,
                excludeZero: true,
                pointLabels: true,
                description: false,
                standards: false,
                scales: [
                    {
                        maxPoints: 4,
                        scorePointsLabels: ["", "", "", ""],
                        traitLabel: "Trait",
                        traits: [
                            {
                                name: "",
                                standards: [],
                                description: "",
                                scorePointsDescriptors: ["", "", "", "", ""]
                            }
                        ]
                    }
                ]
            };
        }
        const configPieContent = pieContentFromConfig(config);
        addPackageToContent(configPieContent, "@pie-element/multi-trait-rubric", multiTraitRubricModel);
        return addMultiTraitRubric(configPieContent);
    }
    render() {
        if (this.pieContentModel && this.pieContentModel.markup) {
            const markup = this.getRenderMarkup();
            if (this.addPreview) {
                return (h("pie-preview-layout", { config: this.config }, h("div", { slot: "configure" }, h("pie-spinner", { active: !this.elementsLoaded }, h("div", { class: "author-container", innerHTML: markup }))), h("input", { type: "file", hidden: true, ref: r => (this.fileInput = r), style: { display: "none" } })));
            }
            else {
                return (h("pie-spinner", { active: !this.elementsLoaded }, h("div", { class: "author-container", innerHTML: markup }), h("input", { type: "file", hidden: true, ref: r => (this.fileInput = r), style: { display: "none" } })));
            }
        }
        else {
            return h("pie-spinner", null);
        }
    }
    get el() { return getElement(this); }
    static get watchers() { return {
        "addPreview": ["watchAddPreview"],
        "config": ["watchConfig"],
        "elementsLoaded": ["watchElementsLoaded"],
        "configSettings": ["watchConfigSettings"]
    }; }
    static get style() { return ".pie-loading {\n  position: absolute;\n  height: 100%;\n  width: 100%;\n  background-color: #fff;\n  bottom: 0;\n  left: 0;\n  right: 0;\n  top: 0;\n  z-index: 9999;\n  opacity: 0.4;\n}\n\n.pie-loading.fixed {\n  position: fixed;\n}\n\n.pie-loading:before {\n  content: \"\";\n  background-color: rgba(0, 0, 0, 0);\n  border: 5px solid rgba(63, 81, 181, 0.9);\n  opacity: 0.9;\n  border-right: 5px solid rgba(0, 0, 0, 0);\n  border-left: 5px solid rgba(0, 0, 0, 0);\n  border-radius: 50px;\n  -webkit-box-shadow: 0 0 35px rgba(63, 81, 181, 0.9);\n  box-shadow: 0 0 35px rgba(63, 81, 181, 0.9);\n  width: 50px;\n  height: 50px;\n  -moz-animation: spinPulse 1s infinite ease-in-out;\n  -webkit-animation: spinPulse 1s infinite linear;\n  margin: -25px 0 0 -25px;\n  position: absolute;\n  top: 50%;\n  left: 50%;\n}\n\n.pie-loading:after {\n  content: \"\";\n  background-color: rgba(0, 0, 0, 0);\n  border: 5px solid rgba(63, 81, 181, 0.9);\n  opacity: 0.9;\n  border-left: 5px solid rgba(0, 0, 0, 0);\n  border-right: 5px solid rgba(0, 0, 0, 0);\n  border-radius: 50px;\n  -webkit-box-shadow: 0 0 15px rgba(63, 81, 181, 0.9);\n  box-shadow: 0 0 15px rgba(63, 81, 181, 0.9);\n  width: 30px;\n  height: 30px;\n  -moz-animation: spinoffPulse 1s infinite linear;\n  -webkit-animation: spinoffPulse 1s infinite linear;\n  margin: -15px 0 0 -15px;\n  position: absolute;\n  top: 50%;\n  left: 50%;\n}\n\n\@-webkit-keyframes spinPulse {\n  0% {\n    -webkit-transform: rotate(160deg);\n    transform: rotate(160deg);\n    opacity: 0;\n    -webkit-box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n    box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n  }\n  50% {\n    -webkit-transform: rotate(145deg);\n    transform: rotate(145deg);\n    opacity: 1;\n  }\n  100% {\n    -webkit-transform: rotate(-320deg);\n    transform: rotate(-320deg);\n    opacity: 0;\n  }\n}\n\n\@keyframes spinPulse {\n  0% {\n    -webkit-transform: rotate(160deg);\n    transform: rotate(160deg);\n    opacity: 0;\n    -webkit-box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n    box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n  }\n  50% {\n    -webkit-transform: rotate(145deg);\n    transform: rotate(145deg);\n    opacity: 1;\n  }\n  100% {\n    -webkit-transform: rotate(-320deg);\n    transform: rotate(-320deg);\n    opacity: 0;\n  }\n}\n\n\@-webkit-keyframes spinoffPulse {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(360deg);\n    transform: rotate(360deg);\n  }\n}\n\n\@keyframes spinoffPulse {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(360deg);\n    transform: rotate(360deg);\n  }\n}\n\n.pie-api-player img {\n  max-width: inherit !important;\n  max-height: inherit !important;\n}\n\n/* These styles are used in the body content of an item -- they also appear in the item body editor. */\n\n.book-title-k5 {\n  text-decoration: underline;\n}\n\n.book-title {\n  font-style: italic;\n}\n\n.block-quote {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.text-block {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.short-quote {\n  font-weight: bold;\n}\n\nspan.short-quote:before {\n  content: open-quote;\n}\n\nspan.short-quote:after {\n  content: close-quote;\n}\n\n.word-callout {\n  text-decoration: underline;\n}\n\n.relative-emphasis {\n  text-transform: uppercase;\n}\n\n.content-emphasis {\n  font-weight: bold;\n  color: red;\n}\n\nspan.passage-title:before {\n  content: open-quote;\n}\n\nspan.passage-title:after {\n  content: close-quote;\n}\n\n.proper-name {\n  font-style: italic;\n}\n\n.variable {\n  font-style: italic;\n}\n\n.equation-block {\n  text-align: center;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  padding: 2px;\n  display: block;\n}\n\n.embedded-error {\n  font-style: italic;\n}\n\n/* PASSAGES */\n\ndiv.passage-title,\ndiv.passage-subtitle {\n  font-weight: bold;\n  text-align: center;\n}\n\ndiv.passage-author {\n  text-align: center;\n}\n\n.p-number {\n  float: left;\n  font-size: 0.85em;\n}\n\n.numbered-paragraph {\n  margin-left: 36px;\n}\n\n/* ch5680 - also temporary kds content styles that need to be externalized */\n/* Inspect Styles */\n\n\@media print {\n  .kds-noprint {\n    display: none;\n  }\n\n  .noprint {\n    display: none;\n  }\n}\n\n.kds-absolute {\n  border-left: 1px solid black;\n  border-right: 1px solid black;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.abs {\n  border-left: #000000 1px solid;\n  border-right: #000000 1px solid;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.kds-border-1 {\n  border: 1px solid black;\n}\n\n.kds-border-2 {\n  border: 2px solid black;\n}\n\n.kds-center {\n  margin: 0px;\n  text-align: center;\n}\n\n.center {\n  text-align: center;\n  margin: 0;\n}\n\n.kds-exponent {\n  font-size: 85%;\n  margin-left: -0.4em;\n  position: relative;\n  top: -1.4em;\n}\n\nsup.frac {\n  position: relative;\n  top: -1.4em;\n  font-size: 70%;\n  margin-left: -0.4em;\n}\n\n.kds-fillin {\n  border: 1px solid black;\n  display: inline-block;\n  margin: 0em 0.2em;\n  padding-left: 1em;\n}\n\n.fillin {\n  margin: 0em 0.2em;\n  padding-left: 1em;\n  border: #000000 1px solid;\n  display: inline-block;\n  *display: inline;\n}\n\n.kds-font-larger {\n  font-size: larger;\n}\n\n.kds-font-smaller {\n  font-size: smaller;\n}\n\ntable.kds-fraction {\n  border: 0px;\n  display: inline-block;\n  font-size: 85%;\n  margin: 0px;\n  padding: 0px;\n  vertical-align: middle;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-numerator {\n  border-bottom: 1px solid black;\n  border-left: 0px;\n  border-right: 0px;\n  border-top: 0px;\n  text-align: center;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-denominator {\n  border: 0px;\n  text-align: center;\n}\n\ntable.frac {\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n  font-size: 85%;\n  border: 0px;\n  margin: 0px;\n  padding: 0px;\n}\n\n.frac .nu {\n  border-left: 0px;\n  border-top: 0px;\n  border-right: 0px;\n  border-bottom: #000000 1px solid;\n  text-align: center;\n}\n\n.frac .de {\n  border: 0px;\n  text-align: center;\n}\n\n.kds-hanging-indent {\n  margin: 0px;\n  padding-left: 3em;\n  text-indent: -3em;\n}\n\n.kds-image-caption {\n  font-family: Verdana, Arial, sans-serif;\n  text-align: center;\n}\n\n.kds-image-left {\n  float: left;\n  margin-right: 5px;\n}\n\n.kds-image-right {\n  float: right;\n  margin-left: 5px;\n}\n\n.kds-indent {\n  text-indent: 3em;\n}\n\n.indent {\n  text-indent: 3em;\n}\n\n.kds-nowrap {\n  display: inline-block;\n  white-space: nowrap;\n}\n\n.kds-overline {\n  text-decoration: overline;\n}\n\n.kds-parentheses {\n  font-family: \"Arial Narrow\", Arial, sans-serif;\n  font-size: 300%;\n  font-weight: normal;\n  position: relative;\n  top: 8px;\n}\n\nb.frac {\n  font-weight: normal;\n  font-size: 300%;\n  font-family: arial narrow;\n  position: relative;\n  top: +8px;\n}\n\n.kds-pi {\n  font-family: \"Times New Roman\", serif;\n  font-size: 1.2em;\n}\n\n.newradical {\n  border-collapse: collapse;\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n}\n\n.newradical td {\n  border: 0px;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical td.vinculum {\n  border-top: 1px solid black;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical div.exp {\n  font-size: 0.7em;\n  position: relative;\n  left: 7px;\n  bottom: 5px;\n}\n\n.kds-underline {\n  text-decoration: underline;\n}\n\n.under {\n  text-decoration: underline;\n}\n\n.kds-verdana2t {\n  border: 1px solid white;\n  text-align: left;\n  text-decoration: none;\n  vertical-align: top;\n}\n\n.Verdana2t {\n  vertical-align: top;\n  text-align: left;\n  text-decoration: none;\n  border: 1px solid white;\n}\n\n.kds-whole-number {\n  display: inline-block;\n  font-size: 120%;\n  margin: 0em 0em 0em 0.2em;\n}\n\n.whole {\n  display: inline-block;\n  *display: inline;\n  margin: 0em 0em 0em 0.2em;\n  font-size: 120%;\n}\n\ntable.KdsTable01 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable01 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > th.bold {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td.bold {\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable02 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable02 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n}\n\ntable.KdsTable02 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n}\n\n.table {\n  width: 100%;\n  margin-bottom: 1em;\n  background-color: transparent;\n  border-collapse: collapse;\n}\n\n.table th, .table td {\n  padding: .75em;\n  vertical-align: top;\n  border-top: 1px solid #dee2e6;\n}\n\n.table tbody + tbody {\n  border-top: 2px solid #dee2e6;\n}\n\n.table thead th {\n  vertical-align: bottom;\n  border-bottom: 2px solid #dee2e6;\n}\n\nth {\n  text-align: inherit;\n}\n\n.table-bordered, .table-bordered th, .table-bordered td {\n  border: 1px solid #dee2e6;\n}\n\n.table-bordered thead td, .table-bordered thead th {\n  border-bottom-width: 2px;\n}\n\n.table-striped tbody tr:nth-of-type(odd) {\n  background-color: rgba(0, 0, 0, .05);\n}\n\n.text-center {\n  text-align: center !important;\n}\n\n.h5, h5 {\n  font-size: 1.25em;\n}\n\nh1, h2, h3, h4, h5, h6 {\n  margin-top: 0;\n}\n\n.h1, .h2, .h3, .h4, .h5, .h6, h1, h2, h3, h4, h5, h6 {\n  margin-bottom: .5em;\n  font-family: inherit;\n  font-weight: 500;\n  line-height: 1.2;\n  color: inherit;\n}\n\n/* LEARNOSITY OVERRIDES */\n.lrn_feature h3 {\n  margin-top: 0 !important;\n}\n\n/* PIE OVERRIDES */\n\n#stimulus {\n  width: 50%;\n  float: left;\n}\n\n#item {\n  width: 50%;\n  float: left;\n}\n\n.lrn_width_auto.table {\n  width: auto;\n}\n\n.TEX-S3 {\n  font-family: MJXZERO, MJXTEX-S3 !important;\n}\n\n.TEX-S2 {\n  font-family: MJXZERO, MJXTEX-S2 !important;\n}\n\n.TEX-S1 {\n  font-family: MJXZERO, MJXTEX-S1 !important;\n}\n\n.TEX-I {\n  font-family: MJXZERO, MJXTEX-I !important;\n}\n\nmjx-c.mjx-c2032 {\n  font-family: \'CerebriSans\', serif !important;\n}\n\nmjx-c.mjx-c22, mjx-c.mjx-c27 {\n  font-family: MJXZERO, serif !important\n}\n\ntable {\n  border-collapse: collapse;\n}\n\n.evaluate-bottom-border > :first-child::after {\n  content: \'\';\n  display: block;\n  border-bottom: 1px solid #d3d3d3;\n}"; }
};

export { Author as pie_author };
