import { r as registerInstance, d as getContext, h, e as getElement } from './core-43f43fa0.js';

const definitions = new Map();
/**
 * Make sure only 1 define call is made for an element.
 * @param name
 * @param def
 * @returns
 */
const define = (name, def) => {
    console.log("define:", name);
    const existing = definitions.get(name);
    console.log("existing:", existing);
    if (existing) {
        if (existing.ready) {
            return;
        }
        if (existing.inProgress) {
            return;
        }
        if (existing.error) {
            throw existing.error;
        }
    }
    definitions.set(name, { inProgress: true });
    try {
        customElements.define(name, def);
    }
    catch (e) {
        /**
         * It can be the case that different tags will use the same CustomElement.
         * We don't want to process all the markup so we wrap the definition in an anonymous class.
         */
        if (e && e.code === DOMException.NOT_SUPPORTED_ERR) {
            try {
                if (customElements.get(name)) {
                    return;
                }
                customElements.define(name, class extends def {
                });
            }
            catch (e) {
                console.error("wrapped class failed");
                definitions.set(name, { inProgress: false, error: e });
            }
        }
        else {
            definitions.set(name, { inProgress: false, error: e });
        }
    }
    customElements
        .whenDefined(name)
        .then(() => {
        definitions.set(name, { inProgress: false, ready: true });
    })
        .catch(e => {
        definitions.set(name, { inProgress: false, error: e });
    });
};
const status = (name) => {
    const existing = definitions.get(name);
    if (existing) {
        if (existing.inProgress) {
            return "inProgress";
        }
        if (existing.ready) {
            return "inRegistry";
        }
        if (existing.error) {
            return "error";
        }
    }
    return "none";
};
const whenDefined = (name) => {
    return customElements.whenDefined(name);
};

const prepareUrl = (u) => {
    if (u.startsWith("http")) {
        return u;
    }
    if (u.startsWith("/")) {
        return u;
    }
    return `https://cdn.jsdelivr.net/npm/${u}`;
};
function loadScript(src) {
    return new Promise(function (resolve, reject) {
        let s;
        s = document.createElement('script');
        s.type = 'module';
        s.src = src;
        s.onload = resolve;
        s.onerror = reject;
        document.head.appendChild(s);
    });
}
const load = async (tag, url) => {
    const def = customElements.get(tag);
    if (def) {
        console.log("already defined - skip load");
        return whenDefined(tag);
    }
    const fullUrl = prepareUrl(url);
    try {
        await loadScript(fullUrl);
    }
    catch (e) {
        console.error(e.toString());
    }
    return import(fullUrl)
        .then(mod => {
        const Def = mod.definition();
        try {
            define(tag, Def);
        }
        catch (e) {
            return whenDefined(tag);
        }
    })
        .catch(e => {
        console.error("failed to load: ", url);
    });
};

const Embed = class {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        this.doc = getContext(this, "document");
    }
    async componentWillLoad() {
        const dataset = this.el.dataset || {};
        const { tag, url } = dataset;
        if (!url) {
            return;
        }
        if (!tag) {
            return;
        }
        this.tag = tag;
        const names = this.el.getAttributeNames();
        this.params = Object.entries(names).reduce((acc, [index, n]) => {
            if (n !== "data-tag" && n !== "data-url") {
                acc[n] = this.el.getAttribute(n);
            }
            return acc;
        }, {});
        await load(tag, url);
    }
    render() {
        if (this.tag) {
            const Tag = this.tag;
            return h(Tag, Object.assign({}, this.params));
        }
    }
    get el() { return getElement(this); }
    static get style() { return ".pie-loading {\n  position: absolute;\n  height: 100%;\n  width: 100%;\n  background-color: #fff;\n  bottom: 0;\n  left: 0;\n  right: 0;\n  top: 0;\n  z-index: 9999;\n  opacity: 0.4;\n}\n\n.pie-loading.fixed {\n  position: fixed;\n}\n\n.pie-loading:before {\n  content: \"\";\n  background-color: rgba(0, 0, 0, 0);\n  border: 5px solid rgba(63, 81, 181, 0.9);\n  opacity: 0.9;\n  border-right: 5px solid rgba(0, 0, 0, 0);\n  border-left: 5px solid rgba(0, 0, 0, 0);\n  border-radius: 50px;\n  -webkit-box-shadow: 0 0 35px rgba(63, 81, 181, 0.9);\n  box-shadow: 0 0 35px rgba(63, 81, 181, 0.9);\n  width: 50px;\n  height: 50px;\n  -moz-animation: spinPulse 1s infinite ease-in-out;\n  -webkit-animation: spinPulse 1s infinite linear;\n  margin: -25px 0 0 -25px;\n  position: absolute;\n  top: 50%;\n  left: 50%;\n}\n\n.pie-loading:after {\n  content: \"\";\n  background-color: rgba(0, 0, 0, 0);\n  border: 5px solid rgba(63, 81, 181, 0.9);\n  opacity: 0.9;\n  border-left: 5px solid rgba(0, 0, 0, 0);\n  border-right: 5px solid rgba(0, 0, 0, 0);\n  border-radius: 50px;\n  -webkit-box-shadow: 0 0 15px rgba(63, 81, 181, 0.9);\n  box-shadow: 0 0 15px rgba(63, 81, 181, 0.9);\n  width: 30px;\n  height: 30px;\n  -moz-animation: spinoffPulse 1s infinite linear;\n  -webkit-animation: spinoffPulse 1s infinite linear;\n  margin: -15px 0 0 -15px;\n  position: absolute;\n  top: 50%;\n  left: 50%;\n}\n\n\@-webkit-keyframes spinPulse {\n  0% {\n    -webkit-transform: rotate(160deg);\n    transform: rotate(160deg);\n    opacity: 0;\n    -webkit-box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n    box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n  }\n  50% {\n    -webkit-transform: rotate(145deg);\n    transform: rotate(145deg);\n    opacity: 1;\n  }\n  100% {\n    -webkit-transform: rotate(-320deg);\n    transform: rotate(-320deg);\n    opacity: 0;\n  }\n}\n\n\@keyframes spinPulse {\n  0% {\n    -webkit-transform: rotate(160deg);\n    transform: rotate(160deg);\n    opacity: 0;\n    -webkit-box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n    box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n  }\n  50% {\n    -webkit-transform: rotate(145deg);\n    transform: rotate(145deg);\n    opacity: 1;\n  }\n  100% {\n    -webkit-transform: rotate(-320deg);\n    transform: rotate(-320deg);\n    opacity: 0;\n  }\n}\n\n\@-webkit-keyframes spinoffPulse {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(360deg);\n    transform: rotate(360deg);\n  }\n}\n\n\@keyframes spinoffPulse {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(360deg);\n    transform: rotate(360deg);\n  }\n}\n\n.pie-api-player img {\n  max-width: inherit !important;\n  max-height: inherit !important;\n}\n\n/* These styles are used in the body content of an item -- they also appear in the item body editor. */\n\n.book-title-k5 {\n  text-decoration: underline;\n}\n\n.book-title {\n  font-style: italic;\n}\n\n.block-quote {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.text-block {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.short-quote {\n  font-weight: bold;\n}\n\nspan.short-quote:before {\n  content: open-quote;\n}\n\nspan.short-quote:after {\n  content: close-quote;\n}\n\n.word-callout {\n  text-decoration: underline;\n}\n\n.relative-emphasis {\n  text-transform: uppercase;\n}\n\n.content-emphasis {\n  font-weight: bold;\n  color: red;\n}\n\nspan.passage-title:before {\n  content: open-quote;\n}\n\nspan.passage-title:after {\n  content: close-quote;\n}\n\n.proper-name {\n  font-style: italic;\n}\n\n.variable {\n  font-style: italic;\n}\n\n.equation-block {\n  text-align: center;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  padding: 2px;\n  display: block;\n}\n\n.embedded-error {\n  font-style: italic;\n}\n\n/* PASSAGES */\n\ndiv.passage-title,\ndiv.passage-subtitle {\n  font-weight: bold;\n  text-align: center;\n}\n\ndiv.passage-author {\n  text-align: center;\n}\n\n.p-number {\n  float: left;\n  font-size: 0.85em;\n}\n\n.numbered-paragraph {\n  margin-left: 36px;\n}\n\n/* ch5680 - also temporary kds content styles that need to be externalized */\n/* Inspect Styles */\n\n\@media print {\n  .kds-noprint {\n    display: none;\n  }\n\n  .noprint {\n    display: none;\n  }\n}\n\n.kds-absolute {\n  border-left: 1px solid black;\n  border-right: 1px solid black;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.abs {\n  border-left: #000000 1px solid;\n  border-right: #000000 1px solid;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.kds-border-1 {\n  border: 1px solid black;\n}\n\n.kds-border-2 {\n  border: 2px solid black;\n}\n\n.kds-center {\n  margin: 0px;\n  text-align: center;\n}\n\n.center {\n  text-align: center;\n  margin: 0;\n}\n\n.kds-exponent {\n  font-size: 85%;\n  margin-left: -0.4em;\n  position: relative;\n  top: -1.4em;\n}\n\nsup.frac {\n  position: relative;\n  top: -1.4em;\n  font-size: 70%;\n  margin-left: -0.4em;\n}\n\n.kds-fillin {\n  border: 1px solid black;\n  display: inline-block;\n  margin: 0em 0.2em;\n  padding-left: 1em;\n}\n\n.fillin {\n  margin: 0em 0.2em;\n  padding-left: 1em;\n  border: #000000 1px solid;\n  display: inline-block;\n  *display: inline;\n}\n\n.kds-font-larger {\n  font-size: larger;\n}\n\n.kds-font-smaller {\n  font-size: smaller;\n}\n\ntable.kds-fraction {\n  border: 0px;\n  display: inline-block;\n  font-size: 85%;\n  margin: 0px;\n  padding: 0px;\n  vertical-align: middle;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-numerator {\n  border-bottom: 1px solid black;\n  border-left: 0px;\n  border-right: 0px;\n  border-top: 0px;\n  text-align: center;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-denominator {\n  border: 0px;\n  text-align: center;\n}\n\ntable.frac {\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n  font-size: 85%;\n  border: 0px;\n  margin: 0px;\n  padding: 0px;\n}\n\n.frac .nu {\n  border-left: 0px;\n  border-top: 0px;\n  border-right: 0px;\n  border-bottom: #000000 1px solid;\n  text-align: center;\n}\n\n.frac .de {\n  border: 0px;\n  text-align: center;\n}\n\n.kds-hanging-indent {\n  margin: 0px;\n  padding-left: 3em;\n  text-indent: -3em;\n}\n\n.kds-image-caption {\n  font-family: Verdana, Arial, sans-serif;\n  text-align: center;\n}\n\n.kds-image-left {\n  float: left;\n  margin-right: 5px;\n}\n\n.kds-image-right {\n  float: right;\n  margin-left: 5px;\n}\n\n.kds-indent {\n  text-indent: 3em;\n}\n\n.indent {\n  text-indent: 3em;\n}\n\n.kds-nowrap {\n  display: inline-block;\n  white-space: nowrap;\n}\n\n.kds-overline {\n  text-decoration: overline;\n}\n\n.kds-parentheses {\n  font-family: \"Arial Narrow\", Arial, sans-serif;\n  font-size: 300%;\n  font-weight: normal;\n  position: relative;\n  top: 8px;\n}\n\nb.frac {\n  font-weight: normal;\n  font-size: 300%;\n  font-family: arial narrow;\n  position: relative;\n  top: +8px;\n}\n\n.kds-pi {\n  font-family: \"Times New Roman\", serif;\n  font-size: 1.2em;\n}\n\n.newradical {\n  border-collapse: collapse;\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n}\n\n.newradical td {\n  border: 0px;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical td.vinculum {\n  border-top: 1px solid black;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical div.exp {\n  font-size: 0.7em;\n  position: relative;\n  left: 7px;\n  bottom: 5px;\n}\n\n.kds-underline {\n  text-decoration: underline;\n}\n\n.under {\n  text-decoration: underline;\n}\n\n.kds-verdana2t {\n  border: 1px solid white;\n  text-align: left;\n  text-decoration: none;\n  vertical-align: top;\n}\n\n.Verdana2t {\n  vertical-align: top;\n  text-align: left;\n  text-decoration: none;\n  border: 1px solid white;\n}\n\n.kds-whole-number {\n  display: inline-block;\n  font-size: 120%;\n  margin: 0em 0em 0em 0.2em;\n}\n\n.whole {\n  display: inline-block;\n  *display: inline;\n  margin: 0em 0em 0em 0.2em;\n  font-size: 120%;\n}\n\ntable.KdsTable01 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable01 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > th.bold {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td.bold {\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable02 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable02 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n}\n\ntable.KdsTable02 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n}\n\n.table {\n  width: 100%;\n  margin-bottom: 1em;\n  background-color: transparent;\n  border-collapse: collapse;\n}\n\n.table th, .table td {\n  padding: .75em;\n  vertical-align: top;\n  border-top: 1px solid #dee2e6;\n}\n\n.table tbody + tbody {\n  border-top: 2px solid #dee2e6;\n}\n\n.table thead th {\n  vertical-align: bottom;\n  border-bottom: 2px solid #dee2e6;\n}\n\nth {\n  text-align: inherit;\n}\n\n.table-bordered, .table-bordered th, .table-bordered td {\n  border: 1px solid #dee2e6;\n}\n\n.table-bordered thead td, .table-bordered thead th {\n  border-bottom-width: 2px;\n}\n\n.table-striped tbody tr:nth-of-type(odd) {\n  background-color: rgba(0, 0, 0, .05);\n}\n\n.text-center {\n  text-align: center !important;\n}\n\n.h5, h5 {\n  font-size: 1.25em;\n}\n\nh1, h2, h3, h4, h5, h6 {\n  margin-top: 0;\n}\n\n.h1, .h2, .h3, .h4, .h5, .h6, h1, h2, h3, h4, h5, h6 {\n  margin-bottom: .5em;\n  font-family: inherit;\n  font-weight: 500;\n  line-height: 1.2;\n  color: inherit;\n}\n\n/* LEARNOSITY OVERRIDES */\n.lrn_feature h3 {\n  margin-top: 0 !important;\n}\n\n/* PIE OVERRIDES */\n\n#stimulus {\n  width: 50%;\n  float: left;\n}\n\n#item {\n  width: 50%;\n  float: left;\n}\n\n.lrn_width_auto.table {\n  width: auto;\n}\n\n.TEX-S3 {\n  font-family: MJXZERO, MJXTEX-S3 !important;\n}\n\n.TEX-S2 {\n  font-family: MJXZERO, MJXTEX-S2 !important;\n}\n\n.TEX-S1 {\n  font-family: MJXZERO, MJXTEX-S1 !important;\n}\n\n.TEX-I {\n  font-family: MJXZERO, MJXTEX-I !important;\n}\n\nmjx-c.mjx-c2032 {\n  font-family: \'CerebriSans\', serif !important;\n}\n\nmjx-c.mjx-c22, mjx-c.mjx-c27 {\n  font-family: MJXZERO, serif !important\n}\n\ntable {\n  border-collapse: collapse;\n}\n\n.evaluate-bottom-border > :first-child::after {\n  content: \'\';\n  display: block;\n  border-bottom: 1px solid #d3d3d3;\n}"; }
};

export { Embed as pie_embed };
