import { EventEmitter } from "../../stencil.core";
import "@pie-framework/pie-player-components";
import { PieContent, ItemConfig } from "@pie-framework/pie-player-components/dist/types/interface";
import { PreReleaseType } from "../../interface";
import { PieApiImageSupport } from "../../clients/image-support";
import { PieApiSoundSupport } from "../../clients/sound-support";
import { HostEnv, ApiEndpoints, BundleEndpoints } from "../../types";
import { NewRelicInstrumentationService } from "../../clients/new-relic";
export declare class PieApiAuthor {
    newRelic: NewRelicInstrumentationService;
    pieConfigEl: HTMLPieAuthorElement;
    doc: Document;
    el: HTMLElement;
    versions: {
        [name: string]: string;
    };
    /**
     * Provide custom endpoints for the player components. Can be proxy or direct.
     */
    bundleEndpoints?: BundleEndpoints;
    /**
     * Choose 'dev' , 'stage' or 'prod' environments for back end.
     */
    host?: HostEnv;
    /**
     * used to automatically re-fetch the bundle (in case we get a 503); defaults to false
     */
    reFetchBundle?: boolean;
    watchHostEnvironment(newVal: any): Promise<void>;
    /**
     * For development purposes only - allows alternate api endpoints to be provided.
     */
    apiConfig: ApiEndpoints;
    /**
     * Content has been loaded from server
     */
    contentLoaded: EventEmitter;
    /**
     * When the `contentId` property is set, it will be parsed to ensure it is a valid identifier and to determine
     * if the version is released or pre-release.
     *
     * the `event.detail` value will be `{valid: true | false, status: 'released' | 'pre-release'}`
     */
    contentIdValidated: EventEmitter;
    /**
     * If `recordScreen` property is set to true this event will be
     * emitted with the url for recorded screen session in `detail` property.
     */
    recordingStarted: EventEmitter;
    /**
     * The api token for `pie-api` access.
     */
    token: string;
    /**
     * Adds a preview view which will render the content in another tab as it may appear to a student or instructor.
     */
    addPreview: boolean;
    /**
     * The model for the content.
     * If set, the view will update to render the provided config.
     */
    contentConfig: PieContent;
    /**
     * Enable screen recording with logrocket.
     * Note: not to be used in production unless for temporary use to track down a specific user problem.
     */
    recordScreen: boolean;
    /**
     * If screen recording enabled for debug purposes, use this property to provide a unique identifer for
     * the user whose session is being recorded.
     */
    recordScreenIdentity: string;
    /**
     * Enable exception tracking.
     */
    exceptions: boolean;
    /**
     * To customize the standard behaviour provided by interaction configuration views you can
     * provide settings key-ed by the package name.  e.g.
     *
     * `{ '@pie-element/inline-choice': { promptLabel: 'Item Stem' } }`
     *
     * The settings that are configurable for each authoring view are documented in
     * the `@package-name/docs` folder for each package.
     */
    configSettings?: {
        [packageName: string]: Object;
    };
    /**
     * Optional. Set a default model for complex-rubric item (works with pie-player-components versions above v1.2.0).
     */
    defaultComplexRubricModel?: Object;
    /**
     * The unique versioned identifier `id@semver-version` for the Pie Content Item stored in the PIE Service CMS.
     *
     */
    contentId?: string;
    /**
     * Optional. The unique identifier for the Collection in which the content item will be stored in the PIE Service CMS.
     *
     */
    collectionId?: string;
    /**
     * Override element versions.
     * Requires the 'overrides' scope in the token in pie-api
     * matches the key against the element names and if matching uses the overridden version
     * if a string it's parsed as a json string
     */
    overrides: string | {
        [key: string]: string;
    };
    /**
     * If true, the `@version` tag for elements in the configuration will be ignored and a static version for that
     * interaction type determined by the current release of this pie-api-author component will be used.
     */
    lockVersions?: boolean;
    lockInProgress: boolean;
    onModelUpdated: (e: CustomEvent) => void;
    /**
     * Utility method to add a `@pie-element/rubric` section to an item config when creating an item should be used before setting the config.
     *
     * @deprecated this method is for temporary use, will be removed at next major release
     *
     * @param config the item config to mutate
     * @param rubricModel optional model to pass to `@pie-element/rubric`
     */
    addRubricToConfig(config: ItemConfig, rubricModel?: any): Promise<ItemConfig>;
    getOverrides(): {
        [key: string]: string;
    } | undefined;
    /**
     * Loads a content item from `pie-api` host into the view.
     *
     * @param contentId the unique versioned identifer `id@version` for the content item to load. If this param
     * is not provided and `content-id` attribute has a value, that value will be used to load the content item.
     */
    load(contentId?: any): Promise<void>;
    /**
     * Creates a new content item from the currently loaded contents.
     *
     * @param preReleaseType - see preRelaseType for the `save` method
     */
    create(preReleaseType?: string): Promise<string>;
    /**
     * Saves an update to an existing pre-released content item.
     * The content item version must be in pre-release state.
     *
     * the `preReleaseType` param determines what kind of version update, if any, will be performed.
     *
     * `null` - (default) writes over existing pre-release content, , e.g.`1.0.0-draft.0 -> 1.0.0-draft.0`\n
     * `prerelease` - writes in increment to the prerelease version, e.g. `1.0.0-draft.0 -> 1.0.0-draft.1`\n
     * `premajor` - creates a new major release from version, e.g. `1.0.0-draft.1 -> 2.0.0-draft.0`\n
     * `preminor` - creates a new minor release from version, e.g. `1.0.0-draft.1 -> 1.1.0-draft.0`\n
     * `prepatch` - creates a new patch release from version, e.g. `1.0.0-draft.1 -> 1.0.1-draft.0`\m
     *
     * @param preReleaseType - the type of update to perform
     */
    save(preReleaseType?: PreReleaseType): Promise<string>;
    version(): Promise<string>;
    /**
     * this method is for having the possibility to access validateModels from player <br/><br/> It returns the model of the item and errors field which contains the errors returned by validate function in controller. It is an asynchronous method, so it needs to be called with await
     *
     */
    validate(): Promise<any>;
    /**
     * Release a pre-release version of the content currently loaded in the authoring view.
     *
     * Throws an exception if the version of the current Content Item does not have a pre-release tag e.g.`1.0.1-draft`
     *
     *  example:
     * (`contentId = 'abc@1.0.1-draft`)
     *
     *  `this.release() => 'abc@1.0.1`'
     *
     */
    release(): Promise<string>;
    /**
     * TODO
     * Gets a list of all versions that exist for this Content Item.
     */
    getVersions(): Promise<string[]>;
    onTokenChange(): Promise<void>;
    watchContentId(newVal: any): Promise<void>;
    private shouldLock;
    watchRecordScreen(newVal: any, oldVal: any): void;
    generateUUID: () => string;
    getImageSupport(): PieApiImageSupport;
    getUploadSoundSupport(): PieApiSoundSupport;
    componentWillLoad(): Promise<void>;
    connectedCallback(): void;
    componentDidLoad(): void;
    getConfigParsed(sanctioned: any): {
        [packageName: string]: Object;
    };
    render(): any;
}
